/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCLightMapDefinition.h
// Interface definition of a lightmap definition object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	23. Aug. 2005
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CILIGHTMAPDEFINITION_H_INCLUDED)
#define PNPTC_CILIGHTMAPDEFINITION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"

// includes
#include "cipnptclightmaplightsource.h"

#pragma pack(push)
#pragma pack(1)



/// @brief Lightmap definition class.
///
/// This class describes the parameters of a lightmap.
class PNPTCPLUGIN_CALLBACK CIPnPTCLightMapDefinition01
{
public:

	/// @brief Returns the name of the lightmap.
	///
	/// Every lightmap has its unique name within a terrain
	/// project. This name is returned by this method.
	/// @return Name of the lightmap definition.
	virtual const char*						GetName() const=0;

	/// @brief Returns the number of light sources.
	///
	/// Returns the number of light sources used within
	/// this lightmap definition.
	/// @return Number of light sources.
	virtual unsigned int					GetNumLightSources() const=0;

	/// @brief Returns one light source.
	///
	/// Returns one light source definition used by
	/// this lightmap definition.
	/// @param nLightSource Index of the light source to be returned
	/// within the range [0..GetNumLightSources()-1].
	/// @return Returns a pointer to the light source definition if
	/// successfull or <i>NULL</i> in case of an error.
	virtual CIPnPTCLightMapLightSource01*	GetLightSource(unsigned int nLightSource)=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_CILIGHTMAPDEFINITION_H_INCLUDED)

