/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCLightmMapLightSource.h
// Interface definition of a light source definition object.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	23. Aug. 2005
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CILIGHTMAPLIGHTSOURCE_H_INCLUDED)
#define PNPTC_CILIGHTMAPLIGHTSOURCE_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)



/// @brief Lightmap light source definition class.
///
/// This class describes the parameters of one light source
/// used within a lightmap.
class PNPTCPLUGIN_CALLBACK CIPnPTCLightMapLightSource01
{
public:

	/// @brief Returns the name of the light source.
	///
	/// The name of this light souece is returned by this method.
	/// @return Name of the light source.
	virtual const char*				GetName() const=0;

	/// @brief Returns the diffuse light color.
	///
	/// Returns the diffuse color of the light source.
	/// @return Diffuse color.
	virtual	PNPTC_RGB				GetDiffuseColor() const=0;

	/// @brief Returns the ambient light color.
	///
	/// Returns the ambient color of the light source.
	/// @return Ambient color.
	virtual PNPTC_RGB				GetAmbientColor() const=0;

	/// @brief Returns the right ascension angle.
	/// 
	/// Returns the the right ascension angle of the light
	/// source which corresponds to the east/west directional
	/// component of the light rays.
	/// @return Right ascension angle in radians.
	virtual float					GetRightAscension() const=0;

	/// @brief Returns the declination angle.
	/// 
	/// Returns the the declination angle of the light
	/// source which corresponds to the north/south directional
	/// component of the light rays.
	/// @return Declination angle in radians.
	virtual float					GetDeclination() const=0;

}; // class

#pragma pack(pop)


#endif // !defined(PNPTC_CILIGHTMAPLIGHTSOURCE_H_INCLUDED)

