/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSectorWater.h
// Interface definition of an object containing the water settings of a sector.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. Sep. 2006
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISECTORWATER_H_INCLUDED)
#define PNPTC_CISECTORWATER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcwaterdefinition.h"
#include "./cipnptcwaterlevel.h"

/// @brief Contains the water settings for a sector.
///
/// An instance of this interface is passed to your
/// plugin to give you information about the water
/// settings of a sector.
class PNPTCPLUGIN_CALLBACK CIPnPTCSectorWater01 : public CIPnPTCWaterLevel01
{
public:

	/// @brief Returns whether or not the sector uses its own water settings.
	/// 
	/// Returns whether or not the sector uses its own water settings and
	/// therefore overwrites the global terrain water settings for the ground level.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// sector has its own ground level settings which overwrite the
	/// terrain's ground level settings.
	virtual bool						GetSectorOverwritesTerrainGroundWaterLevel() const = 0;

	/// @brief Returns whether or not the sector uses its own water settings.
	/// 
	/// Returns whether or not the sector uses its own water settings and
	/// therefore overwrites the global terrain water settings for the ground level.
	/// @param bOverwriteTerrainGroundWaterLevel <i>true</i> or <i>false</i>, whether or not the
	/// sector has its own ground level settings which overwrite the terrain's ground level settings.
	virtual void						SetSectorOverwritesTerrainGroundWaterLevel(bool bOverwriteTerrainGroundWaterLevel) = 0;

}; // class CIPnPTCSectorWater01



#endif // !defined(PNPTC_CISECTORWATER_H_INCLUDED)