/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSelection.h
// Interface definition of a selection object containing
// the currently selected terrain elements.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISELECTION_H_INCLUDED)
#define PNPTC_CISELECTION_H_INCLUDED
#pragma once

/// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"


/// @brief Contains all heightmap elements currently selected.
///
/// For the editing process of a heightmap it is possible
/// for the user to select an almost arbitrary number of
/// heightmap elements (To be supported in future versions).
/// All heightmap elements currently selected can be accessed 
/// by a CIPnPTCSelection01 object.<br>
/// <b>Remarks: </b> The difference between a brush and
/// a selection object is the following:<br>
/// A selection contains single separated heightmap elements.<br>
/// A brush will internally be created from a selection by 
/// aggregating all heightmap elements around the selected
/// heightmap elements.<br>
/// So a brush is a region (or maybe several separated regions)
/// of heightmap elements.
class PNPTCPLUGIN_CALLBACK CIPnPTCSelection01
{
public:
	/// @brief Returns the number of selected elements.
	///
	/// Returns the number of heightmap elements that
	/// are currently selected by the user.
	/// @return The number of selected heightmap elements.
	virtual int					GetNumberOfElements()=0;

	/// @brief Return one selected heightmap element
	///
	/// Returns one element from the list of currently selected
	/// heightmap elements
	/// @param iIndex index of the heightmap element to be returned.
	/// The value of this parameter must be within the range of
	/// [0..GetNumberOfElements()-1].
	/// @param posReturn The returned selection element describing the heightmap position.
	/// @return Returns <i>true</i> or <i>false</i> whether or
	/// not the selected element could be returned.
	virtual bool				GetElement(int iIndex,PnPTCHMapPosition& posReturn)=0;

};


#endif // !defined(PNPTC_CISELECTION_H_INCLUDED)