/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCSound.h
// Interface definition to access a sound.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. January 2005
// ----------------------------------------------------------------------------------
// change history:
//	05. Feb. 2008 (Ralf Westphal)
//		Added GetPropertyContainer()
//  04. May 2008 (Ralf Westphal)
//		Added GetComments()
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CISOUND_H_INCLUDED)
#define PNPTC_CISOUND_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcpropertyvalue.h"


#pragma pack(push)
#pragma pack(1)



/// @brief Callback accessor class for sounds.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to a sound.
class PNPTCPLUGIN_CALLBACK CIPnPTCSound01
{
public:

	// --------------------------------------------------------------------------------------------
	// base sound and file methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the internal name of the sound.
	///
	/// Returns the internal name of the sound as it is used
	/// within the main program.
	/// @return Returns the internal name of the sound.
	virtual char*						GetName()=0;


	// --------------------------------------------------------------------------------------------
	// methods to save the sound
	// --------------------------------------------------------------------------------------------

	/// @brief Saves the sound to a file.
	///
	/// Saves the sound to a file.
	/// @param cPathName the path (not including filename) to that
	/// the sound should be saved.
	/// @param cFileName (output) Pointer to a char array that will
	/// contain the file name to which the sound was saved. The memory 
	/// to store the file name in the char array will be created during 
	/// the call of SaveToFile. The caller is responsible to delete this 
	/// mem (using free) if it is no longer needed.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the output parameter contains valid data.
	virtual bool						SaveToFile(char* cPathName,char** cFileName)=0;
	
	/// @brief Saves the sound to memory.
	///
	/// Saves the sound file to the memory.
	/// @param pMem (output) Pointer to a memory area that contains
	/// the saved data if the call was successfully. The memory will
	/// be created during the call of SaveToMem. The caller is responsible
	/// to delete this mem (using free) if it is no longer needed.
	/// @param dwSize (output) Size of the created memory if the call
	/// was successfull.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// call was successfull and the parameters contain valid data.
	virtual bool						SaveToMem(void** pMem,DWORD* dwSize)=0;

	// --------------------------------------------------------------------------------------------
	// additional methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the property container.
	///
	/// Returns the property container for user defined property
	/// values of this texture.
	/// @return Returns the texture's property container.
	virtual CIPnPTCPropertyContainer01*	GetPropertyContainer() = 0;

	/// @brief Returns a global sound comment.
	///
	/// Returns a comment text which was specified for
	/// for this type of sound globally in the sound manager.
	/// @return Comment text for this sound.
	virtual const char*					GetComments() const=0;

}; // class CIPnPTCSound01


#pragma pack(pop)


#endif // !defined(PNPTC_CISOUND_H_INCLUDED)