/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCTerrainWater.h
// Interface definition of an object containing the basic water settings
// of a terrain.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	03. Sep. 2006
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CITERRAINWATER_H_INCLUDED)
#define PNPTC_CITERRAINWATER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcwaterdefinition.h"
#include "./cipnptcwaterlevel.h"


/// @brief Contains the water settings for a terrain.
///
/// An instance of this interface is passed to your
/// plugin to give you information about the water
/// settings of a terrain.
class PNPTCPLUGIN_CALLBACK CIPnPTCTerrainWater01 : public CIPnPTCWaterLevel01
{
public:

	/// @brief Adds a new water definition to the list.
	/// 
	/// Adds a new water definition to the list.
	/// @return Returns a pointer to the newly created water
	/// definition if successfull, otherwise <i>NULL</i>.
	virtual CIPnPTCWaterDefinition01*	AddWaterDefinition() = 0;

	/// @brief Removes a water definition from the list.
	/// 
	/// Removes a water definition from the list.
	/// @param guidDelete GUID of the water definition to be deleted.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the water definition could be deleted.
	virtual bool						DeleteWaterDefinition(const GUID& guidDelete) = 0;

	/// @brief Returns the water definition for a given GUID.
	/// 
	/// Returns the water definition for a given GUID.
	/// @param guidReturn GUID of the water definition to be returned.
	/// @return Returns a pointer to the desired water
	/// definition if it exists in the list, otherwise <i>NULL</i>.
	virtual CIPnPTCWaterDefinition01*	GetWaterDefinition(const GUID& guidReturn) const = 0;

	/// @brief Returns the number of water definitions of the terrain.
	/// 
	/// Returns the number of water definitions of the terrain.
	/// @return Number of water definitions.
	virtual unsigned int				GetNumWaterDefinitions() const = 0;

	/// @brief Returns the water definition for a given index in the list.
	/// 
	/// Returns the water definition for a given index in the list.
	/// @param nIndex Index of the water definition to be returned
	/// in the range [0..GetNumWaterDefinitions()-1].
	/// @return Returns a pointer to the desired water
	/// definition if it exists in the list, otherwise <i>NULL</i>.
	virtual CIPnPTCWaterDefinition01*	GetWaterDefinition(unsigned int nIndex) const = 0;

}; // class CIPnPTCTerrainWater01



#endif // !defined(PNPTC_CITERRAINWATER_H_INCLUDED)