/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCTexture.h
// Interface definition to access a terrain sector's texturemap.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	25. March 2005
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CITEXTUREMAP_H_INCLUDED)
#define PNPTC_CITEXTUREMAP_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"
#include "cipnptctexture.h"

#pragma pack(push)
#pragma pack(1)



// -----------------------------------------------------------------------------------

/// @brief Callback accessor class for sector texturemaps.
/// 
/// An instance of this interface is passed to your
/// plugin to enable access to a sector's texturemap.
/// A texturemap object holds pointers to several texture.
/// Each texture covers one sector patch. So if a sector has
/// the size of 1024x1024 and is divided into patches of the
/// size 128x128, there are 8x8=64 textures managed by this
/// texturemap object.
class PNPTCPLUGIN_CALLBACK CIPnPTCTextureMap01
{
public:

	// --------------------------------------------------------------------------------------------
	// base texturemap methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns the number of vertices of one texture border.
	///
	/// Returns the number of vertices covered by one texturemap patch.
	/// The returned value is the number of vertices of one texture border.
	/// So the complete number of vertices per texturemap is
	/// GetNumTextureBorderVertices()*GetNumTextureBorderVertices().
	/// @return Returns the number of vertices per texture border.
	virtual int						GetNumTextureBorderVertices()=0;

	/// @brief Returns the number of textures per sector border.
	///
	/// This method returns the number of texture per sector border.
	/// The complete sector's texturemap consists of 
	/// GetNumTexturesPerSectorBorder()*GetNumTexturesPerSectorBorder()
	/// single textures, each one covering a sector patch of a border size
	/// GetNumTextureBorderVertices()-1 (GetNumTextureBorderVertices() vertices).
	/// @return Number of textures per sector border.
	virtual int						GetNumTexturesPerSectorBorder()=0;

	/// @brief Returns the UV coordinates of a texturemap vertex.
	///
	/// Returns the texture UV coordinates used by the texturemap
	/// at a given position on the map.
	/// @param iX X position of the texturemap element [0..GetNumTextureBorderVertices()-1].
	/// @param iY Y position of the texturemap element [0..GetNumTextureBorderVertices()-1].
	/// @return Returns the texturemap's UV coordinates.
	virtual PnPTCUV					GetTextureUV(int iX,int iY)=0;


	// --------------------------------------------------------------------------------------------
	// methods to access the textures
	// --------------------------------------------------------------------------------------------

	/// @brief Returns one texture of the texturemap.
	///
	/// @param iTexX X index of the texturemap's texture to be returned
	/// [0..GetNumTexturesPerSectorBorder()-1].
	/// @param iTexY Y index of the texturemap's texture to be returned
	/// [0..GetNumTexturesPerSectorBorder()-1].
	/// @return Returns a pointer to the texture covering one sector patch
	/// with the given index. Returns <i>NULL</i> in case of an error.
	virtual CIPnPTCTexture01*		GetTexture(int iTexX,int iTexY)=0;

}; // class


#pragma pack(pop)


#endif // !defined(PNPTC_CITEXTUREMAP_H_INCLUDED)