/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCVegetationRenderer.h
// Interface definition to access the main applications vegetation renderer.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	15. October 2005
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIVEGETATIONRENDERER_H_INCLUDED)
#define PNPTC_CIVEGETATIONRENDERER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"


#pragma pack(push)
#pragma pack(1)




/// @brief Main applications vegetation renderer.
///
/// This is an interface class to the main applications
/// vegetation render. It can be used to add dynamically
/// generated grass placements to the 3D scene.
class PNPTCPLUGIN_CALLBACK CIPnPTCVegetationRenderer01
{
public:

	// --------------------------------------------------------------------------------------------
	// scene management
	// --------------------------------------------------------------------------------------------

	/// @brief Adds one grass placement to the scene management.
	///
	/// Adds a new grass placement at a given position to the
	/// main applications vegetation renderer.
	/// @param vecCenter 2D position on the sector, where the grass
	/// placement should be added.
	/// @param iGrass Index/ID of the grass to be added.
	/// @param fScale Scaling factor to be used for the new placement.
	/// @param fRotZ Rotation arount the up axis of the new placement.
	/// @return <i>true</i> or <i>false</i>, whether or not the
	/// grass placement could be added.
	virtual bool		AddGrassPlacement(const PnPTC2DVec& vecCenter,unsigned int iGrass,float fScale,float fRotZ)=0;

}; // class


#pragma pack(pop)

#endif // !defined(PNPTC_CIVEGETATIONRENDERER_H_INCLUDED)