/////////////////////////////////////////////////////////////////////////////////////
// CIPnPTCWaterLevel.h
// Interface definition of an object containing the ground water level parameters
// of a terrain or sector.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	04. Jan. 2008
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CIWATERLEVEL_H_INCLUDED)
#define PNPTC_CIWATERLEVEL_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "./ipnptcplugin.h"
#include "./cipnptcwaterdefinition.h"


/// @brief Contains the ground water level settings for a terrain.
///
/// An instance of this interface is passed to your
/// plugin to give you information about the ground 
/// water level settings of a terrain or sector.
class PNPTCPLUGIN_CALLBACK CIPnPTCWaterLevel01
{
public:

	/// @brief Returns the terrain's ground water level.
	/// 
	/// Returns the terrain's ground water level.
	/// @return Ground water level height in meters.
	virtual float						GetGroundWaterLevel() const = 0;

	/// @brief Sets the terrain's ground water level.
	/// 
	/// Sets the terrain's ground water level.
	/// @param fGroundWaterLevel Ground water level height in meters
	/// to be set. The value is capped to an appropriate range
	/// before it is set.
	virtual void						SetGroundWaterLevel(float fGroundWaterLevel) = 0;

	/// @brief Return a pointer to the water definition object.
	/// 
	/// Return a pointer to the water definition object
	/// which is used to render the terrain's ground water level.
	/// @return Returns a pointer to the ground water definition
	/// object if successfull. If the ground water level object
	/// is not set, <i>NULL</i> is returned.
	virtual CIPnPTCWaterDefinition01*	GetGroundWaterDefinition() const = 0;

	/// @brief Sets the water definition object.
	/// 
	/// Sets the water definition object which is used to render 
	/// the terrain's ground water level.
	/// @param pGroundWaterDef Pointer to the ground water definition
	/// which should be used as the terrain's ground water level.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the ground water definition could be set successfully.
	/// If the ground water level could not be set, the previous
	/// ground water level definition will still be set.
	virtual bool						SetGroundWaterDefinition(CIPnPTCWaterDefinition01* pGroundWaterDef) = 0;

}; // class CIPnPTCWaterLevel01



#endif // !defined(PNPTC_CIWATERLEVEL_H_INCLUDED)