/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCColors.h
// Color types used within the plugin interface.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	18. Feb. 2004
// ----------------------------------------------------------------------------------
// change history:
//  31. Jan. 2007   : Added PNPTC_BGR data type.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2006 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_COLORS_H_INCLUDED)
#define PNPTC_COLORS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#pragma pack(push)
#pragma pack(1)



// ---------------------------------------------------------------------------------
/// @brief RGB Color value
PNPTCPLUGIN_CALLBACK typedef struct PNPTC_RGB
{
	unsigned char	r;	//!< red color component.
	unsigned char	g;	//!< green color component.
	unsigned char	b;	//!< blue color component.


	/// Construction of a black initialized color value.
	PNPTC_RGB()
	{
		this->r=0;this->g=0;this->b=0;
	};

	/// Construction and initialization of a color value.
	/// @param red Red color component for initialization.
	/// @param green Green color component for initialization.
	/// @param blue Blue color component for initialization.
	PNPTC_RGB(unsigned char red,unsigned char green,unsigned char blue)
	{
		this->r=red;this->g=green;this->b=blue;
	};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_RGB&		operator *= (float scalar) {this->r=static_cast<unsigned char>(this->r*scalar);this->g=static_cast<unsigned char>(this->g*scalar);this->b=static_cast<unsigned char>(this->b*scalar);return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_RGB		operator * (float scalar) const {return PNPTC_RGB(static_cast<unsigned char>(this->r*scalar),static_cast<unsigned char>(this->g*scalar),static_cast<unsigned char>(this->b*scalar));};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @param col The color values to be multiplied.
	inline friend PNPTC_RGB operator * (float scalar, const PNPTC_RGB& col) {return PNPTC_RGB(static_cast<unsigned char>(col.r*scalar),static_cast<unsigned char>(col.g*scalar),static_cast<unsigned char>(col.b*scalar));};
	
	/// Adds two color values.
	/// @param col Color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_RGB&		operator += (const PNPTC_RGB& col) {this->r+=col.r;this->g+=col.g;this->b+=col.b;return *this;};

	/// Adds two color values.
	/// @param col color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_RGB		operator + (const PNPTC_RGB& col) const {return PNPTC_RGB(this->r+col.r,this->g+col.g,this->b+col.b);};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are identically, otherwise <i>false</i>.
	inline bool				operator == (const PNPTC_RGB& col) const {return this->r==col.r && this->g==col.g && this->b==col.b;};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are not identically, otherwise <i>false</i>.
	inline bool				operator != (const PNPTC_RGB& col) const {return this->r!=col.r || this->g!=col.g || this->b!=col.b;};

} PNPTC_RGB;



// ---------------------------------------------------------------------------------
/// @brief BGR Color value
PNPTCPLUGIN_CALLBACK typedef struct PNPTC_BGR
{
	unsigned char	b;	//!< blue color component.
	unsigned char	g;	//!< green color component.
	unsigned char	r;	//!< red color component.


	/// Construction of a black initialized color value.
	PNPTC_BGR()
	{
		this->r=0;this->g=0;this->b=0;
	};

	/// Construction and initialization of a color value.
	/// @param blue Blue color component for initialization.
	/// @param green Green color component for initialization.
	/// @param red Red color component for initialization.
	PNPTC_BGR(unsigned char blue,unsigned char green,unsigned char red)
	{
		this->r=red;this->g=green;this->b=blue;
	};

	/// Creates a bgr color value from a rgb color value.
	/// @param rgb Source rgb color value.
	PNPTC_BGR(const PNPTC_RGB& rgb) {this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;};

	/// Converts the bgr color value to a rgb color value.
	operator PNPTC_RGB () {return PNPTC_RGB(this->r,this->g,this->b);};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_BGR&		operator *= (float scalar) {this->r=static_cast<unsigned char>(this->r*scalar);this->g=static_cast<unsigned char>(this->g*scalar);this->b=static_cast<unsigned char>(this->b*scalar);return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_BGR		operator * (float scalar) const {return PNPTC_BGR(static_cast<unsigned char>(this->r*scalar),static_cast<unsigned char>(this->g*scalar),static_cast<unsigned char>(this->b*scalar));};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @param col The color values to be multiplied.
	inline friend PNPTC_BGR operator * (float scalar, const PNPTC_BGR& col) {return PNPTC_BGR(static_cast<unsigned char>(col.r*scalar),static_cast<unsigned char>(col.g*scalar),static_cast<unsigned char>(col.b*scalar));};
	
	/// Adds two color values.
	/// @param col Color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_BGR&		operator += (const PNPTC_BGR& col) {this->r+=col.r;this->g+=col.g;this->b+=col.b;return *this;};

	/// Adds two color values.
	/// @param col color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_BGR		operator + (const PNPTC_BGR& col) const {return PNPTC_BGR(this->r+col.r,this->g+col.g,this->b+col.b);};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are identically, otherwise <i>false</i>.
	inline bool				operator == (const PNPTC_BGR& col) const {return this->r==col.r && this->g==col.g && this->b==col.b;};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are not identically, otherwise <i>false</i>.
	inline bool				operator != (const PNPTC_BGR& col) const {return this->r!=col.r || this->g!=col.g || this->b!=col.b;};

} PNPTC_BGR;



// ---------------------------------------------------------------------------------
/// @brief RGBA Color value
PNPTCPLUGIN_CALLBACK typedef struct PNPTC_RGBA
{
	unsigned char	r;	//!< red color component.
	unsigned char	g;	//!< green color component.
	unsigned char	b;	//!< blue color component.
	unsigned char	a;	//!< alpha color component.


	/// Construction of a black initialized color value.
	PNPTC_RGBA()
	{
		this->r=0;this->g=0;this->b=0;this->a=255;
	};

	/// Construction and initialization of a color value.
	/// @param red Red color component for initialization.
	/// @param green Green color component for initialization.
	/// @param blue Blue color component for initialization.
	/// @param alpha Alpha color component for initialization.
	PNPTC_RGBA(unsigned char red,unsigned char green,unsigned char blue,unsigned char alpha)
	{
		this->r=red;this->g=green;this->b=blue;this->a=alpha;
	};

	/// Creates a rgba color value from a rgb color value.
	/// @param rgb Source rgb color value.
	PNPTC_RGBA(const PNPTC_RGB& rgb) {this->a=255;this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;};

	/// Converts the rgba color value to a rgb color value.
	operator PNPTC_RGB () {return PNPTC_RGB(this->r,this->g,this->b);};

	/// Operator for assigning a rgb value.
	/// @param rgb Color rgb value to be assigned.
	/// @return Returns the newly assigned color value.
	PNPTC_RGBA&	operator = (const PNPTC_RGB& rgb) {this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;this->a=255;return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_RGBA&		operator *= (float scalar) {this->r=static_cast<unsigned char>(this->r*scalar);this->g=static_cast<unsigned char>(this->g*scalar);this->b=static_cast<unsigned char>(this->b*scalar);this->a=static_cast<unsigned char>(this->a*scalar);return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_RGBA		operator * (float scalar) const {return PNPTC_RGBA(static_cast<unsigned char>(this->r*scalar),static_cast<unsigned char>(this->g*scalar),static_cast<unsigned char>(this->b*scalar),static_cast<unsigned char>(this->a*scalar));};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @param col The color values to be multiplied.
	inline friend PNPTC_RGBA operator * (float scalar, const PNPTC_RGBA& col) {return PNPTC_RGBA(static_cast<unsigned char>(col.r*scalar),static_cast<unsigned char>(col.g*scalar),static_cast<unsigned char>(col.b*scalar),static_cast<unsigned char>(col.a*scalar));};

	/// Adds two color values.
	/// @param col Color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_RGBA&		operator += (const PNPTC_RGBA& col) {this->r+=col.r;this->g+=col.g;this->b+=col.b;this->a+=col.a;return *this;};

	/// Adds two color values.
	/// @param col color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_RGBA		operator + (const PNPTC_RGBA& col) const {return PNPTC_RGBA(this->r+col.r,this->g+col.g,this->b+col.b,this->a+col.a);};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are identically, otherwise <i>false</i>.
	inline bool				operator == (const PNPTC_RGBA& col) const {return this->r==col.r && this->g==col.g && this->b==col.b && this->a==col.a;};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are not identically, otherwise <i>false</i>.
	inline bool				operator != (const PNPTC_RGBA& col) const {return this->r!=col.r || this->g!=col.g || this->b!=col.b || this->a!=col.a;};

} PNPTC_RGBA;




// ---------------------------------------------------------------------------------
/// @brief ARGB Color value
PNPTCPLUGIN_CALLBACK typedef struct PNPTC_ARGB
{
	unsigned char	a;	//!< alpha color component.
	unsigned char	r;	//!< red color component.
	unsigned char	g;	//!< green color component.
	unsigned char	b;	//!< blue color component.

	/// Construction of a black initialized color value.
	PNPTC_ARGB()
	{
		this->a=255;this->r=0;this->g=0;this->b=0;
	};

	/// Construction and initialization of a color value.
	/// @param red Red color component for initialization.
	/// @param green Green color component for initialization.
	/// @param blue Blue color component for initialization.
	/// @param alpha Alpha color component for initialization.
	PNPTC_ARGB(unsigned char alpha,unsigned char red,unsigned char green,unsigned char blue)
	{
		this->a=alpha;this->r=red;this->g=green;this->b=blue;
	};

	/// Creates a rgba color value from a rgb color value.
	/// @param rgb Source rgb color value.
	PNPTC_ARGB(const PNPTC_RGB& rgb) {this->a=255;this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;};

	/// Creates a rgba color value from a rgba color value.
	/// @param rgba Source rgba color value.
	PNPTC_ARGB(PNPTC_RGBA rgba) {this->a=rgba.a;this->r=rgba.r;this->g=rgba.g;this->b=rgba.b;};

	/// Converts the argb color value to a rgb color value.
	operator PNPTC_RGB () {return PNPTC_RGB(this->r,this->g,this->b);};

	/// Converts the argb color value to a rgba color value.
	operator PNPTC_RGBA () {return PNPTC_RGBA(this->r,this->g,this->b,this->a);};

	/// Operator for assigning a rgb value.
	/// @param rgb Color rgb value to be assigned.
	/// @return Returns the newly assigned color value.
	PNPTC_ARGB&	operator = (const PNPTC_RGB& rgb) {this->a=255;this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;return *this;};

	/// Operator for assigning a rgba value.
	/// @param rgba Color rgb value to be assigned.
	/// @return Returns the newly assigned color value.
	PNPTC_ARGB&	operator = (const PNPTC_RGBA& rgba) {this->a=rgba.a;this->r=rgba.r;this->g=rgba.g;this->b=rgba.b;return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_ARGB&		operator *= (float scalar) {this->r=static_cast<unsigned char>(this->r*scalar);this->g=static_cast<unsigned char>(this->g*scalar);this->b=static_cast<unsigned char>(this->b*scalar);this->a=static_cast<unsigned char>(this->a*scalar);return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_ARGB		operator * (float scalar) const {return PNPTC_ARGB(static_cast<unsigned char>(this->a*scalar),static_cast<unsigned char>(this->r*scalar),static_cast<unsigned char>(this->g*scalar),static_cast<unsigned char>(this->b*scalar));};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @param col The color values to be multiplied.
	inline friend PNPTC_ARGB operator * (float scalar, const PNPTC_ARGB& col) {return PNPTC_ARGB(static_cast<unsigned char>(col.a*scalar),static_cast<unsigned char>(col.r*scalar),static_cast<unsigned char>(col.g*scalar),static_cast<unsigned char>(col.b*scalar));};

	/// Adds two color values.
	/// @param col Color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_ARGB&		operator += (const PNPTC_ARGB& col) {this->r+=col.r;this->g+=col.g;this->b+=col.b;this->a+=col.a;return *this;};

	/// Adds two color values.
	/// @param col color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_ARGB		operator + (const PNPTC_ARGB& col) const {return PNPTC_ARGB(this->a+col.a,this->r+col.r,this->g+col.g,this->b+col.b);};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are identically, otherwise <i>false</i>.
	inline bool				operator == (const PNPTC_ARGB& col) const {return this->r==col.r && this->g==col.g && this->b==col.b && this->a==col.a;};

	/// Comparison.
	/// @param col Second color value with which to compare the color.
	/// @return Returns <i>true</i> if both color values are not identically, otherwise <i>false</i>.
	inline bool				operator != (const PNPTC_ARGB& col) const {return this->r!=col.r || this->g!=col.g || this->b!=col.b || this->a!=col.a;};

} PNPTC_ARGB;




// ---------------------------------------------------------------------------------
/// @brief BGRA Color value
PNPTCPLUGIN_CALLBACK typedef struct PNPTC_BGRA
{
	unsigned char	b;	//!< blue color component.
	unsigned char	g;	//!< green color component.
	unsigned char	r;	//!< red color component.
	unsigned char	a;	//!< alpha color component.

	/// Construction of a black initialized color value.
	PNPTC_BGRA()
	{
		this->a=255;this->r=0;this->g=0;this->b=0;
	};

	/// Construction and initialization of a color value.
	/// @param blue Blue color component for initialization.
	/// @param green Green color component for initialization.
	/// @param red Red color component for initialization.
	/// @param alpha Alpha color component for initialization.
	PNPTC_BGRA(unsigned char blue,unsigned char green,unsigned char red,unsigned char alpha)
	{
		this->a=alpha;this->r=red;this->g=green;this->b=blue;
	};

	/// Creates a rgba color value from a rgb color value.
	/// @param rgb Source rgb color value.
	PNPTC_BGRA(const PNPTC_RGB& rgb) {this->a=255;this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;};

	/// Creates a rgba color value from a bgr color value.
	/// @param rgb Source rgb color value.
	PNPTC_BGRA(const PNPTC_BGR& bgr) {this->a=255;this->r=bgr.r;this->g=bgr.g;this->b=bgr.b;};

	/// Creates a rgba color value from a rgba color value.
	/// @param rgba Source rgba color value.
	PNPTC_BGRA(PNPTC_RGBA rgba) {this->a=rgba.a;this->r=rgba.r;this->g=rgba.g;this->b=rgba.b;};

	/// Creates a bgra color value from a argb color value.
	/// @param argb Source argb color value.
	PNPTC_BGRA(PNPTC_ARGB argb) {this->a=argb.a;this->r=argb.r;this->g=argb.g;this->b=argb.b;};

	/// Converts the bgra color value to a rgb color value.
	operator PNPTC_RGB () {return PNPTC_RGB(this->r,this->g,this->b);};

	/// Converts the bgra color value to a bgr color value.
	operator PNPTC_BGR () {return PNPTC_BGR(this->b,this->g,this->r);};

	/// Converts the bgra color value to a rgba color value.
	operator PNPTC_RGBA () {return PNPTC_RGBA(this->r,this->g,this->b,this->a);};

	/// Converts the bgra color value to a argb color value.
	operator PNPTC_ARGB () {return PNPTC_ARGB(this->a,this->r,this->g,this->b);};

	/// Operator for assigning a rgb value.
	/// @param rgb Color rgb value to be assigned.
	/// @return Returns the newly assigned color value.
	PNPTC_BGRA&	operator = (const PNPTC_RGB& rgb) {this->a=255;this->r=rgb.r;this->g=rgb.g;this->b=rgb.b;return *this;};

	/// Operator for assigning a rgba value.
	/// @param rgba Color rgba value to be assigned.
	/// @return Returns the newly assigned color value.
	PNPTC_BGRA&	operator = (const PNPTC_RGBA& rgba) {this->a=rgba.a;this->r=rgba.r;this->g=rgba.g;this->b=rgba.b;return *this;};

	/// Operator for assigning a argb value.
	/// @param argb Color argb value to be assigned.
	/// @return Returns the newly assigned color value.
	PNPTC_BGRA&	operator = (const PNPTC_ARGB& argb) {this->a=argb.a;this->r=argb.r;this->g=argb.g;this->b=argb.b;return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_BGRA&		operator *= (float scalar) {this->r=static_cast<unsigned char>(this->r*scalar);this->g=static_cast<unsigned char>(this->g*scalar);this->b=static_cast<unsigned char>(this->b*scalar);this->a=static_cast<unsigned char>(this->a*scalar);return *this;};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @return Returns the result of the multiplication.
	inline PNPTC_BGRA		operator * (float scalar) const {return PNPTC_BGRA(static_cast<unsigned char>(this->b*scalar),static_cast<unsigned char>(this->g*scalar),static_cast<unsigned char>(this->r*scalar),static_cast<unsigned char>(this->a*scalar));};

	/// Scalar multiply a color value.
	/// @param scalar Scalar value which should be multiplied with the color value.
	/// @param col The color values to be multiplied.
	inline friend PNPTC_BGRA operator * (float scalar, const PNPTC_BGRA& col) {return PNPTC_BGRA(static_cast<unsigned char>(col.b*scalar),static_cast<unsigned char>(col.g*scalar),static_cast<unsigned char>(col.r*scalar),static_cast<unsigned char>(col.a*scalar));};

	/// Adds two color values.
	/// @param col Color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_BGRA&		operator += (const PNPTC_BGRA& col) {this->r+=col.r;this->g+=col.g;this->b+=col.b;this->a+=col.a;return *this;};

	/// Adds two color values.
	/// @param col color value to be added.
	/// @return Returns the two added color values.
	inline PNPTC_BGRA		operator + (const PNPTC_BGRA& col) const {return PNPTC_BGRA(this->b+col.b,this->g+col.g,this->r+col.r,this->a+col.a);};

} PNPTC_BGRA;




#pragma pack(pop)


#endif // !defined(PNPTC_COLORS_H_INCLUDED)