/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCImportPlugIn.h
// Interface definition for importer plugins.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	23. July 2005
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2008 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_IIMPORTPLUGIN_H_INCLUDED)
#define PNPTC_IIMPORTPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// include base definition of PnP-TC PlugIn interface
#include "ipnptcplugin.h"

// includes
#include "CIPnPTCTerrain.h"
#include "CIPnPTCSector.h"
#include "CIPnPTCApplication.h"



/// @brief Interface definition for importer plugin API.
/// 
/// Imports all definitions of single sectors or the whole
/// terrain.
class PNPTCPLUGIN_API IPnPTCImportPlugIn01 : public IPnPTCPlugIn
{
public:

	// --------------------------------------------------------------------------------------------
	// Initialization methods
	// --------------------------------------------------------------------------------------------

	/// @brief Returns interface version.
	///
	/// Returns the version of the implemented Plugin.<br>
	/// Don't override this method!
	/// @return Implemented API version
	int		GetVersion() {return 1;};

	/// @brief Returns plugin's guid.
	///
	/// Get a unique identifier for this plugin.
	/// @return The plugin's uid.
	virtual GUID	GetGUID()=0;


	// --------------------------------------------------------------------------------------------
	// Importing methods
	// --------------------------------------------------------------------------------------------

	/// @brief Imports one sector.
	///
	/// @param App Reference to the PnP TerrainCreator main application.
	/// @param Sector Reference to the sector to be imported.
	/// @param Terrain Reference to the terrain of the imported sector.
	/// @return whether or not the import was successfull.
	virtual bool	ImportSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain)=0;

	/// @brief Imports the whole terrain.
	///
	/// @param App Reference to the PnP TerrainCreator main application.
	/// @param Terrain Reference to the terrain to be imported.
	/// @return whether or not the import was successfull.
	virtual bool	ImportTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain)=0;


}; // class

#endif // !defined(PNPTC_IIMPORTPLUGIN_H_INCLUDED)