/////////////////////////////////////////////////////////////////////////////////////
// IPnPTCVectors.h
// Vector types used within the plugin interface.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// creation:	26. Mai. 2007
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2006 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_VECTORS_H_INCLUDED)
#define PNPTC_VECTORS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#pragma pack(push)
#pragma pack(1)



// ---------------------------------------------------------------------------------
/// @brief A 2D point.
///
/// Data structure containing 2D real value coordinates.
PNPTCPLUGIN_CALLBACK typedef struct PnPTC2DVec
{
	float x;			//!< x-position.
	float y;			//!< y-position.

	/// Construction of a zero initialized vector.
	PnPTC2DVec()
		: x(0), y(0) {};

	/// Construction and initialization of a vector.
	/// @param fX X component of the vector.
	/// @param fY Y component of the vector.
	PnPTC2DVec(float fX,float fY)
		: x(fX), y(fY) {};

	/// Comparison.
	/// @param vec Second vector with which to compare the vector.
	/// @return Returns <i>true</i> if both vectors are identically, otherwise <i>false</i>.
	inline bool			operator == (const PnPTC2DVec& vec) const {return this->x==vec.x && this->y==vec.y;};

	/// Comparison.
	/// @param vec Second vector with which to compare the vector.
	/// @return Returns <i>true</i> if both vectors are not identically, otherwise <i>false</i>.
	inline bool			operator != (const PnPTC2DVec& vec) const {return this->x!=vec.x || this->y!=vec.y;};
} PnPTC2DVec;





// ---------------------------------------------------------------------------------
/// @brief A 3D vector.
///
/// Data structure containing 2D real value coordinates.
PNPTCPLUGIN_CALLBACK typedef struct PnPTC3DVec
{
	float x;			//!< x-position.
	float y;			//!< y-position.
	float z;			//!< z-position.

	/// Construction of a zero initialized vector.
	PnPTC3DVec()
		: x(0), y(0), z(0) {};

	/// Construction and initialization of a vector.
	/// @param fX X component of the vector.
	/// @param fY Y component of the vector.
	/// @param fZ Z component of the vector.
	PnPTC3DVec(float fX,float fY,float fZ)
		: x(fX), y(fY), z(fZ) {};

	/// Comparison.
	/// @param vec Second vector with which to compare the vector.
	/// @return Returns <i>true</i> if both vectors are identically, otherwise <i>false</i>.
	inline bool			operator == (const PnPTC3DVec& vec) const {return this->x==vec.x && this->y==vec.y && this->z==vec.z;};

	/// Comparison.
	/// @param vec Second vector with which to compare the vector.
	/// @return Returns <i>true</i> if both vectors are not identically, otherwise <i>false</i>.
	inline bool			operator != (const PnPTC3DVec& vec) const {return this->x!=vec.x || this->y!=vec.y || this->z!=vec.z;};

} PnPTC3DVec;



#pragma pack(pop)


#endif // !defined(PNPTC_VECTORS_H_INCLUDED)