/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectInstance.cpp
// 3D object instance data.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  04. Oct. 2006 : Created
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include ".\3dobjectinstance.h"
#include "./I3DObjectRenderer.h"
#include "./3DObject.h"
#include "../pnptcutils/pnpse/pnpseray.h"


C3DObjectInstance::C3DObjectInstance(C3DObject* pObj)
	: m_pObject(pObj)
{
	// read initial bounding box from render
	I3DObjectRenderer* pRenderer=NULL;
	if (this->m_pObject)
	{
		pRenderer=this->m_pObject->GetRenderer();
	}

	// get bounding box
	if (pRenderer)
	{
		this->m_BoundingBox=pRenderer->GetBoundingBox();
	}
}

C3DObjectInstance::~C3DObjectInstance(void)
{
}

void C3DObjectInstance::SetPosition(const D3DXMATRIX& matPosition)
{
	// store position and comput inverse
	this->m_matWorld=matPosition;
	D3DXMatrixInverse(&this->m_matInvWorld,NULL,&matPosition);

	// update bounding box from render
	I3DObjectRenderer* pRenderer=NULL;
	if (this->m_pObject)
	{
		pRenderer=this->m_pObject->GetRenderer();
	}

	// update bounding box
	if (pRenderer)
	{
		// compute bounding box
		const pnpse::CBoundingBox& bbox=pRenderer->GetBoundingBox();
		D3DXVECTOR3 vecMin(bbox.GetMin());
		D3DXVECTOR3 vecMax(bbox.GetMax());
		D3DXVECTOR3 verticesSource[8];
		verticesSource[0]=D3DXVECTOR3(vecMin.x,vecMin.y,vecMin.z);
		verticesSource[1]=D3DXVECTOR3(vecMax.x,vecMin.y,vecMin.z);
		verticesSource[2]=D3DXVECTOR3(vecMin.x,vecMax.y,vecMin.z);
		verticesSource[3]=D3DXVECTOR3(vecMin.x,vecMin.y,vecMax.z);
		verticesSource[4]=D3DXVECTOR3(vecMax.x,vecMax.y,vecMin.z);
		verticesSource[5]=D3DXVECTOR3(vecMax.x,vecMin.y,vecMax.z);
		verticesSource[6]=D3DXVECTOR3(vecMin.x,vecMax.y,vecMax.z);
		verticesSource[7]=D3DXVECTOR3(vecMax.x,vecMax.y,vecMax.z);
		D3DXVECTOR3 verticesDest[8];
		D3DXVec3TransformCoord(&verticesDest[0],&verticesSource[0],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[1],&verticesSource[1],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[2],&verticesSource[2],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[3],&verticesSource[3],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[4],&verticesSource[4],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[5],&verticesSource[5],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[6],&verticesSource[6],&this->m_matWorld);
		D3DXVec3TransformCoord(&verticesDest[7],&verticesSource[7],&this->m_matWorld);

		// set bounding box
		this->m_BoundingBox.SetBorderPoints(min(min(min(min(min(min(min(verticesDest[0].x,verticesDest[1].x),verticesDest[2].x),verticesDest[3].x),verticesDest[4].x),verticesDest[5].x),verticesDest[6].x),verticesDest[7].x),
											min(min(min(min(min(min(min(verticesDest[0].y,verticesDest[1].y),verticesDest[2].y),verticesDest[3].y),verticesDest[4].y),verticesDest[5].y),verticesDest[6].y),verticesDest[7].y),
											min(min(min(min(min(min(min(verticesDest[0].z,verticesDest[1].z),verticesDest[2].z),verticesDest[3].z),verticesDest[4].z),verticesDest[5].z),verticesDest[6].z),verticesDest[7].z),
											max(max(max(max(max(max(max(verticesDest[0].x,verticesDest[1].x),verticesDest[2].x),verticesDest[3].x),verticesDest[4].x),verticesDest[5].x),verticesDest[6].x),verticesDest[7].x),
											max(max(max(max(max(max(max(verticesDest[0].y,verticesDest[1].y),verticesDest[2].y),verticesDest[3].y),verticesDest[4].y),verticesDest[5].y),verticesDest[6].y),verticesDest[7].y),
											max(max(max(max(max(max(max(verticesDest[0].z,verticesDest[1].z),verticesDest[2].z),verticesDest[3].z),verticesDest[4].z),verticesDest[5].z),verticesDest[6].z),verticesDest[7].z));
	}
}

bool C3DObjectInstance::Render(bool bAlpha/*=false*/,int iMaterial/*=-1*/,bool bSetResources/*=true*/) const
{
	// get renderer
	I3DObjectRenderer* pRenderer=NULL;
	if (this->m_pObject)
	{
		pRenderer=this->m_pObject->GetRenderer();
	}

	// render it
	if (pRenderer)
	{
		return pRenderer->Render(this->m_matWorld,bAlpha,iMaterial,bSetResources);
	}
	else
	{
		// no renderer
		return false;
	}
}

bool C3DObjectInstance::Intersect(const D3DXVECTOR3& vecStart,const D3DXVECTOR3& vecDir)
{
	// convert vector into local space and create a ray object
	D3DXMATRIX	matWorldInv=this->m_matInvWorld;
	D3DXVECTOR3	vecStartLocal;
	D3DXVECTOR3	vecDirLocal;
	D3DXVec3TransformCoord(&vecStartLocal,&vecStart,&matWorldInv);
	matWorldInv._41=0;
	matWorldInv._42=0;
	matWorldInv._43=0;
	D3DXVec3TransformCoord(&vecDirLocal,&vecDir,&matWorldInv);
	pnpse::CRay ray;
	ray.MakeRayFromPointDirection((FLOAT*)vecStartLocal,(FLOAT*)vecDirLocal);

	// let the renderer object test for intersection
	I3DObjectRenderer* pRenderer=NULL;
	if (this->m_pObject)
	{
		pRenderer=this->m_pObject->GetRenderer();
	}

	// test intersection
	if (pRenderer)
	{
		return pRenderer->Intersect(ray);
	}
	else
	{
		// no renderer => no intersection
		return false;
	}
}
