/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectRendererDX.h
// Mesh renderer which is based on DX.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////
#if !defined(PNPTC_3DOBJECTRENDERERDX_H_INCLUDED)
#define PNPTC_3DOBJECTRENDERERDX_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// includes
#include <vector>
#include <d3dx9.h>
#include "I3DObjectRenderer.h"
#include "TextureResourceManager.h"


/// Object renderer class, which is based on DirectX.
class C3DObjectRendererDX : public I3DObjectRenderer
{
public:

	// --------------------------------------------------------------
	// construction / destruction
	// --------------------------------------------------------------

	/// Construction of a new DX renderer.
	/// @param pD3DDevice The DX rendering device.
	C3DObjectRendererDX(IDirect3DDevice9* pD3DDevice);

	/// Destruction of the renderer
	virtual ~C3DObjectRendererDX(void);


	// ---------------------------------------------------------------------
	// 3D rendering methods
	// ---------------------------------------------------------------------

	/// Returns the untransformed/original bounding box of the mesh.
	/// @return The mesh's bounding box.
	const pnpse::CBoundingBox&		GetBoundingBox() const {return this->m_BoundingBox;};


	// ---------------------------------------------------------------------
	// Retrieve information about the mesh
	// ---------------------------------------------------------------------

	/// Returns the number of faces.
	/// @return Number of faces of this object.
	int								GetNumFaces() const {return this->m_iNumFaces;};

	/// Returns the number of vertices.
	/// @return Number of vertices of this object.
	int								GetNumVertices() const {return this->m_iNumVertices;};


protected:

	// ---------------------------------------------------------------------
	// static variables
	// ---------------------------------------------------------------------

	/// resource manager for textures
	static CTextureResourceManager	s_TextureManager;


	// ---------------------------------------------------------------------
	// member variables
	// ---------------------------------------------------------------------

	/// The DX rendering device.
	IDirect3DDevice9*				m_pD3DDevice;

	/// number of faces
	int								m_iNumFaces;

	/// number of vertices
	int								m_iNumVertices;

	/// Bounding box of this mesh
	pnpse::CBoundingBox				m_BoundingBox;
};

#endif // !defined(PNPTC_3DOBJECTRENDERERDX_H_INCLUDED)