/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerDXStudio.cpp
// 3d object serialization class for .dxmesh files.
// // ----------------------------------------------------------------------------------
// author:		miko, based on pnpTC demos
// ----------------------------------------------------------------------------------
// change history:
//   xx. aaa. 2009 : miko
// ----------------------------------------------------------------------------------
// copyright notice:
/////////////////////////////////////////////////////////////////////////////////////

#define _CRT_SECURE_NO_DEPRECATE 1

#include "3DObjectData.h"
#include ".\3dobjectserializerdxstudio.h"
#include "../PnPTCUtils/Utils.h"

#include "dxmeshapi.h"


C3DObjectSerializerDXStudio::C3DObjectSerializerDXStudio(void)
{
}

C3DObjectSerializerDXStudio::~C3DObjectSerializerDXStudio(void)
{
}

bool C3DObjectSerializerDXStudio::Load(C3DObjectData& ObjectData,const std::string& strFileName)
{
	// loading not yet supported => loading is implemented using the D3DXLoadMesh routines
	return(false);
}

bool C3DObjectSerializerDXStudio::Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale/*=1*/,int iUpRotation/*=0*/,int iFlipAxis/*=0*/,const char* cBasePathName/*=NULL*/,bool bSeparateMeshFiles/*=false*/,bool bExportTextures/*=false*/,const char* cSourceTexturePath/*=NULL*/,IDirect3DDevice9* pD3DDevice/*=NULL*/)
{
	// Extract the save path
	int ic = (int) strFileName.find_last_of('\\');
	std::string savePath = strFileName.substr(0,ic+1);
	
	// Return value, used for basic error checking with DXMeshAPI
	DX::DXError dxErr;
	
	// Create the dxstudio model
	DX::IDXModel* aModel = DX::CreateModel();
	if (!aModel) { OutputDebugString("Error on CreateModel"); return(false); }

	// Add some info to the model's meta data
	DX::IDXModelInfo* info = aModel->GetInfo();
	info->SetTitle(L"PnPTC Terrain" );
	info->SetSummary(L"Created with PnPTC and exported with miko's PnPTCDXStudioFileHMapExportPlugIn v1.4");
	info->AddTag(L"Terrain");
	info->AddTag(L"DXMeshAPI");
	info->AddTag(L"PnPTC");

	// Create the one group which contains all the meshes
	DX::IDXGroup* aGroup;
	dxErr = aModel->CreateGroup("Terrain",&aGroup); 
	if (dxErr != DX::Error_Success) { DX::DestroyModel(aModel); OutputDebugString("Error on CreateGroup"); return(false); }
	
	// -----------------------------------------------------------------------------
	// First, create an entry for each material 
	for (unsigned long lMaterial=0;lMaterial<ObjectData.GetNumMaterials();lMaterial++)
	{
		const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(lMaterial);
		if (pMaterial)
		{
			DX::IDXMaterial *mat;
			dxErr = aModel->CreateMaterial(MakeCompatibleString(pMaterial->strName).c_str(), &mat);
			if (dxErr != DX::Error_Success) { DX::DestroyModel(aModel); OutputDebugString("Error on CreateMaterial"); return(false); }

			// color argb
			DX::DXColor colAmb = DX::CreateColor(pMaterial->colAmbient.a,
												 pMaterial->colAmbient.r,
												 pMaterial->colAmbient.g,
												 pMaterial->colAmbient.b); 

			DX::DXColor colDif = DX::CreateColor(pMaterial->colDiffuse.a,
												 pMaterial->colDiffuse.r,
												 pMaterial->colDiffuse.g,
												 pMaterial->colDiffuse.b); 
			
			mat->SetAmbient(colAmb);
			mat->SetDiffuse(colDif);
			mat->SetSpecular(DX::CreateColor(255,5,5,5)); // a standard value
			mat->SetSpecularSharpness(50);
			
			// Add the texture map if present
			if (pMaterial->iTextures>0)
			{		
				wchar_t wc1[1024];
				wchar_t wc2[1024];
				swprintf(wc1,L"%S%S",savePath.c_str(),pMaterial->strTexture[0].c_str());
				swprintf(wc2,L"%S",pMaterial->strTexture[0].c_str());
				
				dxErr = mat->AddStandardMap(DX::StandardMap_Diffuse,wc2,NULL);
				if (dxErr != DX::Error_Success) { DX::DestroyModel(aModel); OutputDebugString("Error on AddStandardMap"); return(false); }

				// Changed with 1.4 / version 6/18/09 of DXMesh API 1.0.0.1
				DX::IDXFileSystemPtr pFileSystem = aModel->GetFileSystem();
				
				DX::IDXFileResourcePtr pTextureFile;
				if( DX::Error_Success != pFileSystem->AddResourceFile(wc1, wc2, &pTextureFile))
				{ DX::DestroyModel(aModel); OutputDebugString("Error on AddResourceFile"); return(false); }

				// Maybe do something with pTextureFile here in later versions...

				// If requested, delete the stand-alone texture file
				if (this->m_bDeleteTextures) ::DeleteFile(pMaterial->strTexture[0].c_str());
			}
	
		} // if (pMaterial)

	} // for (unsigned long lMaterial=0;lMaterial<ObjectData.GetNumMaterials();lMaterial++)


	// Backup: If no materials added, submit a standard one
	if (aModel->GetNumMaterials() == 0)
	{
		DX::IDXMaterial* mat;
		dxErr = aModel->CreateMaterial("std_mat_0", &mat);
		if (dxErr != DX::Error_Success) { DX::DestroyModel(aModel); OutputDebugString("Error on CreateMaterial2"); return(false); }
			
		// color argb
		DX::DXColor colAmb = DX::CreateColor(255, 150,150,150); 
		DX::DXColor colDif = DX::CreateColor(255, 150,150,150);  
			
		mat->SetAmbient(colAmb);
		mat->SetDiffuse(colDif);
		mat->SetSpecular(DX::CreateColor(255,5,5,5)); // a standard value
		mat->SetSpecularSharpness(50);

	} // if (aModel->GetNumMaterials() == 0)

	
	// -----------------------------------------------------------------------------
	// Second, create all meshes of the object 

	float fXCenterOffset=0.0f;
	float fZCenterOffset=0.0f;

	if (this->m_bCenterPatches) GetXZCenterOffset(ObjectData,fScale,iUpRotation,iFlipAxis,fXCenterOffset,fZCenterOffset);

	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		
		if (pMesh)
		{
			// A flag indicating that this mesh has no material, so we create one our own
			bool bNoMaterials = false;
			
			// Create mesh
			DX::IDXMesh* aMesh;               
			dxErr = aGroup->CreateMesh(&aMesh);
			if (dxErr != DX::Error_Success) { DX::DestroyModel(aModel); OutputDebugString("Error on CreateMesh"); return(false); }

			// Create all vertices of this mesh
			DWORD dwVertices=(DWORD)pMesh->GetNumVertices();
			
			for (unsigned long lVert=0;lVert<dwVertices;lVert++)
			{
				DX::DXVector vPosition;
				DX::DXVector vNormal = DX::CreateVector(0.0f,1.0f,0.0f); // dummy if no normals present
				DX::DXUV uv = DX::CreateUV(0.0f, 0.0f); // dummy if no UVs present
				
				// Calculate the position -----------------------------
				const PNPTC_VERTEX& vert = pMesh->GetVertex(lVert);
				PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y-m_fHeightOffset,vert.vPos.z);
				GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);
				
				vPosition = DX::CreateVector(vecPos.x+fXCenterOffset, vecPos.y, vecPos.z+fZCenterOffset);

				// Get the normal there (if any) -----------------------------
				if (pMesh->HasVertexNormals())
				{
					const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);
					PnPTC3DVec vecNormal(vert.vNormal.x,vert.vNormal.y,vert.vNormal.z);
					GlobalVectorTransform(vecNormal,1,iUpRotation,iFlipAxis);

					vNormal = DX::CreateVector(vecNormal.x, vecNormal.y, vecNormal.z);

				} // if (pMesh->HasVertexNormals())

				// Get the uv there, if any -----------------------------
				if (pMesh->GetNumMappingCoords()>0)
				{
					const PNPTC_VERTEX& vert=pMesh->GetVertex(lVert);

					uv = DX::CreateUV(vert.uv[0].u, vert.uv[0].v);
				} // if (pMesh->GetNumMappingCoords()>0)


				// Create the vertex in DXMeshAPI ----------------------------- 
				aMesh->AddVertex(vPosition, vNormal, 1, &uv); 

			} // for (unsigned long lVert=0;lVert<dwVertices;lVert++)

			// Assign material(s) to the mesh ----------------------------- 
			// Using two lists (mat names/mat indices) for later use
			
			std::vector<std::string> matNames;
			std::vector<int> matIndices;
						
			for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
			{
				const C3DFaceSet* pFaceSet = pMesh->GetFaceSet(lFaceSet);
				if (pFaceSet)
				{
					int iMaterial = pFaceSet->GetMaterial();
					if (iMaterial>=0 && iMaterial < static_cast<int>(ObjectData.GetNumMaterials()))
					{
						const PNPTC_MATERIAL* pMaterial = ObjectData.GetMaterial(iMaterial);
						if (pMaterial)
						{
							bool bFound = false;
							for (unsigned long lMat=0;lMat<matNames.size();lMat++)
							{
								if (strcmp(matNames[lMat].c_str(),MakeCompatibleString(pMaterial->strName).c_str())==0)
								{
									bFound=true;
									break;
								}
							}
							if (!bFound)
							{
								// add material to list
								matNames.push_back(MakeCompatibleString(pMaterial->strName).c_str());
							}
	
						} // if (pMaterial)
					
					} // if (iMaterial>=0 && iMaterial < static_cast<int>(ObjectData.GetNumMaterials()))

				} // if (pFaceSet)

			} // for (unsigned long lFaceSet=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)

			for (unsigned long lMat=0;lMat<matNames.size();lMat++)
			{
				UINT iMatIndex;
				aMesh->AddMaterialMap(matNames[lMat].c_str(), true, &iMatIndex);
				matIndices.push_back(iMatIndex);
			}
			
			// Backup: Use the standard material if none submitted above
			if (matNames.size() == 0)
			{
				aMesh->AddMaterialMap("std_mat_0", false);
				bNoMaterials = true;
			}

			// Assign faces to the mesh ----------------------------- 
			DWORD dwFaces = static_cast<DWORD>(pMesh->GetNumFaces());
			UINT iMaterialIndex = -1;

			for (unsigned long lFaceSet=0,lFaceCount=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
			{
				const C3DFaceSet* pFaceSet=pMesh->GetFaceSet(lFaceSet);
				if (pFaceSet)
				{
					// Get material for this face set and look up DXMeshAPI index
					iMaterialIndex = pFaceSet->GetMaterial();
					if (iMaterialIndex>=0 && iMaterialIndex<static_cast<int>(ObjectData.GetNumMaterials()))
					{
						const PNPTC_MATERIAL* pMaterial=ObjectData.GetMaterial(iMaterialIndex);
						if (pMaterial)
						{
							for (unsigned long lMat=0;lMat<matNames.size();lMat++)
							{
								if (strcmp(matNames[lMat].c_str(),MakeCompatibleString(pMaterial->strName).c_str())==0)
								{
									iMaterialIndex = matIndices[(int)lMat];
									break;
								}
							}

						} // if (pMaterial)

					} // if (iMaterialIndex>=0 && iMaterialIndex<static_cast<int>(ObjectData.GetNumMaterials())) 
					
					// Backup, if no material originally present
					if (bNoMaterials) iMaterialIndex = 0;
				
					for (unsigned long lFace=0;lFace<pFaceSet->GetNumFaces();lFace++,lFaceCount++)
					{
						const PNPTC_FACE& face = pFaceSet->GetFace(lFace);
						aMesh->AddFace(face.dwVert[0],face.dwVert[1],face.dwVert[2], iMaterialIndex); 
					}

				} // if (pFaceSet)

			} // for (unsigned long lFaceSet=0,lFaceCount=0;lFaceSet<pMesh->GetNumFaceSets();lFaceSet++)
	
		} // if (pMesh) 

	} // for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)

	// -----------------------------------------------------------------------------
	// Third, save the file (textures will be included automatically, if set) 

	// Check if file is already there. Delete, if so
	DWORD attrib = GetFileAttributes(strFileName.c_str());
	if (attrib != 0xFFFFFFFF) ::DeleteFile(strFileName.c_str());
	
	// New with 1.4: Optimize mesh
	aModel->Optimize();

	// Now, save
	wchar_t wc[500];
	swprintf(wc,L"%S",strFileName.c_str());
	dxErr = aModel->SaveToFile(wc);
	if (dxErr != DX::Error_Success) { DX::DestroyModel(aModel); OutputDebugString("Error on SaveToFile"); return(false); }

	// -----------------------------------------------------------------------------
	// Fourth, clean up: get rid of the model explicitely
	DX::DestroyModel(aModel);

	return(true);	
}

std::string C3DObjectSerializerDXStudio::MakeCompatibleString(const std::string& strInput)
{
	// initialize return string
	std::string strReturn(strInput);

	// Replace points. DXMeshAPI doesn't seem to like it in tags
	Replace(&strReturn,".","_");
	// Replace blanks
	Replace(&strReturn," ","_");
	// Replace +
	Replace(&strReturn,"+","_");
	// Replace :
	Replace(&strReturn,":","_");
	// Replace ;
	Replace(&strReturn,";","_");
	// Replace /
	Replace(&strReturn,"/","_");
	// Replace \ 
	Replace(&strReturn,"\\","_");

	// ensure not to start with a digit
	if (strReturn.length()>0)
	{
		if (strReturn.at(0)>='0' && strReturn.at(0)<='9')
		{
			strReturn=std::string("_")+strReturn;
		}
	}

	// return string
	return(strReturn);
}

// Helper function that calculates a xy-offset to apply when the terrain shall be centered
// at 0,0. Would only work if there are no other objects exceeding the terrain vertices in xy
bool C3DObjectSerializerDXStudio::GetXZCenterOffset(const C3DObjectData& ObjectData, float fScale, int iUpRotation, int iFlipAxis, float& fXCenterOffset, float& fZCenterOffset)
{
	// Move through all vertices, get min/max and calculate an offset to center
	// all meshes around (0,0) 
	float fXmin = 10000; float fXmax = -10000;
	float fZmin = 10000; float fZmax = -10000;
	
	for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)
	{
		const C3DMesh* pMesh=ObjectData.GetMesh(lMesh);
		
		if (pMesh)
		{
			DWORD dwVertices=(DWORD)pMesh->GetNumVertices();
			
			for (unsigned long lVert=0;lVert<dwVertices;lVert++)
			{
				const PNPTC_VERTEX& vert = pMesh->GetVertex(lVert);
				PnPTC3DVec vecPos(vert.vPos.x,vert.vPos.y,vert.vPos.z);
				GlobalVectorTransform(vecPos,fScale,iUpRotation,iFlipAxis);

				if (vecPos.x<fXmin) fXmin=vecPos.x;
				else if(vecPos.x>fXmax) fXmax=vecPos.x;
				
				if (vecPos.z<fZmin) fZmin=vecPos.z;
				else if(vecPos.z>fZmax) fZmax=vecPos.z;

			} // for (unsigned long lVert=0;lVert<dwVertices;lVert++)

		} // if (pMesh) 

	} // for (unsigned long lMesh=0;lMesh<ObjectData.GetNumMeshes();lMesh++)

	fXCenterOffset = -(fXmax+fXmin)/2.0f;
	fZCenterOffset = -(fZmax+fZmin)/2.0f;

	return(true);
}



