/////////////////////////////////////////////////////////////////////////////////////
// 3DObjectSerializerDXStudio.h
// 3d object serialization class for .dxmesh files.
// ----------------------------------------------------------------------------------
// author:		miko, based on pnpTC demos
// ----------------------------------------------------------------------------------
// change history:
//   xx. aaa. 2009 : miko
// ----------------------------------------------------------------------------------
// copyright notice:
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_3DOBJECTSERIALIZERDXSTUDIO_H_INCLUDED)
#define PNPTC_3DOBJECTSERIALIZERDXSTUDIO_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// include base class
#include "I3DObjectSerializer.h"


/// 3D Object serialization class for .x files.
/// Reads/writes objects/meshes to/from an .x file. 
class C3DObjectSerializerDXStudio : public I3DObjectSerializer
{
public:

	// -----------------------------------------------------------------
	// construction / destruction
	// -----------------------------------------------------------------

	/// Constructs a new serializer.
	C3DObjectSerializerDXStudio(void);

	/// Destruction of the serializer.
	~C3DObjectSerializerDXStudio(void);


	// -----------------------------------------------------------------
	// file options
	// -----------------------------------------------------------------

	/// Sets special to "center all patches".
	/// @param bCenterPatches
	void C3DObjectSerializerDXStudio::SetCenterPatches(bool bCenterPatches) {this->m_bCenterPatches=bCenterPatches;};
	void C3DObjectSerializerDXStudio::SetDeleteTextures(bool bDeleteTextures) {this->m_bDeleteTextures=bDeleteTextures;};
	void C3DObjectSerializerDXStudio::SetHeightOffset(float fHeightOffset) {this->m_fHeightOffset=fHeightOffset;};
	bool GetXZCenterOffset(const C3DObjectData& ObjectData, float fScale, int iUpRotation, int iFlipAxis, float& fXCenterOffset, float& fZCenterOffset);

	// -----------------------------------------------------------------
	// methods to load/save objects
	// -----------------------------------------------------------------

	/// Loads an object from a file.
	/// @param ObjectData 3D Object data into which to load the file.
	/// @param strFileName Object file (including path) to be loaded.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be loaded successfully.
	bool	Load(C3DObjectData& ObjectData,const std::string& strFileName);

	/// Writes an object to a file
	/// @param ObjectData 3D Object data to be saved to the file.
	/// @param strFileName Object file (including path) to be saved to.
	/// @param fScale A scaling factor, which will be applied to all vertex
	/// positions during export.
	/// @param iUpRotation A rotation, which will be applied to all vertices
	/// in order to correclty align the object's up axis. Possible values are
	/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
	/// @param iFlipAxis The axis along which all vertex positions are negated
	/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
	/// @param cBasePathName (optional) The base path name of the export. If specified,
	/// all further file references have to be specified with respect to this path.
	/// For example, if a texture file has to be saved, the location of the texture
	/// file in the object's material definition will be given with respect to this
	/// base path. If this parameter is not specified (<i>NULL</i>), only the file
	/// name of the texture will be used, without any path references.
	/// @param bSeparateMeshFiles Whether or not to generate seperate files for the 
	/// meshes of the object.
	/// @param bExportTextures Whether or not the textures of the mesh should be exported.
	/// @param cSourceTexturePath The path, below which the source texture files
	/// of the mesh to be exported are located.
	/// @param pD3DDevice Pointer to the rendering device of the object and its textures
	/// @return Returns <i>true</i> or <i>false</i>, whether or not
	/// the object could be saved successfully.
	bool	Save(const C3DObjectData& ObjectData,const std::string& strFileName,float fScale=1,int iUpRotation=0,int iFlipAxis=0,const char* cBasePathName=NULL,bool bSeparateMeshFiles=false,bool bExportTextures=false,const char* cSourceTexturePath=NULL,IDirect3DDevice9* pD3DDevice=NULL);


private:

	/// Helper routine to check for strings the DXMeshAPI would not accept
	std::string MakeCompatibleString(const std::string& strInput);

	// -----------------------------------------------------------------
	// member variables
	// -----------------------------------------------------------------

	/// Flag to store if all patches shall be centered
	bool	m_bCenterPatches;
	bool    m_bDeleteTextures;

	// An optional height offset that is used to move the lowest terrain vertex to height 0
	float m_fHeightOffset;

}; // class C3DObjectSerializerDXStudio

#endif // !defined(PNPTC_3DOBJECTSERIALIZERDXSTUDIO_H_INCLUDED)