/////////////////////////////////////////////////////////////////////////////////////
// TextureResourceManager.cpp
// Texture resource manager implementation.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  21. Aug. 2007 :	Added LoadTexture().
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include ".\textureresourcemanager.h"
#include "../PnPTCUtils/Utils.h"


CTextureResourceManager::CTextureResourceManager(void)
{
}

CTextureResourceManager::~CTextureResourceManager(void)
{
	// release everything thats not released so far
	for (TextureList::iterator itTex=this->m_vecTextures.begin();itTex!=this->m_vecTextures.end();itTex++)
	{
		if (itTex->pTexture!=NULL)
		{
			while (itTex->pTexture->Release()>0) ;
		}
	}
}

LPDIRECT3DTEXTURE9 CTextureResourceManager::LoadTexture(IDirect3DDevice9* pD3DDevice,const std::string& strFileName,bool bCompressed/*=false*/)
{
	// create the texture
	LPDIRECT3DTEXTURE9 pTexture=NULL;
	if (FAILED(D3DXCreateTextureFromFileEx(	pD3DDevice,
											strFileName.c_str(),
											(bCompressed)?D3DX_DEFAULT:D3DX_DEFAULT_NONPOW2,
											(bCompressed)?D3DX_DEFAULT:D3DX_DEFAULT_NONPOW2,
											D3DX_DEFAULT,0,
											(bCompressed)?D3DFMT_DXT5:D3DFMT_FROM_FILE,
											D3DPOOL_DEFAULT,
											D3DX_DEFAULT,D3DX_DEFAULT,
											0xFF000000,
											NULL,NULL,
											&pTexture)))
	{
		pTexture=NULL;
	}

	// return the texture
	return pTexture;
}

LPDIRECT3DTEXTURE9 CTextureResourceManager::CreateTexture(IDirect3DDevice9* pD3DDevice,const std::string& strFileName)
{
	// check input
	if (!pD3DDevice || strFileName.length()==0)
	{
		return NULL;
	}

	// try to find this texture in the list of already created textures
	for (TextureList::iterator itTex=this->m_vecTextures.begin();itTex!=this->m_vecTextures.end();itTex++)
	{
		if (EqualNoCase(itTex->strFileName,strFileName))
		{
			// this is the texture => increment reference count and return pointer
			if (itTex->pTexture)
			{
				itTex->pTexture->AddRef();
			}
			return itTex->pTexture;
		}
	}

	// texture not found => load it
	LPDIRECT3DTEXTURE9 pTexture=CTextureResourceManager::LoadTexture(pD3DDevice,strFileName);
	if (pTexture)
	{
		// add texture to texture list
		TEXTURE_RESOURCE trNew;
		trNew.strFileName=strFileName;
		trNew.pTexture=pTexture;
		this->m_vecTextures.push_back(trNew);
	}

	// return newly created texture
	return pTexture;
}

void CTextureResourceManager::ReleaseTexture(IDirect3DTexture9* pTexture)
{
	// check input
	if (!pTexture)
	{
		return;
	}

	// try to find that texture
	for (TextureList::iterator itTex=this->m_vecTextures.begin();itTex!=this->m_vecTextures.end();itTex++)
	{
		if (itTex->pTexture==pTexture)
		{
			if (itTex->pTexture->Release()==0)
			{
				// no more references
				this->m_vecTextures.erase(itTex);
				return;
			}
		}
	}
}
