/////////////////////////////////////////////////////////////////////////////////////
// TextureResourceManager.h
// Texture resource manager definition.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  21. Aug. 2007 :	Added LoadTexture().
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#pragma once

// direct-x includes
#include <d3dx9.h>

// standard includes
#include <string>
#include <vector>


/// description of a texture resource
typedef struct TEXTURE_RESOURCE
{
	std::string			strFileName;		//!< path and file name of the texture
	IDirect3DTexture9*	pTexture;			//!< the texture object for this resource
} TEXTURE_RESOURCE;

/// texture list
typedef std::vector<TEXTURE_RESOURCE>	TextureList;


/// texture resource management class
class CTextureResourceManager
{
public:
	/// construction
	CTextureResourceManager(void);
	/// destruction
	~CTextureResourceManager(void);


	/// Create a new texture resource from a given texture file.
	/// If a texture object from the same file object already exists,
	/// the same object will be returned, only the reference counter
	/// will be increased. Therefore it is necessary to release
	/// textures which are no longer required by a call to the
	/// ReleaseTexture() method.
	/// @param pD3DDevice Pointer to the rendering device for which
	/// the texture should be created.
	/// @param strFileName name of the file to create the texture from.
	/// @return returns a pointer to the created texture resource if
	/// successfull or <i>NULL</i> in case of an error.
	LPDIRECT3DTEXTURE9			CreateTexture(IDirect3DDevice9* pD3DDevice,const std::string& strFileName);

	/// Releases a texture object, previously created by the
	/// CreateTexture() method. If no other references to this
	/// texture object exist, it will be deleted.
	/// @param pTexture Pointer to the texture object to be deleted.
	void						ReleaseTexture(IDirect3DTexture9* pTexture);

	/// Loads a texture from a file. The loaded texture is 
	/// not managed by a texture resource manager.
	/// @param pD3DDevice Pointer to the rendering device for which
	/// the texture should be created.
	/// @param strFileName name of the file to create the texture from.
	/// @param bCompressed Whether or not the texture should be loaded compressed.
	/// @return returns a pointer to the created texture resource if
	/// successfull or <i>NULL</i> in case of an error.
	static LPDIRECT3DTEXTURE9	LoadTexture(IDirect3DDevice9* pD3DDevice,const std::string& strFileName,bool bCompressed=false);


private:

	// ---------------------------------------------------------------------------
	// member variables
	// ---------------------------------------------------------------------------

	/// list of all textures
	TextureList			m_vecTextures;

};
