/////////////////////////////////////////////////////////////////////////////////////
// DXStudioFileFormatDlg.cpp
// *.dxmesh file format specification dialog.
// ----------------------------------------------------------------------------------
// author:		miko, based on pnpTC demos
// ----------------------------------------------------------------------------------
// change history:
//   v1.2  2009/5/9: first release in new design /miko
//   v1.3  2009/5/12: fixed "delete textures could be ON when exporting detail textures"
// ----------------------------------------------------------------------------------
// copyright notice:
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PnPTCDXStudioFileHMapExportPlugIn.h"
#include ".\dxstudiofileformatdlg.h"



IMPLEMENT_DYNAMIC(DXStudioFileFormatDlg, CDialog)
DXStudioFileFormatDlg::DXStudioFileFormatDlg(CWnd* pParent /*=NULL*/)
	: CDialog(DXStudioFileFormatDlg::IDD, pParent)
{
	// initializations
	this->m_fScaleFactor=1.f;
	this->m_iPatchSize=1024;
	this->m_iMaxPatchSize=1024;
	this->m_iTextureExport=0;
	this->m_iTextureFormat=0;
	this->m_bCenterPatches=0;
	this->m_iUpAxis=0;
	this->m_bGlobalPositioning=false;
	this->m_bCenterPatches=false;
	this->m_strLightMap="";
	this->m_bDeleteTextures=false;
	this->m_bHeightAdaption=false;
	this->m_iReductionMethod=0;
	this->m_iGridSize=1;
	this->m_fErrorValue=0;
	this->m_iTextureMapSize=-1;
	this->m_bSimplifyBorder=true;
}

DXStudioFileFormatDlg::~DXStudioFileFormatDlg()
{
}

void DXStudioFileFormatDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_CHK_CENTERPATCHES, m_chkCenterPatches);
	DDX_Control(pDX, IDC_COMBO_PATCHSIZE, m_combPatchSize);
	DDX_Control(pDX, IDC_COMBO_TEXTUREEXPORT, m_combTextureExport);
	DDX_Control(pDX, IDC_COMBO_TEXTUREFORMAT, m_combTextureFormat);
	DDX_Control(pDX, IDC_TXT_SCALEFACTOR, m_txtFactor);
	DDX_Control(pDX, IDC_COMBO_UPAXIS, m_combUpAxis);
	DDX_Control(pDX, IDC_CHK_GLOBALPOSITIONING, m_chkGlobalPositioning);
	DDX_Control(pDX, IDC_COMBO_LIGHTMAP, m_combLightMap);
	DDX_Control(pDX, IDC_CHK_DELTEXTURES, m_chkDeleteTextures);
	DDX_Control(pDX, IDC_CHK_HEIGHTADAPTION, m_chkHeightAdaption);
	DDX_Control(pDX, IDC_CHK_SIMPLIFYBORDER, m_chkSimplifyBorder);
	DDX_Control(pDX, IDC_COMBO_TEXTUREMAPSIZE, m_combTextureMapSize);
	DDX_Control(pDX, IDC_COMBO_REDUCTIONMETHOD, m_combReductionMethod);
	DDX_Control(pDX, IDC_COMBO_GRIDSIZE, m_combGridSize);
	DDX_Control(pDX, IDC_TXT_ERRORVALUE, m_txtErrorValue);
	DDX_Control(pDX, IDC_DEL_TEXT, m_ctrlDeleteTexturesText);
}


BEGIN_MESSAGE_MAP(DXStudioFileFormatDlg, CDialog)
	ON_CBN_SELCHANGE(IDC_COMBO_TEXTUREEXPORT, OnCbnSelchangeComboTextureexport)
	ON_CBN_SELCHANGE(IDC_COMBO_REDUCTIONMETHOD, OnCbnSelchangeComboReductionmethod)
	ON_CBN_SELCHANGE(IDC_COMBO_PATCHSIZE, OnCbnSelchangeComboPatchsize)
	ON_BN_CLICKED(IDC_CHK_GLOBALPOSITIONING, OnBnClickedChkGlobalpositioning)
	ON_BN_CLICKED(IDC_CHK_CENTERPATCHES, OnBnClickedChkCenterpatches)
END_MESSAGE_MAP()



BOOL DXStudioFileFormatDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// initialize text boxes
	this->m_txtFactor.SetAllowNegatives(false);
	this->m_txtFactor.SetAllowDecimals(true);
	this->m_txtFactor.SetValue(this->m_fScaleFactor);

	// fill patch and texture map size combo
	this->m_combPatchSize.ResetContent();
	this->m_combTextureMapSize.ResetContent();
	int iNewIndex=this->m_combTextureMapSize.AddString("Use patch size");
	if (iNewIndex>=0)
	{
		this->m_combTextureMapSize.SetItemData(iNewIndex,-1);
	}
	for (int iPatchSize=64;iPatchSize<=this->m_iMaxPatchSize;iPatchSize*=2)
	{
		CString str;
		str.Format("%d",iPatchSize);
		iNewIndex=this->m_combPatchSize.AddString(str);
		if (iNewIndex>=0)
		{
			this->m_combPatchSize.SetItemData(iNewIndex,iPatchSize);
		}
		iNewIndex=this->m_combTextureMapSize.AddString(str);
		if (iNewIndex>=0)
		{
			this->m_combTextureMapSize.SetItemData(iNewIndex,iPatchSize);
		}
	}

	// initialize combos
	CString str;
	str.Format("%d",this->m_iPatchSize);
	this->m_combPatchSize.SelectString(-1,str);
	this->m_combTextureExport.SetCurSel(this->m_iTextureExport);
	this->m_combTextureFormat.SetCurSel(this->m_iTextureFormat);
	this->m_combUpAxis.SetCurSel(this->m_iUpAxis);
	if (this->m_iTextureMapSize==-1)
	{
		this->m_combTextureMapSize.SetCurSel(0);
	}
	else
	{
		str.Format("%d",this->m_iTextureMapSize);
		this->m_combTextureMapSize.SelectString(-1,str);
	}
	this->m_combGridSize.AddString("1");
	this->m_combGridSize.AddString("2");
	this->m_combGridSize.AddString("4");
	this->m_combGridSize.AddString("8");
	this->m_combGridSize.AddString("16");
	this->m_combGridSize.AddString("32");
	this->FillReductionMethodCombo();
	this->FillTextureMapSizeCombo();

	// initialize light map combo name
	this->m_combLightMap.ResetContent();
	this->m_combLightMap.AddString("(No lightmap)");
	for (unsigned long n=0;n<this->m_vecLightMapNames.size();n++)
	{
		this->m_combLightMap.AddString(this->m_vecLightMapNames[n].c_str());
	}
	this->m_combLightMap.SelectString(-1,this->m_strLightMap);
	if (this->m_combLightMap.GetCurSel()<0) this->m_combLightMap.SelectString(-1,"(No lightmap)");
	if (this->m_combLightMap.GetCurSel()<0) this->m_combLightMap.SetCurSel(0);

	// initialize check boxes
	this->m_chkGlobalPositioning.SetCheck((this->m_bGlobalPositioning)?BST_CHECKED:BST_UNCHECKED);
	this->m_chkDeleteTextures.SetCheck((this->m_bDeleteTextures)?BST_CHECKED:BST_UNCHECKED);
	this->m_chkSimplifyBorder.SetCheck((this->m_bSimplifyBorder)?BST_CHECKED:BST_UNCHECKED);
	this->m_chkCenterPatches.SetCheck((this->m_bCenterPatches)?BST_CHECKED:BST_UNCHECKED);
	this->m_chkHeightAdaption.SetCheck((this->m_bHeightAdaption)?BST_CHECKED:BST_UNCHECKED);

	// set reduction values
	str.Format("%d",this->m_iGridSize);
	this->m_combGridSize.SelectString(-1,str);
	for (int i=0;i<this->m_combReductionMethod.GetCount();++i)
	{
		if (this->m_combReductionMethod.GetItemData(i)==this->m_iReductionMethod)
		{
			this->m_combReductionMethod.SetCurSel(i);
			break;
		}
	}
	this->m_txtErrorValue.SetValue(this->m_fErrorValue);

	// ensure anything is selected
	if (this->m_combPatchSize.GetCurSel()<0) this->m_combPatchSize.SetCurSel(0);
	if (this->m_combTextureExport.GetCurSel()<0) this->m_combTextureExport.SetCurSel(0);
	if (this->m_combTextureFormat.GetCurSel()<0) this->m_combTextureFormat.SetCurSel(0);
	if (this->m_combUpAxis.GetCurSel()<0) this->m_combUpAxis.SetCurSel(0);
	if (this->m_combTextureMapSize.GetCurSel()<0) this->m_combTextureMapSize.SetCurSel(0);
	if (this->m_combGridSize.GetCurSel()<0) this->m_combGridSize.SetCurSel(0);
	if (this->m_combReductionMethod.GetCurSel()<0) this->m_combReductionMethod.SetCurSel(0);

	// enable texture and reduction export controls
	this->EnableTextureControls();
	this->EnableReductionControls();

	// Check if the positioning checkboxes need to be activeated
	OnBnClickedChkGlobalpositioning();
	OnBnClickedChkCenterpatches();

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void DXStudioFileFormatDlg::OnOK()
{
	// get patch size
	int iSelected=this->m_combPatchSize.GetCurSel();
	if (iSelected>=0)
	{
		this->m_iPatchSize=static_cast<int>(this->m_combPatchSize.GetItemData(iSelected));
	}

	// read combo values
	this->m_iTextureExport=this->m_combTextureExport.GetCurSel();
	this->m_iTextureFormat=this->m_combTextureFormat.GetCurSel();
	this->m_iUpAxis=this->m_combUpAxis.GetCurSel();

	// get texture map size
	iSelected=this->m_combTextureMapSize.GetCurSel();
	if (iSelected>=0)
	{
		this->m_iTextureMapSize=static_cast<int>(this->m_combTextureMapSize.GetItemData(iSelected));
	}

	// get light map name
	this->m_combLightMap.GetWindowText(this->m_strLightMap);
	if (this->m_strLightMap.CompareNoCase("(No lightmap)")==0) this->m_strLightMap="";

	// read check boxes
	this->m_bGlobalPositioning=(this->m_chkGlobalPositioning.GetCheck()==BST_CHECKED);
	this->m_bDeleteTextures=(this->m_chkDeleteTextures.GetCheck()==BST_CHECKED);
	this->m_bSimplifyBorder=(this->m_chkSimplifyBorder.GetCheck()==BST_CHECKED);
	this->m_bCenterPatches=(this->m_chkCenterPatches.GetCheck()==BST_CHECKED);
	this->m_bHeightAdaption=(this->m_chkHeightAdaption.GetCheck()==BST_CHECKED);

	// get reduction values
	iSelected=this->m_combReductionMethod.GetCurSel();
	if (iSelected>=0)
	{
		this->m_iReductionMethod=static_cast<int>(this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel()));
	}
	iSelected=this->m_combGridSize.GetCurSel();
	if (iSelected>=0)
	{
		CString strSelected;
		this->m_combGridSize.GetWindowText(strSelected);
		this->m_iGridSize=atoi((LPCSTR)strSelected);
	}
	this->m_fErrorValue=this->m_txtErrorValue.GetValue();

	// get scale factor
	this->m_fScaleFactor=this->m_txtFactor.GetValue();

	// close dialog
	CDialog::OnOK();
}

void DXStudioFileFormatDlg::EnableTextureControls()
{
	// controls are only enabled, if texture export is active
	this->m_combTextureFormat.EnableWindow(this->m_combTextureExport.GetCurSel()>=1);
	this->m_combLightMap.EnableWindow(this->m_combTextureExport.GetCurSel()==2);
	this->m_chkDeleteTextures.EnableWindow(this->m_combTextureExport.GetCurSel()==2);
	this->m_ctrlDeleteTexturesText.EnableWindow(this->m_combTextureExport.GetCurSel()==2);
	this->m_combTextureMapSize.EnableWindow(this->m_combTextureExport.GetCurSel()==2);

	// Added with v1.3 to have "Delete textures" always OFF when exporting "Detail textures"
	if (this->m_combTextureExport.GetCurSel()==1) this->m_chkDeleteTextures.SetCheck(0); 
}

void DXStudioFileFormatDlg::EnableReductionControls()
{
	// enable vertex reduction controls
	this->m_combGridSize.EnableWindow(this->m_combReductionMethod.GetCurSel()>=0 && this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel())==1);
	this->m_txtErrorValue.EnableWindow(this->m_combReductionMethod.GetCurSel()>=0 && this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel())==2);
	this->m_chkSimplifyBorder.EnableWindow(this->m_combReductionMethod.GetCurSel()>=0 && this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel())==2);
}

void DXStudioFileFormatDlg::OnCbnSelchangeComboTextureexport()
{
	// enable texture export controls
	this->EnableTextureControls();

	// regular reduction is not possible for detail blended textures
	CString strReductionMethod;
	this->m_combReductionMethod.GetWindowText(strReductionMethod);
	this->FillReductionMethodCombo();
	this->m_combReductionMethod.SelectString(-1,strReductionMethod);
	if (this->m_combReductionMethod.GetCurSel()<0)
	{
		this->m_combReductionMethod.SetCurSel(0);
		this->EnableReductionControls();
	}
}

void DXStudioFileFormatDlg::OnCbnSelchangeComboReductionmethod()
{
	// enable reduction controls
	this->EnableReductionControls();
}

void DXStudioFileFormatDlg::OnCbnSelchangeComboPatchsize()
{
	// refill texture map size combo
	this->FillTextureMapSizeCombo();
}

void DXStudioFileFormatDlg::FillReductionMethodCombo()
{
	// fill reduction methods
	this->m_combReductionMethod.ResetContent();
	int iNewItem=this->m_combReductionMethod.AddString("no reduction");
	if (iNewItem>=0) this->m_combReductionMethod.SetItemData(iNewItem,0);
	if (this->m_combTextureExport.GetCurSel()!=1)
	{
		iNewItem=this->m_combReductionMethod.AddString("regular");
		if (iNewItem>=0) this->m_combReductionMethod.SetItemData(iNewItem,1);
	}
	iNewItem=this->m_combReductionMethod.AddString("height deviation");
	if (iNewItem>=0) this->m_combReductionMethod.SetItemData(iNewItem,2);
}

void DXStudioFileFormatDlg::FillTextureMapSizeCombo()
{
	// get current patch size
	int iPatchSize=this->m_iMaxPatchSize;
	if (this->m_combPatchSize.GetCurSel()>=0)
	{
		iPatchSize=static_cast<int>(this->m_combPatchSize.GetItemData(this->m_combPatchSize.GetCurSel()));
	}

	// refill texture map size combo
	int iLastSelection=this->m_combTextureMapSize.GetCurSel();
	this->m_combTextureMapSize.ResetContent();
	int iNewIndex=this->m_combTextureMapSize.AddString("Use patch size");
	if (iNewIndex>=0)
	{
		this->m_combTextureMapSize.SetItemData(iNewIndex,-1);
	}
	for (int iTextureMapSize=64;iTextureMapSize<=iPatchSize;iTextureMapSize*=2)
	{
		CString str;
		str.Format("%d",iTextureMapSize);
		iNewIndex=this->m_combTextureMapSize.AddString(str);
		if (iNewIndex>=0)
		{
			this->m_combTextureMapSize.SetItemData(iNewIndex,iTextureMapSize);
		}
	}

	// restore selection
	this->m_combTextureMapSize.SetCurSel(iLastSelection);
	if (this->m_combTextureMapSize.GetCurSel()<0) this->m_combTextureMapSize.SetCurSel(0);
}


void DXStudioFileFormatDlg::OnBnClickedChkGlobalpositioning()
{
	if (this->m_chkGlobalPositioning.GetCheck()!=0)
		this->m_chkCenterPatches.EnableWindow(false);
	else 
		this->m_chkCenterPatches.EnableWindow(true);
}

void DXStudioFileFormatDlg::OnBnClickedChkCenterpatches()
{
	if (this->m_chkCenterPatches.GetCheck()!=0)
		this->m_chkGlobalPositioning.EnableWindow(false);
	else 
		this->m_chkGlobalPositioning.EnableWindow(true);

}
