/////////////////////////////////////////////////////////////////////////////////////
// PnPTCDXStudioFileHMapExportPlugIn.cpp
// *.dxmesh file heightmap exporter PlugIn implementation.
// ----------------------------------------------------------------------------------
// author:		miko, based on pnpTC demos
// ----------------------------------------------------------------------------------
// change history:
//   xx. aaa. 2009 : miko 
// ----------------------------------------------------------------------------------
// copyright notice:
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PnPTCDXStudioFileHMapExportPlugIn.h"
#include "DXStudioFileFormatDlg.h"
#include "../pnptc3dobject/3dobjectserializerdxstudio.h"
#include "../pnptcutils/progressbar.h"


// --------------------------------------------------------------------------------------------
// Plugin global definitions
// --------------------------------------------------------------------------------------------

/// Default name of this PlugIn module.
#define MODULE_NAME "PnPTCDXStudioFileHMapExportPlugIn"

/// The name of this plugin.<br>
/// Try to give your plugins an unique name.
#define	PLUGIN_NAME	"Heightmap to *.dxmesh file export (1.4/miko)"

/// The unique identifier (GUID) of this plugin
/// Create your own uid for your plugins using
/// the guidgen-utility of VisualC or any other creation utility.
// {5F5F4142-9FAF-4138-BD05-95D80D839435}
static const GUID g_uidPlugIn = { 0x5f5f4142, 0x9faf, 0x4138, { 0xbd, 0x5, 0x95, 0xd8, 0xd, 0x83, 0x94, 0x35 } };


// --------------------------------------------------------------------------------------------
// Get informations of the plugin
// --------------------------------------------------------------------------------------------

/// Returns the API-ID used for this plugin.
/// @return ID of PlugIn-API.
__declspec(dllexport) int CALLBACK GetPlugInApiID()
{
	// return generator ID
	return PNPTC_PLUGIN_APIID_EXPORT;
}

/// Returns the name of this plugin.
/// @return Null-terminated character string containing
/// the plugin's name.
__declspec(dllexport) char* CALLBACK GetPlugInName()
{
	// return name
	return PLUGIN_NAME;
}

/// Get a unique identifier for this plugin.
/// @return The plugin's uid.
__declspec(dllexport) GUID CALLBACK GetPlugInGUID()
{
	// return the uid of your plugin
	return g_uidPlugIn;
}


// --------------------------------------------------------------------------------------------
// Creation of the plugin instance
// --------------------------------------------------------------------------------------------

/// Get an instance of the plugin
/// @return Returns a pointer to the create plugin.<br>
/// Return <i>NULL</i> if the plugin could not be created.
__declspec(dllexport) IPnPTCPlugIn* CALLBACK CreatePlugin()
{
	// create and return the plugin pointer
	return new CPnPTCDXStudioFileHMapExportPlugIn;
}

/// Free the plugin instance.
/// @param plugin Instance to be freed.
__declspec(dllexport) void CALLBACK DestroyPlugin(IPnPTCPlugIn* plugin)
{
	// deletes the plugin
	if (plugin)
	{
		delete (CPnPTCDXStudioFileHMapExportPlugIn*)plugin;
	}
}


// --------------------------------------------------------------------------------------------
// The application object
// --------------------------------------------------------------------------------------------


//
//	Note!
//
//		If this DLL is dynamically linked against the MFC
//		DLLs, any functions exported from this DLL which
//		call into MFC must have the AFX_MANAGE_STATE macro
//		added at the very beginning of the function.
//
//		For example:
//
//		extern "C" BOOL PASCAL EXPORT ExportedFunction()
//		{
//			AFX_MANAGE_STATE(AfxGetStaticModuleState());
//			// normal function body here
//		}
//
//		It is very important that this macro appear in each
//		function, prior to any calls into MFC.  This means that
//		it must appear as the first statement within the 
//		function, even before any object variable declarations
//		as their constructors may generate calls into the MFC
//		DLL.
//
//		Please see MFC Technical Notes 33 and 58 for additional
//		details.
//

// CPnPTCDXStudioFileHMapExportPlugInApp

BEGIN_MESSAGE_MAP(CPnPTCDXStudioFileHMapExportPlugInApp, CWinApp)
END_MESSAGE_MAP()


// CPnPTCDXStudioFileHMapExportPlugInApp construction

CPnPTCDXStudioFileHMapExportPlugInApp::CPnPTCDXStudioFileHMapExportPlugInApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CPnPTCDXStudioFileHMapExportPlugInApp object
CPnPTCDXStudioFileHMapExportPlugInApp theApp;


// CPnPTCDXStudioFileHMapExportPlugInApp initialization

BOOL CPnPTCDXStudioFileHMapExportPlugInApp::InitInstance()
{
	CWinApp::InitInstance();

	// get ini file name
	char cModuleFileName[MAX_PATH+1];
	::GetModuleFileName(theApp.m_hInstance,cModuleFileName,MAX_PATH);
	std::string strModuleFileName(cModuleFileName);
	std::string	strPlugInPath=GetPath(strModuleFileName);
	this->m_strIniFile=strPlugInPath+"PlugIns.ini";

	return TRUE;
}




// --------------------------------------------------------------------------------------------
// Exporter PlugIn
// --------------------------------------------------------------------------------------------

CPnPTCDXStudioFileHMapExportPlugIn::CPnPTCDXStudioFileHMapExportPlugIn()
{
	// constructor => nothing to do
	return;
}

CPnPTCDXStudioFileHMapExportPlugIn::~CPnPTCDXStudioFileHMapExportPlugIn()
{
	// destructor => nothing to do
	return;
}

const char* CPnPTCDXStudioFileHMapExportPlugIn::GetName()
{
	// return plugin name
	return PLUGIN_NAME;
}

GUID CPnPTCDXStudioFileHMapExportPlugIn::GetGUID()
{
	// return guid of plugin
	return g_uidPlugIn;
}

bool CPnPTCDXStudioFileHMapExportPlugIn::ExportSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain)
{
	// MFC needs this
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// read settings
	static char cTemp[1000];
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ScaleFactor","1",cTemp,999,theApp.m_strIniFile.c_str());
	float fScaleFactor=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"PatchSize","128",cTemp,999,theApp.m_strIniFile.c_str());
	int iPatchSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureExport","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureExport=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureFormat","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureFormat=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"CenterPatches","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bCenterPatches=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"UpAxis","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iUpAxis=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"GlobalPositioning","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bGlobalPositioning=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"LightMap","",cTemp,999,theApp.m_strIniFile.c_str());
	CString strLightMap(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"DeleteTextures","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bDeleteTextures=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"HeightAdaption","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bHeightAdaption=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureMapSize","-1",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureMapSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReducedGridSize","1",cTemp,999,theApp.m_strIniFile.c_str());
	int iReducedGridSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionMethod","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iReductionMethod=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionErrorValue","0",cTemp,999,theApp.m_strIniFile.c_str());
	float fReductionErrorValue=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"SimplifyBorder","1",cTemp,999,theApp.m_strIniFile.c_str());
	bool bSimplifyBorder=(atoi(cTemp)==1);

	// select format
	DXStudioFileFormatDlg dlgFormat;
	dlgFormat.m_iMaxPatchSize=Sector.GetBorderSize();
	dlgFormat.m_fScaleFactor=fScaleFactor;
	dlgFormat.m_iPatchSize=iPatchSize;
	dlgFormat.m_iTextureExport=iTextureExport;
	dlgFormat.m_iTextureFormat=iTextureFormat;
	dlgFormat.m_bCenterPatches=bCenterPatches;
	dlgFormat.m_bHeightAdaption=bHeightAdaption;
	dlgFormat.m_iUpAxis=iUpAxis;
	dlgFormat.m_bGlobalPositioning=bGlobalPositioning;
	dlgFormat.m_strLightMap=strLightMap;
	dlgFormat.m_bDeleteTextures=bDeleteTextures;
	dlgFormat.m_iTextureMapSize=iTextureMapSize;
	dlgFormat.m_iGridSize=iReducedGridSize;
	dlgFormat.m_iReductionMethod=iReductionMethod;
	dlgFormat.m_fErrorValue=fReductionErrorValue;
	dlgFormat.m_bSimplifyBorder=bSimplifyBorder;

	// fill all light map names into the dialog
	for (unsigned long nLightMap=0;nLightMap<Terrain.GetNumLightMapDefinitions();nLightMap++)
	{
		CIPnPTCLightMapDefinition01* pLightMapDef=Terrain.GetLightMapDefinition(nLightMap);
		if (pLightMapDef)
		{
			dlgFormat.m_vecLightMapNames.push_back(std::string(pLightMapDef->GetName()));
		}
	}

	// open dialog
	if (dlgFormat.DoModal()!=IDOK)
	{
		return false;
	}

	// store settings
	CString str;
	str.Format("%f",dlgFormat.m_fScaleFactor);
	::WritePrivateProfileString(MODULE_NAME,"ScaleFactor",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iPatchSize);
	::WritePrivateProfileString(MODULE_NAME,"PatchSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureExport);
	::WritePrivateProfileString(MODULE_NAME,"TextureExport",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureFormat);
	::WritePrivateProfileString(MODULE_NAME,"TextureFormat",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"CenterPatches",(dlgFormat.m_bCenterPatches)?"1":"0",theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iUpAxis);
	::WritePrivateProfileString(MODULE_NAME,"UpAxis",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"GlobalPositioning",(dlgFormat.m_bGlobalPositioning)?"1":"0",theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"LightMap",dlgFormat.m_strLightMap,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"DeleteTextures",(dlgFormat.m_bDeleteTextures)?"1":"0",theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"HeightAdaption",(dlgFormat.m_bHeightAdaption)?"1":"0",theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureMapSize);
	::WritePrivateProfileString(MODULE_NAME,"TextureMapSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iGridSize);
	::WritePrivateProfileString(MODULE_NAME,"ReducedGridSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iReductionMethod);
	::WritePrivateProfileString(MODULE_NAME,"ReductionMethod",str,theApp.m_strIniFile.c_str());
	str.Format("%f",dlgFormat.m_fErrorValue);
	::WritePrivateProfileString(MODULE_NAME,"ReductionErrorValue",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"SimplifyBorder",(dlgFormat.m_bSimplifyBorder)?"1":"0",theApp.m_strIniFile.c_str());

	// get last export path
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ExportPath","",cTemp,999,theApp.m_strIniFile.c_str());
	std::string strDefaultExportPath(cTemp);

	// get export file name
	const char* cSaveFile=App.SaveFileDlg("DXStudio-file","dxmesh",strDefaultExportPath.c_str());
	if (cSaveFile)
	{
		// store last path
		std::string strNewPath=GetPath(std::string(cSaveFile));
		::WritePrivateProfileString(MODULE_NAME,"ExportPath",strNewPath.c_str(),theApp.m_strIniFile.c_str());

		// show hourglass
		HCURSOR curRestore=GetCursor();
		SetCursor(LoadCursor(NULL,IDC_WAIT));

		// set vertex reduction
		this->SetSimplificationMethod(static_cast<SIMPLIFICATION_METHOD>(dlgFormat.m_iReductionMethod));
		switch (dlgFormat.m_iReductionMethod)
		{
		case 1:
			this->SetRegularGridSpacing(dlgFormat.m_iGridSize);
			break;
		case 2:
			this->SetErrorValue(fabs(dlgFormat.m_fErrorValue));
			this->SetSimplifyBorderQuads(dlgFormat.m_bSimplifyBorder);
			break;
		}

		// The possible height correction value
		float fHeightOffset = 0.0;
		if (dlgFormat.m_bHeightAdaption) fHeightOffset = getSectorHeightOffset(&Sector);

		// write sector data
		bool bReturn=this->WriteSector(App,Sector,Terrain,(CString)cSaveFile,dlgFormat.m_bCenterPatches,dlgFormat.m_fScaleFactor,dlgFormat.m_iPatchSize,dlgFormat.m_iTextureExport,dlgFormat.m_iTextureFormat,dlgFormat.m_iUpAxis,dlgFormat.m_bGlobalPositioning,dlgFormat.m_strLightMap,dlgFormat.m_bDeleteTextures,dlgFormat.m_iTextureMapSize,fHeightOffset);

		// restore cursor
		SetCursor(curRestore);

		// return result
		return bReturn;
	}

	// an error occured
	return false;
}

bool CPnPTCDXStudioFileHMapExportPlugIn::ExportTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain)
{
	// MFC needs this
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// read settings
	static char cTemp[1000];
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ScaleFactor","1",cTemp,999,theApp.m_strIniFile.c_str());
	float fScaleFactor=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"PatchSize","128",cTemp,999,theApp.m_strIniFile.c_str());
	int iPatchSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureExport","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureExport=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureFormat","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureFormat=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"CenterPatches","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bCenterPatches=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"UpAxis","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iUpAxis=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"GlobalPositioning","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bGlobalPositioning=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"LightMap","",cTemp,999,theApp.m_strIniFile.c_str());
	CString strLightMap(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"DeleteTextures","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bDeleteTextures=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"HeightAdaption","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bHeightAdaption=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureMapSize","-1",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureMapSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReducedGridSize","1",cTemp,999,theApp.m_strIniFile.c_str());
	int iReducedGridSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionMethod","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iReductionMethod=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionErrorValue","0",cTemp,999,theApp.m_strIniFile.c_str());
	float fReductionErrorValue=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"SimplifyBorder","1",cTemp,999,theApp.m_strIniFile.c_str());
	bool bSimplifyBorder=(atoi(cTemp)==1);

	// select format
	DXStudioFileFormatDlg dlgFormat;
	dlgFormat.m_iMaxPatchSize=Terrain.GetSectorDetailBorderSize();
	dlgFormat.m_fScaleFactor=fScaleFactor;
	dlgFormat.m_iPatchSize=iPatchSize;
	dlgFormat.m_iTextureExport=iTextureExport;
	dlgFormat.m_iTextureFormat=iTextureFormat;
	dlgFormat.m_bCenterPatches=bCenterPatches;
	dlgFormat.m_bHeightAdaption=bHeightAdaption;
	dlgFormat.m_iUpAxis=iUpAxis;
	dlgFormat.m_bGlobalPositioning=bGlobalPositioning;
	dlgFormat.m_strLightMap=strLightMap;
	dlgFormat.m_bDeleteTextures=bDeleteTextures;
	dlgFormat.m_iTextureMapSize=iTextureMapSize;
	dlgFormat.m_iGridSize=iReducedGridSize;
	dlgFormat.m_iReductionMethod=iReductionMethod;
	dlgFormat.m_fErrorValue=fReductionErrorValue;
	dlgFormat.m_bSimplifyBorder=bSimplifyBorder;

	// fill all light map names into the dialog
	for (unsigned long nLightMap=0;nLightMap<Terrain.GetNumLightMapDefinitions();nLightMap++)
	{
		CIPnPTCLightMapDefinition01* pLightMapDef=Terrain.GetLightMapDefinition(nLightMap);
		if (pLightMapDef)
		{
			dlgFormat.m_vecLightMapNames.push_back(std::string(pLightMapDef->GetName()));
		}
	}

	// open dialog
	if (dlgFormat.DoModal()!=IDOK)
	{
		return false;
	}

	// store settings
	CString str;
	str.Format("%f",dlgFormat.m_fScaleFactor);
	::WritePrivateProfileString(MODULE_NAME,"ScaleFactor",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iPatchSize);
	::WritePrivateProfileString(MODULE_NAME,"PatchSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureExport);
	::WritePrivateProfileString(MODULE_NAME,"TextureExport",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureFormat);
	::WritePrivateProfileString(MODULE_NAME,"TextureFormat",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"CenterPatches",(dlgFormat.m_bCenterPatches)?"1":"0",theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iUpAxis);
	::WritePrivateProfileString(MODULE_NAME,"UpAxis",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"GlobalPositioning",(dlgFormat.m_bGlobalPositioning)?"1":"0",theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"LightMap",dlgFormat.m_strLightMap,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"DeleteTextures",(dlgFormat.m_bDeleteTextures)?"1":"0",theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"HeightAdaption",(dlgFormat.m_bHeightAdaption)?"1":"0",theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureMapSize);
	::WritePrivateProfileString(MODULE_NAME,"TextureMapSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iGridSize);
	::WritePrivateProfileString(MODULE_NAME,"ReducedGridSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iReductionMethod);
	::WritePrivateProfileString(MODULE_NAME,"ReductionMethod",str,theApp.m_strIniFile.c_str());
	str.Format("%f",dlgFormat.m_fErrorValue);
	::WritePrivateProfileString(MODULE_NAME,"ReductionErrorValue",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"SimplifyBorder",(dlgFormat.m_bSimplifyBorder)?"1":"0",theApp.m_strIniFile.c_str());

	// get last export path
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ExportPath","",cTemp,999,theApp.m_strIniFile.c_str());
	std::string strDefaultExportPath(cTemp);

	// get export path
	const char* cSavePath=App.SaveFolderDlg(strDefaultExportPath.c_str());
	if (cSavePath)
	{
		// show hourglass
		HCURSOR curRestore=GetCursor();
		SetCursor(LoadCursor(NULL,IDC_WAIT));

		// append backslash to path
		CString strPath=cSavePath;
		if (strPath.Right(1)!=(CString)"\\")
		{
			strPath+="\\";
		}

		// store storage path
		::WritePrivateProfileString(MODULE_NAME,"ExportPath",strPath,theApp.m_strIniFile.c_str());

		// get number of sectors
		unsigned int nNumSectors=Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY();

		// set progress bar
		pnptc::CProgressBar ProgressBar(App,"Exporting terrain",Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY());

		// The possible height correction value
		float fHeightOffset = 0.0;
		if (dlgFormat.m_bHeightAdaption) fHeightOffset = getTerrainHeightOffset(Terrain);

		// export all sectors
		unsigned int nSector=0;
		for (unsigned int nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
		{
			for (unsigned int nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++,nSector++)
			{
				// get sector to be exported
				CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
				if (!pSector)
				{
					SetCursor(curRestore);
					CString strMessage;
					strMessage.Format("The detail sector heightmap of sector %u/%u could not be opened.",nSecX,nSecY);
					MessageBox(NULL,strMessage,"Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// create sector file name
				CString strX;
				strX.Format("%u",nSecX);
				strX="0000"+strX;
				CString strY;
				strY.Format("%u",nSecY);
				strY="0000"+strY;
				CString strFileName=strPath+strX.Right(5)+"_"+strY.Right(5)+".dxmesh";

				// set vertex reduction
				this->SetSimplificationMethod(static_cast<SIMPLIFICATION_METHOD>(dlgFormat.m_iReductionMethod));
				switch (dlgFormat.m_iReductionMethod)
				{
				case 1:
					this->SetRegularGridSpacing(dlgFormat.m_iGridSize);
					break;
				case 2:
					this->SetErrorValue(fabs(dlgFormat.m_fErrorValue));
					this->SetSimplifyBorderQuads(dlgFormat.m_bSimplifyBorder);
					break;
				}

				// export sector
				bool bReturn=this->WriteSector(App,*pSector,Terrain,strFileName,dlgFormat.m_bCenterPatches,dlgFormat.m_fScaleFactor,dlgFormat.m_iPatchSize,dlgFormat.m_iTextureExport,dlgFormat.m_iTextureFormat,dlgFormat.m_iUpAxis,dlgFormat.m_bGlobalPositioning,dlgFormat.m_strLightMap,dlgFormat.m_bDeleteTextures,dlgFormat.m_iTextureMapSize,fHeightOffset);

				// successfull?
				if (!bReturn)
				{
					// error exporting sector
					SetCursor(curRestore);
					return false;
				}

				// increment progress bar
				ProgressBar.IncrementPos();
			}
		}

		// ok
		SetCursor(curRestore);
		return true;
	}

	// an error occured
	return false;
}



// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// Write sector
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------




bool CPnPTCDXStudioFileHMapExportPlugIn::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,CString Filename,bool bCenterPatches,float fScale,int iPatchSize,int iTextureExport,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,const CString& strLightMap,bool bDeleteTextures,int iExportTextureMapSize,float fHeightOffset)
{
	// store export format
	this->m_bCenterPatches=bCenterPatches;

	// store the delete textures flag
	this->m_bDeleteTextures=bDeleteTextures;

	// store the height adaption offset
	this->m_fHeightOffset=fHeightOffset;

	// write sector with the appropriate method
	switch (iTextureExport)
	{
	case 0:
		return this->WriteSectorUntextured(App,Sector,Terrain,(LPCSTR)Filename,fScale,iPatchSize,iUpAxis,bGlobalPositioning);
		break;
	case 1:
		return this->WriteSectorDetailTexture(App,Sector,Terrain,(LPCSTR)Filename,fScale,iPatchSize,iTextureFormat,iUpAxis,bGlobalPositioning,false,"",false,iExportTextureMapSize);
		break;
	case 2:
		return this->WriteSectorTexturemap(App,Sector,Terrain,(LPCSTR)Filename,fScale,iPatchSize,iTextureFormat,iUpAxis,bGlobalPositioning,false,(LPCSTR)strLightMap,!strLightMap.IsEmpty(),false,false,iExportTextureMapSize);
		break;
	default:
		return false;
		break;
	}
}

bool CPnPTCDXStudioFileHMapExportPlugIn::SaveObject(C3DObjectData* pObj,const std::string& strFileName)
{
	// save it to a dxmesh file
	C3DObjectSerializerDXStudio serializer;
	serializer.SetCenterPatches(this->m_bCenterPatches);
	serializer.SetDeleteTextures(this->m_bDeleteTextures);
	serializer.SetHeightOffset(this->m_fHeightOffset);	
	return serializer.Save(*pObj,strFileName);
}

// --------------------------------------------------------------------------------------------
// Step through all terrain vertices. Get the lowest one and return a offset that is
// needed to set the lowest terrain vertex to "0"
// --------------------------------------------------------------------------------------------
float CPnPTCDXStudioFileHMapExportPlugIn::getTerrainHeightOffset(CIPnPTCTerrain01& Terrain)
{
	float minHeight = 100000.0f;
	
	for (UINT nSecX=0; nSecX<Terrain.GetNumSectorsX(); nSecX++)
	{
		for (UINT nSecY=0; nSecY<Terrain.GetNumSectorsY(); nSecY++)	
		{
			CIPnPTCSector01* pSector = Terrain.GetSectorDetail(nSecX,nSecY);

			float tmpHeight = getSectorHeightOffset(pSector);
			if (tmpHeight<minHeight) minHeight = tmpHeight;
		}
	}

	return(minHeight);
}

// --------------------------------------------------------------------------------------------
// Step through all sector vertices. Get the lowest one and return a offset that is
// needed to set the lowest sector vertex to "0"
// --------------------------------------------------------------------------------------------
float CPnPTCDXStudioFileHMapExportPlugIn::getSectorHeightOffset(CIPnPTCSector01* pSector)
{
	float minHeight = 100000.0f;

	int iVertsXY = (int) pSector->GetBorderSize();

	for (int x=0;x<iVertsXY;x++)
	{
		for (int y=0;y<iVertsXY;y++)
		{
			if (pSector->GetHeight(x,y) < minHeight) minHeight = pSector->GetHeight(x,y);
		}
	}

	return(minHeight);
}



