/////////////////////////////////////////////////////////////////////////////////////
// PnPTCDXStudioFileHMapExportPlugIn.h
// *.dxmesh file heightmap exporter PlugIn definition.
// ----------------------------------------------------------------------------------
// author:		miko, based on pnpTC demos
// ----------------------------------------------------------------------------------
// change history:
//   v1.2  2009/5/9: first release in new design /miko
//   v1.3  2009/5/12: fixed "delete textures could be ON when exporting detail textures"
// ----------------------------------------------------------------------------------
// copyright notice:
/////////////////////////////////////////////////////////////////////////////////////


#if !defined(PNPTC_DXSTUDIOFILEHMAPEXPORTPLUGIN_H_INCLUDED)
#define PNPTC_DXXTUDIOFILEHMAPEXPORTPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols

#include "../PnPTC3DObject/TerrainMeshCreator.h"


// --------------------------------------------------------------------------------------------
// Application class
// --------------------------------------------------------------------------------------------

// CPnPTCDXStudioFileHMapExportPlugInApp
// See PnPTCDXStudioFileHMapExportPlugIn.cpp for the implementation of this class
class CPnPTCDXStudioFileHMapExportPlugInApp : public CWinApp
{
public:
	CPnPTCDXStudioFileHMapExportPlugInApp();

	/// file storing the plugin specific settings
	std::string		m_strIniFile;

// Overrides
public:
	virtual BOOL InitInstance();
	
	DECLARE_MESSAGE_MAP()
};



// --------------------------------------------------------------------------------------------
// PlugIn class
// --------------------------------------------------------------------------------------------

/// generator plugin class
class CPnPTCDXStudioFileHMapExportPlugIn  : public IPnPTCExportPlugIn01, public CTerrainMeshCreator
{
public:
	/// construction / destruction
	CPnPTCDXStudioFileHMapExportPlugIn();
	virtual ~CPnPTCDXStudioFileHMapExportPlugIn();

	/// Implementation of initialization/deinitialization methods of this plugin.<br>
	/// See the description of IPnPTCExportPlugIn01 for further details.
	GUID		GetGUID();
	const char*	GetName();

	/// Implementation of the terrain exporting methods.
	/// See the description of IPnPTCExportPlugIn01 for further details.
	bool		ExportSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain);
	bool		ExportTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain);

protected:

	/// Internal method to store one sector to a file
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,CString Filename,bool bCenterPatches,float fScale,int iPatchSize,int iTextureExport,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,const CString& strLightMap,bool bDeleteTextures,int iExportTextureMapSize,float bHeightOffset);

	/// saves the created object to a file
	bool		SaveObject(C3DObjectData* pObj,const std::string& strFileName);


private:

	// ----------------------------------------------------------------------
	// member routines
	// ----------------------------------------------------------------------
	
	float getTerrainHeightOffset(CIPnPTCTerrain01& Terrain);
	float getSectorHeightOffset(CIPnPTCSector01* pSector);
	
	// ----------------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------------

	/// special option "center all PATCHES on 0,0 (useful for DXStudio positioning with 3.1)
	bool		m_bCenterPatches;
	/// special option "delete textures" after adding them to the *.dxmesh file
	bool		m_bDeleteTextures;
	/// special option "Set lowest terrain to height 0": This is the offset that is compensated
	float		m_fHeightOffset;

};


#endif // !defined(PNPTC_DXSTUDIOFILEHMAPEXPORTPLUGIN_H_INCLUDED)
