#ifndef IDXANIMATIONKEYFRAME_H
#define IDXANIMATIONKEYFRAME_H

namespace DX
{
	/**
		Used to store events and data that will happen at
		the specified time. Currently contains script.
	*/
	struct  IDXAnimationKeyframe : public IDXUserData, public IDXModelOwned, public IDXAnimationOwned
	{
		virtual				DXMCALL ~IDXAnimationKeyframe() = 0;
		/**
			Get the time that this keyframe is at.
			@return Time.
		*/
		virtual float		DXMCALL GetTime( ) const = 0;
		/**
			Get the script at this keyframe.
			@param out A const char** where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NotFound if there was no script.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError		DXMCALL GetScript( const char** out ) const = 0;
		/**
			Set the time that this keyframe is at.
		*/
		virtual void		DXMCALL SetTime( float time ) = 0;
		/**
			Set the script at this keyframe.
			@param script A const char* which is the new script.
			@return Error_NullParam if script was null.\n
					Error_Success if script was changed.
		*/
		virtual DXError		DXMCALL SetScript( const char* script ) = 0;
		/**
			Remove the script on this keyframe.
			@return Error_NotFound if there was no script to remove.\n
					Error_Success if the script was removed.
		*/
		virtual DXError		DXMCALL RemoveScript( ) = 0;
	};
}

#endif