#ifndef IDXFACE_H
#define IDXFACE_H

namespace DX
{
	struct IDXMesh;
	/**
		An IDXFace represents a face on a model.
		It has vertex indices and a material index.
	*/
	struct  IDXFace : public IDXModelOwned, public IDXMeshOwned, public IDXUserData
	{
		virtual			DXMCALL ~IDXFace() = 0;
		/**
			Get the vertex index at the specified index in the face.
			@param index The index in the face to retrieve.
			@param out A pointer to a uint where the vertex index (index into IDXMesh vertex array) will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetVertexIndex( uint index, uint* out ) const = 0;
		/**
			Get the index into the mesh's materialmaps array that this
			face is using.
		*/
		virtual uint	DXMCALL GetMaterialIndex( ) const = 0;
		/**
			Set the vertex at the specified index in the face to the specified mesh vertex index.
			@param index The index of the vertex in the face (0, 1, 2)
			@param value The index of the vertex in the mesh
			@return Error_OutOfRange if index was out of range.\n
					Error_OutOfRange2 if value was out of range.\n
					Error_Success if the vertex index was set.
		*/
		virtual DXError DXMCALL SetVertexIndex( uint index, uint value ) = 0;
		/**
			Set the material index for this face. This is an index into the materialmaps array
			on the IDXMesh.
			@param index The index into the materialmaps array to use.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the material index was updated.
		*/
		virtual DXError DXMCALL SetMaterialIndex( uint index ) = 0;
	};

	typedef IDXFacePtr* IDXFaceArray;
}

#endif