#ifndef IDXFILEINT32_H
#define IDXFILEINT32_H

namespace DX
{
	typedef int* IntPtr;
	typedef IntPtr IntArray;
	/**
		File for storing ints.
	*/
	struct IDXFileInt32 : public IDXFile
	{
		virtual DXMCALL ~IDXFileInt32( ) = 0;
		/**
			Get the number of ints in this file.
		*/
		virtual uint DXMCALL GetCount( ) const = 0;
		/**
			Get the int at the specified index.
			@param index The index of the int to retrieve.
			@param out A pointer to a int where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( uint index, IntPtr out ) const = 0;
		/**
			Get all the ints.
			@param out A pointer to a user allocated array of ints. Must be
					   of length greater than or equal to GetCount.
			@return Error_NullParam if out was null.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( IntArray out ) const = 0;
		/**
			Get a group of ints.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of ints of length at least count.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL Get( uint start, uint count, IntArray out ) const = 0;
		/**
			Set the int at the specified index.
			@param index The index of the int to set.
			@param value The value to set the int to.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the int was updated.
		*/
		virtual DXError DXMCALL Set( uint index, int value ) = 0;
		/**
			Replace all the values in the array with the values in the int array
			passed in.
			@param values An array of ints.
			@param length The number of ints to copy from the array.
			@return Error_NullParam if values was null.\n
					Error_Success if the values were updated.
		*/
		virtual DXError DXMCALL Set( IntArray values, uint length ) = 0;
		/**
			Add the value to the end of the data.
			@param value The int to add.
			@DX_SUCCESS always.
		*/
		virtual DXError DXMCALL Add( int value ) = 0;
		/**
			Add the data in the int array specified to the end of the data.
			@param values A int array to copy data from.
			@param length A uint which is the number of ints to copy from the
						  array passed in.
			@return Error_NullParam if values was null.\n
					Error_Success if the values were appended,
		*/
		virtual DXError DXMCALL Add( IntArray values, uint length ) = 0;
		/**
			Add the data in the int array specified to any point in the data.
			@param value A int array to copy data from.
			@param length The amount of data to copy.
			@param start The point to start adding data.
			@return Error_NullParam if value was null.\n
					Error_OutOfRange if start was greater than GetCount.
		*/
		virtual DXError DXMCALL Add( IntArray value, uint length, uint start ) = 0;
		/**
			Remove the element at the specified index.
			@param index The index of the element to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL Remove( uint index ) = 0;
		/**
			Remove a range of elements from the file.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL Remove( uint start, uint count ) = 0;
	};
}

#endif