#ifndef IDXFILERESOURCE_H
#define IDXFILERESOURCE_H

namespace DX
{
	/**
		Interface for file resources. Things like textures, or things that
		the user just wants included.
	*/
	struct IDXFileResource : public IDXFile
	{
		virtual ~IDXFileResource( ) = 0;
		/**
			Get the filename that this resource comes from.
			@return A wchar_t* which is the filename.
		*/
		virtual const wchar_t* DXMCALL GetSource( ) const = 0;
		/**
			Add a byte to the end of the file.
			@param value The byte to add.
			@return Error_Success always.
		*/
		virtual DXError DXMCALL Add( byte value ) = 0;
		/**
			Add an array of bytes to the end of the file.
			@param values An array of bytes to append.
			@param length The number of bytes to add.
			@return Error_NullParam if values was null.\n
					Error_Success if the values were added.
		*/
		virtual DXError DXMCALL Add( ByteArray values, uint length ) = 0;
		/**
			Insert an array of values into the data.
			@param values A byte array to add.
			@param length The number of bytes to add.
			@param start The index to insert the bytes.
			@return Error_NullParam if values was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the values were inserted.
		*/
		virtual DXError DXMCALL Add( ByteArray values, uint length, uint start ) = 0;
		/**
			Set the filename that this resource comes from. This is not the filename
			that will be used in the zip.
			@param source The source filename
			@return Error_NullParam if source was null.\n
					Error_Success if source was set.
		*/
		virtual DXError DXMCALL SetSource( const wchar_t* source ) = 0;
		/**
			Set the data in this file.
			@param array An array of bytes to use for the data.
			@param length The number of bytes.
			@return Error_NullParam if array was null.\n
					Error_Success if the data was set.
		*/
		virtual DXError DXMCALL Set( ByteArray array, uint length ) = 0;
		/**
			Remove the element at the specified index.
			@param index The index of the element to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL Remove( uint index ) = 0;
		/**
			Remove a range of elements from the file.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL Remove( uint start, uint count ) = 0;
	};
}

#endif