#ifndef IDXGETGROUPRESULT_H
#define IDXGETGROUPRESULT_H

namespace DX
{
	/**
		IDXGetGroupResult facilitates the finding of multiple groups. Its 
		lifetime is limited to until the next search is performed or the object
		is deleted.
	*/
	struct  IDXGetGroupResult
	{
		virtual			DXMCALL ~IDXGetGroupResult() = 0;
		/**
			Get the number of groups that are in the group search result.
			@return The number of groups.
		*/
		virtual uint	DXMCALL GetNumGroups( ) const = 0;
		/**
			Get the group at the specified index.
			@param index The index of the group to retrieve.
			@param out A pointer to an IDXGroupPtr where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetGroup( uint index, IDXGroupPtr* out ) const = 0;
		/**
			Add a group to the results.
			@param group An IDXGroupPtr to add.
			@return Error_NullParam if group was null.\n
					Error_Duplicate if group was already present.\n
					Error_Success if group was added.
		*/
		virtual DXError DXMCALL AddGroup( IDXGroupPtr group ) = 0;
	};
}

#endif