#ifndef IDXMATERIALMAP_H
#define IDXMATERIALMAP_H

namespace DX
{
	enum DXMapType
	{
		MapType_Standard = 0,
		MapType_Light,
		MapType_Multi,
		MapType_Reserved
	};

	/**
		IDXMaterialMap is used for standard maps and is the 
		base for multi and light maps.
	*/
	struct  IDXMaterialMap : public IDXModelOwned, public IDXMaterialOwned, public IDXUserData
	{
		virtual					DXMCALL ~IDXMaterialMap( ) = 0;
		/**
			Get the coordinate channel index that this map will be applied to.
			@return The coord index.
		*/
		virtual uint			DXMCALL GetMapCoordChannel( ) const = 0;
		/**
			Get the name of the texture that will be used for this map.
			@param The texture map.
		*/
		virtual const wchar_t*	DXMCALL GetSource( ) const = 0;
		/**
			Get the type of map this is.
			@return A member of the DXMapType enumeration.
		*/
		virtual DXMapType		DXMCALL GetType( ) const = 0;
		/**
			Set the coordinate channel that this map will be applied to.
			@param channel The channel index that will be used.
		*/
		virtual void			DXMCALL SetMapCoordChannel( uint channel ) = 0;
		/**
			Set the filename that will be used for the map.
			@param source The name of the texture to use.
			@return Error_NullParam if source was null.\n
					Error_Success if the source was updated.
		*/
		virtual DXError			DXMCALL SetSource( const wchar_t* source ) = 0;
	};
}

#endif