#ifndef IDXMULTIMAP_H
#define IDXMULTIMAP_H

namespace DX
{
	enum DXMapCombine
	{
		Combine_Normal = 0,
		Combine_Multiply,
		Combine_Lighten,
		Combine_Darken,
		Combine_Reserved
	};

	/**
		A MultiMap is a nonstandard map that is used as an additional 
		map, which can be used for overlays and the like.
	*/
	struct  IDXMultiMap : public IDXMaterialMap
	{
		virtual				 DXMCALL ~IDXMultiMap() = 0;
		/**
			Get the opacity of this mulitmap. This will affect
			appearance when combined with existing maps on the mesh.
		*/
		virtual float		 DXMCALL GetOpacity() const = 0;
		/**
			Get the uv channel that this multimap is using for its alpha
			channel.
		*/
		virtual uint		 DXMCALL GetMapCoordChannelAlpha( ) const = 0;
		/**
			Get the combine type for this multimap. This controls how
			the texture will be combined with the existing texture.
			@return A member of DXMapCombine enumeration.
		*/
		virtual DXMapCombine DXMCALL GetCombineType() const = 0;	
		/**
			Set the uv channel that the multimap will use for its
			alpha channel.
		*/
		virtual void		 DXMCALL SetMapCoordChannelAlpha( uint value ) = 0;
		/**
			Set the opacity that this multimap will use.
		*/
		virtual void		 DXMCALL SetOpacity( float value ) = 0;
		/**
			Set the combine type. This will affect how the multimap will
			be blended with the existing maps.
			@param type A member of the DXMapCombine enumeration.
			@return Error_OutOfRange if type was out of range.\n
					Error_Success if the combine type was updated.
		*/
		virtual DXError		 DXMCALL SetCombineType( DXMapCombine type ) = 0;
	};
}

#endif