#ifndef IDXOBJECTPROPATTRIBUTE_H
#define IDXOBJECTPROPATTRIBUTE_H

namespace DX
{
	/**
		Used by IDXObjectProp to store specific attributes.
	*/
	struct  IDXObjectPropAttribute : public IDXModelOwned, public IDXObjectPropOwned, public IDXUserData
	{
		virtual				DXMCALL ~IDXObjectPropAttribute() = 0;
		/**
			Get the current ID of this attribute.
			@return The ID of the attribute.
		*/
		virtual const char* DXMCALL GetID() const = 0;
		/**
			Get the value of the attribute.
			@return The value of the attribute.
		*/
		virtual const char* DXMCALL GetValue() const = 0;
		/**
			Set the ID of the attribute.
			@param id The new ID.
			@return Error_NullParam if id was null.\n
					Error_InvalidParam if id contained invalid character.\n
					Error_Success if the ID was updated.
		*/
		virtual DXError		DXMCALL SetID( const char* id ) = 0;
		/**
			Set the value of the attribute.
			@param value The new value for the attribute.
			@return Error_NullParam if value was null.\n
					Error_Success if value was updated.
		*/
		virtual DXError		DXMCALL SetValue( const char* value ) = 0;
	};
}


#endif