#ifndef IDXSURFACEDETAILXYZ_H
#define IDXSURFACEDETAILXYZ_H

namespace DX
{
	/**
		
	*/
	struct IDXSurfaceDetailXYZ : public IDXUserData, public IDXHasFileBinding
	{
		virtual DXMCALL ~IDXSurfaceDetailXYZ( ) = 0;
		/**
			Get the current filename that will be used to save the .dxxyz data.
			@return The filename that will be used.
		*/
		virtual const wchar_t* DXMCALL GetSource( ) const = 0;
		/**
			Get the number of elements in this xyz.
			@return The number of elements.
		*/
		virtual uint DXMCALL GetNumElements( ) const = 0;
		/**
			Get the element at the specified index.
			@param index The index of the element to retrieve.
			@param out A pointer to an IDXSurfaceDetailXYZElementPtr to store the result.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if index was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetElement( uint index, IDXSurfaceDetailXYZElementPtr* out ) const = 0;
		/**
			Get all the elements in this xyz
			@param out A user allocated array of length not less than GetNumElements.
			@return Error_NullParam if out was null.\n
					Error_Success if the elements were stored in out.
		*/
		virtual DXError DXMCALL GetElements( IDXSurfaceDetailXYZElementArray out ) const = 0;
		/**
			Get a chunk of elements in this xyz.
			@param start The index of the first element to retrieve.
			@param count The number of elements to retrieve.
			@param out A user allocated array of IDXSurfaceDetailXYZElementPtr of length at least count where
					   the result will be stored.
		    @return Error_NullParam if out was null.\n
					Error_OutOfRange if start was out of range.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetElements( uint start, uint count, IDXSurfaceDetailXYZElementArray out ) const = 0;
		/**
			Get the index of the specified element.
			@param element The element to get the index of
			@param out A pointer to a uint where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_NullParam2 if element was null.\n
					Error_NotFound if the element was not found.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError DXMCALL GetElementIndex( IDXSurfaceDetailXYZElementPtr element, uint* out ) const = 0;
		/**
			Set the filename that will be used to save the .dxxyz data.
			@param source A filename to use.
			@return Error_NullParam if source was null.\n
					Error_Success if the source was updated.
		*/
		virtual DXError DXMCALL SetSource( const wchar_t* source ) = 0;
		/**
			Add an element.
			@param type The type of element. A member of the DXXYZEntryType enumeration.
			@param subtype The index into the subtypes of the type.
			@param pos The position that the mesh will be added at.
			@param normal The normal of the element.
			@param rotation The rotation of the elemen.
			@param scale The scale of the element,
			@param out An optional pointer to an IDXSurfaceDetailXYZElementPtr where the result is stored.
			@return Error_OutOfRange if type was out of range.\n
					Error_Success if the element was added.
		*/
		virtual DXError DXMCALL AddElement( DXXYZEntryType type, byte subtype, const DXVector& pos, const DXVector& normal,
			const DXRotation& rotation, const DXVector& scale, IDXSurfaceDetailXYZElementPtr* out = 0 ) = 0;
		/**
			Remove the element at the specified index.
			@param index The index of the element to remove.
			@return Error_OutOfRange if index was out of range.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL RemoveElement( uint index ) = 0;
		/**
			Remove a range of elements.
			@param start The index of the first element to remove.
			@param count The number of elements to remove.
			@return Error_OutOfRange if start was out of range.\n
					Error_Success if the range was removed.
		*/
		virtual DXError DXMCALL RemoveElement( uint start, uint count ) = 0;
		/**
			Remove the specified element.
			@param element The element to remove.
			@return Error_NullParam if element was null.\n
					Error_NotFound if the element was not found.\n
					Error_Success if the element was removed.
		*/
		virtual DXError DXMCALL RemoveElement( IDXSurfaceDetailXYZElementPtr element ) = 0;
	};
}

#endif