#ifndef IDXSURFACEDETAILXYZELEMENT_H
#define IDXSURFACEDETAILXYZELEMENT_H

namespace DX
{
	/**
		The type of the entry for this element.
	*/
	enum DXXYZEntryType
	{
		XYZType_None = 0,
		XYZType_Billboard,
		XYZType_Mesh,
		XYZType_Reserved
	};

	/**
		This represents an entry in an xyz based surface detail map.
	*/
	struct IDXSurfaceDetailXYZElement : public IDXUserData
	{
		
		virtual DXMCALL ~IDXSurfaceDetailXYZElement( ) = 0;
		/**
			Get the type of the XYZ Element.
			@return A member of the DXXYZEntryType enumeration.
		*/
		virtual DXXYZEntryType DXMCALL GetType( ) const = 0;
		/**
			Get the subtype for this element.
			@return A byte which is the subtype index.
		*/
		virtual byte DXMCALL GetSubType( ) const = 0;
		/**
			Get the position of the element.
			@return A DXVector which is the position of the element.
		*/
		virtual const DXVector& DXMCALL GetPosition( ) const = 0;
		/**
			Get the scale of the element.
			@return A DXVector which is the scale of the element.
		*/
		virtual const DXVector& DXMCALL GetScale( ) const = 0;
		/**
			Get the rotation of the element.
			@return A DXRotation which is the rotation of the element.
		*/
		virtual const DXRotation& DXMCALL GetRotation( ) const = 0;
		/**
			Get the normal of the element.
			@return A DXVector which is the normal of the element.
		*/
		virtual const DXVector& DXMCALL GetNormal( ) const = 0;
		/**
			Set the type of this XYZ Element.
			@param type A member of the DXXYZEntryType enumeration
			@return Error_OutOfRange if type was greater than or equal to Type_Reserved\n
					Error_Success if the type was updated.
		*/
		virtual DXError DXMCALL SetType( DXXYZEntryType type ) = 0;
		/**
			Set the index into the type that will be used. (Eg, mesh id 3)
			@param index The index to use
		*/
		virtual void DXMCALL SetSubtype( byte index ) = 0;
		/**
			Set the position of the element.
			@param pos A DXVector which is the position of the new element.
		*/
		virtual void DXMCALL SetPosition( const DXVector& pos ) = 0;
		/**
			Set the scale of the element.
			@param pos A DXVector which is the scale of the new element.
		*/
		virtual void DXMCALL SetScale( const DXVector& scale ) = 0;
		/**
			Set the rotation of the element.
			@param pos A DXRotation which is the rotation of the new element.
		*/
		virtual void DXMCALL SetRotation( const DXRotation& rotation ) = 0;
		/**
			Set the normal of the element.
			@param pos A DXVector which is the normal of the new element.
		*/
		virtual void DXMCALL SetNormal( const DXVector& normal ) = 0;
	};
}

#endif