#ifndef IDXUSERTAG_H
#define IDXUSERTAG_H

namespace DX
{
	/**
		IDXUserTag allows for storage of user data in the model.
		In the form <usertag id="ID" value="value" />
	*/
	struct IDXUserTag : public IDXUserData
	{
		virtual DXMCALL ~IDXUserTag( ) = 0;
		/**
			Get the ID that this tag will have. 
			@return The tag ID.
		*/
		virtual const wchar_t* DXMCALL GetID( ) const = 0;
		/**
			Get the value that this tag will have.
			@return The tag value.
		*/
		virtual const wchar_t* DXMCALL GetValue( ) const = 0;
		/**
			Set the ID that this tag will have.
			@param id The id.
			@return Error_NullParam if id was null.\n
					Error_Success if the id was set
		*/
		virtual DXError DXMCALL SetID( const wchar_t* id ) = 0;
		/**
			Set the value that this tag will have.
			@param value The new value.
			@return Error_NullParam if value was null.\n
					Error_Success if the value was set.
		*/
		virtual DXError DXMCALL SetValue( const wchar_t* value ) = 0;
	};
}

#endif