#ifndef IDXVERTEX_H
#define IDXVERTEX_H

namespace DX
{
	/**
		Added to meshes to describe a vertex - a position, normal and 
		a number of UV coordinates.
	*/
	struct  IDXVertex : public IDXUserData, public IDXModelOwned, public IDXMeshOwned
	{
		virtual					DXMCALL ~IDXVertex() = 0;
		/**
			Get the position vector of this IDXVertex.
			@return The position
		*/
		virtual const DXVector&	DXMCALL GetPosition() const = 0;
		/**
			Get the normal vector of this IDXVertex.
			@return The normal.
		*/
		virtual const DXVector&	DXMCALL GetNormal() const = 0;
		/**
			Get the UV data for this vertex on the specified 
			channel.
			@param channel The channel number to retrieve.
			@param out A pointer to a DXUV where the result will be stored.
			@return Error_NullParam if out was null.\n
					Error_OutOfRange if channel was higher than this vert supports.\n
					Error_Success if the result was stored in out.
		*/
		virtual DXError			DXMCALL GetUV(uint channel, DXUV* out) = 0;
		/**
			Get the number of uvs that this vertex has.
		*/
		virtual uint			DXMCALL GetNumUVS() const = 0;
		/**
			Set the position vector of this IDXVertex.
			@param position A DXVector representing the new position of the IDXVertex.
		*/
		virtual void			DXMCALL SetPosition(const DXVector& position) = 0;
		/**
			Set the normal of this IDXVertex.
			@param normal A DXVector representing the new normal of the IDXVertex.
		*/
		virtual void			DXMCALL SetNormal(const DXVector& normal) = 0;
		/**
			Set the UV coordinates on the specified channel.
			@param channel The number of the channel to set the uvs for
			@param uv A DXUV which represents the uv coordinates.
		*/
		virtual DXError			DXMCALL SetUV(uint channel, const DXUV& uv) = 0;
	};
}

#endif