/////////////////////////////////////////////////////////////////////////////////////
// PnPTCXFileHMapExportPlugIn.cpp
// x file heightmap exporter PlugIn implementation.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//   06. Dec. 2005 : Added lightmap export possibilities
//   01. Jan. 2006 : Added y flipping of texture maps.
//   10. Oct. 2006 : Achieved VS2005 compatibility, made compilation easier.
//   10. Mai  2008 : Changed some CString to std::string
//   12. Mai  2008 : Added texture map size option and vertex reduction.
//   23. Jun. 2008 : Added possibility to exclude border quads from terrain 
//					 mesh simplification.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PnPTCXFileHMapExportPlugIn.h"
#include "XFileFormatDlg.h"
#include "../pnptc3dobject/3dobjectserializerx.h"
#include "../pnptcutils/progressbar.h"


// --------------------------------------------------------------------------------------------
// Plugin global definitions
// --------------------------------------------------------------------------------------------

/// Default name of this PlugIn module.
#define MODULE_NAME "PnPTCXFileHMapExportPlugIn"

/// The name of this plugin.<br>
/// Try to give your plugins an unique name.
#define	PLUGIN_NAME	"Heightmap to X-File Export (PnPTC)"

/// The unique identifier (GUID) of this plugin
/// Create your own uid for your plugins using
/// the guidgen-utility of VisualC or any other creation utility.
// {2794EA05-C50E-4598-8BAD-ACCF6B2C8206}
static const GUID g_uidPlugIn = { 0x2794ea05, 0xc50e, 0x4598, { 0x8b, 0xad, 0xac, 0xcf, 0x6b, 0x2c, 0x82, 0x6 } };


// --------------------------------------------------------------------------------------------
// Get informations of the plugin
// --------------------------------------------------------------------------------------------

/// Returns the API-ID used for this plugin.
/// @return ID of PlugIn-API.
__declspec(dllexport) int CALLBACK GetPlugInApiID()
{
	// return generator ID
	return PNPTC_PLUGIN_APIID_EXPORT;
}

/// Returns the name of this plugin.
/// @return Null-terminated character string containing
/// the plugin's name.
__declspec(dllexport) char* CALLBACK GetPlugInName()
{
	// return name
	return PLUGIN_NAME;
}

/// Get a unique identifier for this plugin.
/// @return The plugin's uid.
__declspec(dllexport) GUID CALLBACK GetPlugInGUID()
{
	// return the uid of your plugin
	return g_uidPlugIn;
}


// --------------------------------------------------------------------------------------------
// Creation of the plugin instance
// --------------------------------------------------------------------------------------------

/// Get an instance of the plugin
/// @return Returns a pointer to the create plugin.<br>
/// Return <i>NULL</i> if the plugin could not be created.
__declspec(dllexport) IPnPTCPlugIn* CALLBACK CreatePlugin()
{
	// create and return the plugin pointer
	return new CPnPTCXFileHMapExportPlugIn;
}

/// Free the plugin instance.
/// @param plugin Instance to be freed.
__declspec(dllexport) void CALLBACK DestroyPlugin(IPnPTCPlugIn* plugin)
{
	// deletes the plugin
	if (plugin)
	{
		delete (CPnPTCXFileHMapExportPlugIn*)plugin;
	}
}




// --------------------------------------------------------------------------------------------
// The application object
// --------------------------------------------------------------------------------------------


//
//	Note!
//
//		If this DLL is dynamically linked against the MFC
//		DLLs, any functions exported from this DLL which
//		call into MFC must have the AFX_MANAGE_STATE macro
//		added at the very beginning of the function.
//
//		For example:
//
//		extern "C" BOOL PASCAL EXPORT ExportedFunction()
//		{
//			AFX_MANAGE_STATE(AfxGetStaticModuleState());
//			// normal function body here
//		}
//
//		It is very important that this macro appear in each
//		function, prior to any calls into MFC.  This means that
//		it must appear as the first statement within the 
//		function, even before any object variable declarations
//		as their constructors may generate calls into the MFC
//		DLL.
//
//		Please see MFC Technical Notes 33 and 58 for additional
//		details.
//

// CPnPTCXFileHMapExportPlugInApp

BEGIN_MESSAGE_MAP(CPnPTCXFileHMapExportPlugInApp, CWinApp)
END_MESSAGE_MAP()


// CPnPTCXFileHMapExportPlugInApp construction

CPnPTCXFileHMapExportPlugInApp::CPnPTCXFileHMapExportPlugInApp()
{
	// TODO: add construction code here,
	// Place all significant initialization in InitInstance
}


// The one and only CPnPTCXFileHMapExportPlugInApp object
CPnPTCXFileHMapExportPlugInApp theApp;


// CPnPTCXFileHMapExportPlugInApp initialization

BOOL CPnPTCXFileHMapExportPlugInApp::InitInstance()
{
	CWinApp::InitInstance();

	// get ini file name
	char cModuleFileName[MAX_PATH+1];
	::GetModuleFileName(theApp.m_hInstance,cModuleFileName,MAX_PATH);
	std::string strModuleFileName(cModuleFileName);
	std::string	strPlugInPath=GetPath(strModuleFileName);
	this->m_strIniFile=strPlugInPath+"PlugIns.ini";

	return TRUE;
}




// --------------------------------------------------------------------------------------------
// Exporter PlugIn
// --------------------------------------------------------------------------------------------

CPnPTCXFileHMapExportPlugIn::CPnPTCXFileHMapExportPlugIn()
{
	// constructor => nothing to do
	return;
}

CPnPTCXFileHMapExportPlugIn::~CPnPTCXFileHMapExportPlugIn()
{
	// destructor => nothing to do
	return;
}

const char* CPnPTCXFileHMapExportPlugIn::GetName()
{
	// return plugin name
	return PLUGIN_NAME;
}

GUID CPnPTCXFileHMapExportPlugIn::GetGUID()
{
	// return guid of plugin
	return g_uidPlugIn;
}

bool CPnPTCXFileHMapExportPlugIn::ExportSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain)
{
	// MFC needs this
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// read settings
	static char cTemp[1000];
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ScaleFactor","1",cTemp,999,theApp.m_strIniFile.c_str());
	float fScaleFactor=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"PatchSize","128",cTemp,999,theApp.m_strIniFile.c_str());
	int iPatchSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureExport","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureExport=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureFormat","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureFormat=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"FileFormat","1",cTemp,999,theApp.m_strIniFile.c_str());
	int iFileFormat=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"UpAxis","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iUpAxis=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"GlobalPositioning","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bGlobalPositioning=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"LightMap","",cTemp,999,theApp.m_strIniFile.c_str());
	CString strLightMap(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"FlipY","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bFlipY=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureMapSize","-1",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureMapSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReducedGridSize","1",cTemp,999,theApp.m_strIniFile.c_str());
	int iReducedGridSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionMethod","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iReductionMethod=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionErrorValue","0",cTemp,999,theApp.m_strIniFile.c_str());
	float fReductionErrorValue=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"SimplifyBorder","1",cTemp,999,theApp.m_strIniFile.c_str());
	bool bSimplifyBorder=(atoi(cTemp)==1);

	// select format
	CXFileFormatDlg dlgFormat;
	dlgFormat.m_iMaxPatchSize=Sector.GetBorderSize();
	dlgFormat.m_fScaleFactor=fScaleFactor;
	dlgFormat.m_iPatchSize=iPatchSize;
	dlgFormat.m_iTextureExport=iTextureExport;
	dlgFormat.m_iTextureFormat=iTextureFormat;
	dlgFormat.m_iXFileFormat=iFileFormat;
	dlgFormat.m_iUpAxis=iUpAxis;
	dlgFormat.m_bGlobalPositioning=bGlobalPositioning;
	dlgFormat.m_strLightMap=strLightMap;
	dlgFormat.m_bFlipY=bFlipY;
	dlgFormat.m_iTextureMapSize=iTextureMapSize;
	dlgFormat.m_iGridSize=iReducedGridSize;
	dlgFormat.m_iReductionMethod=iReductionMethod;
	dlgFormat.m_fErrorValue=fReductionErrorValue;
	dlgFormat.m_bSimplifyBorder=bSimplifyBorder;

	// fill all light map names into the dialog
	for (unsigned long nLightMap=0;nLightMap<Terrain.GetNumLightMapDefinitions();nLightMap++)
	{
		CIPnPTCLightMapDefinition01* pLightMapDef=Terrain.GetLightMapDefinition(nLightMap);
		if (pLightMapDef)
		{
			dlgFormat.m_vecLightMapNames.push_back(std::string(pLightMapDef->GetName()));
		}
	}

	// open dialog
	if (dlgFormat.DoModal()!=IDOK)
	{
		return false;
	}

	// store settings
	CString str;
	str.Format("%f",dlgFormat.m_fScaleFactor);
	::WritePrivateProfileString(MODULE_NAME,"ScaleFactor",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iPatchSize);
	::WritePrivateProfileString(MODULE_NAME,"PatchSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureExport);
	::WritePrivateProfileString(MODULE_NAME,"TextureExport",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureFormat);
	::WritePrivateProfileString(MODULE_NAME,"TextureFormat",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iXFileFormat);
	::WritePrivateProfileString(MODULE_NAME,"FileFormat",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iUpAxis);
	::WritePrivateProfileString(MODULE_NAME,"UpAxis",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"GlobalPositioning",(dlgFormat.m_bGlobalPositioning)?"1":"0",theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"LightMap",dlgFormat.m_strLightMap,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"FlipY",(dlgFormat.m_bFlipY)?"1":"0",theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureMapSize);
	::WritePrivateProfileString(MODULE_NAME,"TextureMapSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iGridSize);
	::WritePrivateProfileString(MODULE_NAME,"ReducedGridSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iReductionMethod);
	::WritePrivateProfileString(MODULE_NAME,"ReductionMethod",str,theApp.m_strIniFile.c_str());
	str.Format("%f",dlgFormat.m_fErrorValue);
	::WritePrivateProfileString(MODULE_NAME,"ReductionErrorValue",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"SimplifyBorder",(dlgFormat.m_bSimplifyBorder)?"1":"0",theApp.m_strIniFile.c_str());

	// get last export path
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ExportPath","",cTemp,999,theApp.m_strIniFile.c_str());
	std::string strDefaultExportPath(cTemp);

	// get export file name
	const char* cSaveFile=App.SaveFileDlg("x-file","x",strDefaultExportPath.c_str());
	if (cSaveFile)
	{
		// store last path
		std::string strNewPath=GetPath(std::string(cSaveFile));
		::WritePrivateProfileString(MODULE_NAME,"ExportPath",strNewPath.c_str(),theApp.m_strIniFile.c_str());

		// show hourglass
		HCURSOR curRestore=GetCursor();
		SetCursor(LoadCursor(NULL,IDC_WAIT));

		// set vertex reduction
		this->SetSimplificationMethod(static_cast<SIMPLIFICATION_METHOD>(dlgFormat.m_iReductionMethod));
		switch (dlgFormat.m_iReductionMethod)
		{
		case 1:
			this->SetRegularGridSpacing(dlgFormat.m_iGridSize);
			break;
		case 2:
			this->SetErrorValue(fabs(dlgFormat.m_fErrorValue));
			this->SetSimplifyBorderQuads(dlgFormat.m_bSimplifyBorder);
			break;
		}

		// write sector data
		bool bReturn=this->WriteSector(App,Sector,Terrain,(CString)cSaveFile,dlgFormat.m_iXFileFormat,dlgFormat.m_fScaleFactor,dlgFormat.m_iPatchSize,dlgFormat.m_iTextureExport,dlgFormat.m_iTextureFormat,dlgFormat.m_iUpAxis,dlgFormat.m_bGlobalPositioning,dlgFormat.m_strLightMap,dlgFormat.m_bFlipY,dlgFormat.m_iTextureMapSize);

		// restore cursor
		SetCursor(curRestore);

		// return result
		return bReturn;
	}

	// an error occured
	return false;
}

bool CPnPTCXFileHMapExportPlugIn::ExportTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain)
{
	// MFC needs this
	AFX_MANAGE_STATE(AfxGetStaticModuleState());

	// read settings
	static char cTemp[1000];
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ScaleFactor","1",cTemp,999,theApp.m_strIniFile.c_str());
	float fScaleFactor=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"PatchSize","128",cTemp,999,theApp.m_strIniFile.c_str());
	int iPatchSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureExport","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureExport=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureFormat","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureFormat=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"FileFormat","1",cTemp,999,theApp.m_strIniFile.c_str());
	int iFileFormat=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"UpAxis","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iUpAxis=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"GlobalPositioning","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bGlobalPositioning=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"LightMap","",cTemp,999,theApp.m_strIniFile.c_str());
	CString strLightMap(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"FlipY","0",cTemp,999,theApp.m_strIniFile.c_str());
	bool bFlipY=(atoi(cTemp)==1);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"TextureMapSize","-1",cTemp,999,theApp.m_strIniFile.c_str());
	int iTextureMapSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReducedGridSize","1",cTemp,999,theApp.m_strIniFile.c_str());
	int iReducedGridSize=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionMethod","0",cTemp,999,theApp.m_strIniFile.c_str());
	int iReductionMethod=atoi(cTemp);
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ReductionErrorValue","0",cTemp,999,theApp.m_strIniFile.c_str());
	float fReductionErrorValue=static_cast<float>(atof(cTemp));
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"SimplifyBorder","1",cTemp,999,theApp.m_strIniFile.c_str());
	bool bSimplifyBorder=(atoi(cTemp)==1);

	// select format
	CXFileFormatDlg dlgFormat;
	dlgFormat.m_iMaxPatchSize=Terrain.GetSectorDetailBorderSize();
	dlgFormat.m_fScaleFactor=fScaleFactor;
	dlgFormat.m_iPatchSize=iPatchSize;
	dlgFormat.m_iTextureExport=iTextureExport;
	dlgFormat.m_iTextureFormat=iTextureFormat;
	dlgFormat.m_iXFileFormat=iFileFormat;
	dlgFormat.m_iUpAxis=iUpAxis;
	dlgFormat.m_bGlobalPositioning=bGlobalPositioning;
	dlgFormat.m_strLightMap=strLightMap;
	dlgFormat.m_bFlipY=bFlipY;
	dlgFormat.m_iTextureMapSize=iTextureMapSize;
	dlgFormat.m_iGridSize=iReducedGridSize;
	dlgFormat.m_iReductionMethod=iReductionMethod;
	dlgFormat.m_fErrorValue=fReductionErrorValue;
	dlgFormat.m_bSimplifyBorder=bSimplifyBorder;

	// fill all light map names into the dialog
	for (unsigned long nLightMap=0;nLightMap<Terrain.GetNumLightMapDefinitions();nLightMap++)
	{
		CIPnPTCLightMapDefinition01* pLightMapDef=Terrain.GetLightMapDefinition(nLightMap);
		if (pLightMapDef)
		{
			dlgFormat.m_vecLightMapNames.push_back(std::string(pLightMapDef->GetName()));
		}
	}

	// open dialog
	if (dlgFormat.DoModal()!=IDOK)
	{
		return false;
	}

	// store settings
	CString str;
	str.Format("%f",dlgFormat.m_fScaleFactor);
	::WritePrivateProfileString(MODULE_NAME,"ScaleFactor",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iPatchSize);
	::WritePrivateProfileString(MODULE_NAME,"PatchSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureExport);
	::WritePrivateProfileString(MODULE_NAME,"TextureExport",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureFormat);
	::WritePrivateProfileString(MODULE_NAME,"TextureFormat",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iXFileFormat);
	::WritePrivateProfileString(MODULE_NAME,"FileFormat",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iUpAxis);
	::WritePrivateProfileString(MODULE_NAME,"UpAxis",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"GlobalPositioning",(dlgFormat.m_bGlobalPositioning)?"1":"0",theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"LightMap",dlgFormat.m_strLightMap,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"FlipY",(dlgFormat.m_bFlipY)?"1":"0",theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iTextureMapSize);
	::WritePrivateProfileString(MODULE_NAME,"TextureMapSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iGridSize);
	::WritePrivateProfileString(MODULE_NAME,"ReducedGridSize",str,theApp.m_strIniFile.c_str());
	str.Format("%d",dlgFormat.m_iReductionMethod);
	::WritePrivateProfileString(MODULE_NAME,"ReductionMethod",str,theApp.m_strIniFile.c_str());
	str.Format("%f",dlgFormat.m_fErrorValue);
	::WritePrivateProfileString(MODULE_NAME,"ReductionErrorValue",str,theApp.m_strIniFile.c_str());
	::WritePrivateProfileString(MODULE_NAME,"SimplifyBorder",(dlgFormat.m_bSimplifyBorder)?"1":"0",theApp.m_strIniFile.c_str());

	// get last export path
	cTemp[0]=0;
	::GetPrivateProfileString(MODULE_NAME,"ExportPath","",cTemp,999,theApp.m_strIniFile.c_str());
	std::string strDefaultExportPath(cTemp);

	// get export path
	const char* cSavePath=App.SaveFolderDlg(strDefaultExportPath.c_str());
	if (cSavePath)
	{
		// show hourglass
		HCURSOR curRestore=GetCursor();
		SetCursor(LoadCursor(NULL,IDC_WAIT));

		// append backslash to path
		CString strPath=cSavePath;
		if (strPath.Right(1)!=(CString)"\\")
		{
			strPath+="\\";
		}

		// store storage path
		::WritePrivateProfileString(MODULE_NAME,"ExportPath",strPath,theApp.m_strIniFile.c_str());

		// get number of sectors
		unsigned int nNumSectors=Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY();

		// set progress bar
		pnptc::CProgressBar ProgressBar(App,"Exporting terrain",Terrain.GetNumSectorsX()*Terrain.GetNumSectorsY());

		// export all sectors
		unsigned int nSector=0;
		for (unsigned int nSecY=0;nSecY<Terrain.GetNumSectorsY();nSecY++)
		{
			for (unsigned int nSecX=0;nSecX<Terrain.GetNumSectorsX();nSecX++,nSector++)
			{
				// get sector to be exported
				CIPnPTCSector01* pSector=Terrain.GetSectorDetail(nSecX,nSecY);
				if (!pSector)
				{
					SetCursor(curRestore);
					CString strMessage;
					strMessage.Format("The detail sector heightmap of sector %u/%u could not be opened.",nSecX,nSecY);
					MessageBox(NULL,strMessage,"Error exporting terrain heightmap",MB_ICONSTOP|MB_TASKMODAL|MB_OK);
					return false;
				}

				// create sector file name
				CString strX;
				strX.Format("%u",nSecX);
				strX="0000"+strX;
				CString strY;
				strY.Format("%u",nSecY);
				strY="0000"+strY;
				CString strFileName=strPath+strX.Right(5)+"_"+strY.Right(5)+".x";

				// set vertex reduction
				this->SetSimplificationMethod(static_cast<SIMPLIFICATION_METHOD>(dlgFormat.m_iReductionMethod));
				switch (dlgFormat.m_iReductionMethod)
				{
				case 1:
					this->SetRegularGridSpacing(dlgFormat.m_iGridSize);
					break;
				case 2:
					this->SetErrorValue(fabs(dlgFormat.m_fErrorValue));
					this->SetSimplifyBorderQuads(dlgFormat.m_bSimplifyBorder);
					break;
				}

				// export sector
				bool bReturn=this->WriteSector(App,*pSector,Terrain,strFileName,dlgFormat.m_iXFileFormat,dlgFormat.m_fScaleFactor,dlgFormat.m_iPatchSize,dlgFormat.m_iTextureExport,dlgFormat.m_iTextureFormat,dlgFormat.m_iUpAxis,dlgFormat.m_bGlobalPositioning,dlgFormat.m_strLightMap,dlgFormat.m_bFlipY,dlgFormat.m_iTextureMapSize);

				// successfull?
				if (!bReturn)
				{
					// error exporting sector
					SetCursor(curRestore);
					return false;
				}

				// increment progress bar
				ProgressBar.IncrementPos();
			}
		}

		// ok
		SetCursor(curRestore);
		return true;
	}

	// an error occured
	return false;
}



// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------
// Write sector
// ------------------------------------------------------------------------------------------
// ------------------------------------------------------------------------------------------




bool CPnPTCXFileHMapExportPlugIn::WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,CString Filename,int iFormat,float fScale,int iPatchSize,int iTextureExport,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,const CString& strLightMap,bool bFlipY,int iExportTextureMapSize)
{
	// store export format
	this->m_iFormat=iFormat;

	// write sector with the appropriate method
	switch (iTextureExport)
	{
	case 0:
		return this->WriteSectorUntextured(App,Sector,Terrain,(LPCSTR)Filename,fScale,iPatchSize,iUpAxis,bGlobalPositioning);
		break;
	case 1:
		return this->WriteSectorDetailTexture(App,Sector,Terrain,(LPCSTR)Filename,fScale,iPatchSize,iTextureFormat,iUpAxis,bGlobalPositioning,false,"",false,iExportTextureMapSize);
		break;
	case 2:
		return this->WriteSectorTexturemap(App,Sector,Terrain,(LPCSTR)Filename,fScale,iPatchSize,iTextureFormat,iUpAxis,bGlobalPositioning,false,(LPCSTR)strLightMap,!strLightMap.IsEmpty(),false,bFlipY,iExportTextureMapSize);
		break;
	default:
		return false;
		break;
	}
}

bool CPnPTCXFileHMapExportPlugIn::SaveObject(C3DObjectData* pObj,const std::string& strFileName)
{
	// save it to a x file
	C3DObjectSerializerX serializer;
	serializer.SetSaveFormat(this->m_iFormat);
	return serializer.Save(*pObj,strFileName);
}



