/////////////////////////////////////////////////////////////////////////////////////
// PnPTCXFileHMapExportPlugIn.h
// x file heightmap exporter PlugIn definition.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//   06. Dec. 2005 : Added lightmap export possibilities
//   01. Jan. 2006 : Added y flipping of texture maps.
//   10. Mai  2008 : Changed some CString to std::string
//   12. Mai  2008 : Added texture map size option and vertex reduction.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_XFILEHMAPEXPORTPLUGIN_H_INCLUDED)
#define PNPTC_XFILEHMAPEXPORTPLUGIN_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#ifndef __AFXWIN_H__
	#error include 'stdafx.h' before including this file for PCH
#endif

#include "resource.h"		// main symbols

#include "../PnPTC3DObject/TerrainMeshCreator.h"


// --------------------------------------------------------------------------------------------
// Application class
// --------------------------------------------------------------------------------------------

// CPnPTCXFileHMapExportPlugInApp
// See PnPTCXFileHMapExportPlugIn.cpp for the implementation of this class
class CPnPTCXFileHMapExportPlugInApp : public CWinApp
{
public:
	CPnPTCXFileHMapExportPlugInApp();

	/// file storing the plugin specific settings
	std::string		m_strIniFile;

// Overrides
public:
	virtual BOOL InitInstance();

	DECLARE_MESSAGE_MAP()
};



// --------------------------------------------------------------------------------------------
// PlugIn class
// --------------------------------------------------------------------------------------------

/// generator plugin class
class CPnPTCXFileHMapExportPlugIn  : public IPnPTCExportPlugIn01, public CTerrainMeshCreator
{
public:
	/// construction / destruction
	CPnPTCXFileHMapExportPlugIn();
	virtual ~CPnPTCXFileHMapExportPlugIn();

	/// Implementation of initialization/deinitialization methods of this plugin.<br>
	/// See the description of IPnPTCExportPlugIn01 for further details.
	GUID		GetGUID();
	const char*	GetName();

	/// Implementation of the terrain exporting methods.
	/// See the description of IPnPTCExportPlugIn01 for further details.
	bool		ExportSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain);
	bool		ExportTerrain(CIPnPTCApplication01& App,CIPnPTCTerrain01& Terrain);


protected:

	/// Internal method to store one sector to a file
	bool		WriteSector(CIPnPTCApplication01& App,CIPnPTCSector01& Sector,CIPnPTCTerrain01& Terrain,CString Filename,int iFormat,float fScale,int iPatchSize,int iTextureExport,int iTextureFormat,int iUpAxis,bool bGlobalPositioning,const CString& strLightMap,bool bFlipY,int iExportTextureMapSize);

	/// saves the created object to a file
	bool		SaveObject(C3DObjectData* pObj,const std::string& strFileName);


private:

	// ----------------------------------------------------------------------
	// member variables
	// ----------------------------------------------------------------------

	/// x-file format used for export
	int			m_iFormat;

};


#endif // !defined(PNPTC_XFILEHMAPEXPORTPLUGIN_H_INCLUDED)
