/////////////////////////////////////////////////////////////////////////////////////
// XFileFormatDlg.cpp
// x file format specification dialog.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  06. Dec. 2005 : Added lightmap export possibilities
//  01. Jan. 2006 : Added y flipping of texture maps.
//  10. Oct. 2006 : Achieved VS2005 compatibility, made compilation easier.
//  12. Mai  2008 : Added texture map size option and vertex reduction.
//  23. Jun. 2008 : Added possibility to exclude border quads from terrain 
//					mesh simplification.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "PnPTCDXStudioFileHMapExportPlugIn.h"
#include ".\xfileformatdlg.h"



IMPLEMENT_DYNAMIC(CXFileFormatDlg, CDialog)
CXFileFormatDlg::CXFileFormatDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CXFileFormatDlg::IDD, pParent)
{
	// initializations
	this->m_fScaleFactor=1.f;
	this->m_iPatchSize=1024;
	this->m_iMaxPatchSize=1024;
	this->m_iTextureExport=0;
	this->m_iTextureFormat=0;
	this->m_iXFileFormat=1;
	this->m_iUpAxis=0;
	this->m_bGlobalPositioning=false;
	this->m_strLightMap="";
	this->m_bFlipY=false;
	this->m_iReductionMethod=0;
	this->m_iGridSize=1;
	this->m_fErrorValue=0;
	this->m_iTextureMapSize=-1;
	this->m_bSimplifyBorder=true;
}

CXFileFormatDlg::~CXFileFormatDlg()
{
}

void CXFileFormatDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Radio(pDX, IDC_XFILE_FORMAT, m_iXFileFormat);
	DDX_Control(pDX, IDC_COMBO_PATCHSIZE, m_combPatchSize);
	DDX_Control(pDX, IDC_COMBO_TEXTUREEXPORT, m_combTextureExport);
	DDX_Control(pDX, IDC_COMBO_TEXTUREFORMAT, m_combTextureFormat);
	DDX_Control(pDX, IDC_TXT_SCALEFACTOR, m_txtFactor);
	DDX_Control(pDX, IDC_COMBO_UPAXIS, m_combUpAxis);
	DDX_Control(pDX, IDC_CHK_GLOBALPOSITIONING, m_chkGlobalPositioning);
	DDX_Control(pDX, IDC_COMBO_LIGHTMAP, m_combLightMap);
	DDX_Control(pDX, IDC_CHK_FLIPY, m_chkFlipY);
	DDX_Control(pDX, IDC_CHK_SIMPLIFYBORDER, m_chkSimplifyBorder);
	DDX_Control(pDX, IDC_COMBO_TEXTUREMAPSIZE, m_combTextureMapSize);
	DDX_Control(pDX, IDC_COMBO_REDUCTIONMETHOD, m_combReductionMethod);
	DDX_Control(pDX, IDC_COMBO_GRIDSIZE, m_combGridSize);
	DDX_Control(pDX, IDC_TXT_ERRORVALUE, m_txtErrorValue);
}


BEGIN_MESSAGE_MAP(CXFileFormatDlg, CDialog)
	ON_CBN_SELCHANGE(IDC_COMBO_TEXTUREEXPORT, OnCbnSelchangeComboTextureexport)
	ON_CBN_SELCHANGE(IDC_COMBO_REDUCTIONMETHOD, OnCbnSelchangeComboReductionmethod)
	ON_CBN_SELCHANGE(IDC_COMBO_PATCHSIZE, OnCbnSelchangeComboPatchsize)
	ON_BN_CLICKED(IDC_CHK_GLOBALPOSITIONING, OnBnClickedChkGlobalpositioning)
END_MESSAGE_MAP()



BOOL CXFileFormatDlg::OnInitDialog()
{
	CDialog::OnInitDialog();

	// initialize text boxes
	this->m_txtFactor.SetAllowNegatives(false);
	this->m_txtFactor.SetAllowDecimals(true);
	this->m_txtFactor.SetValue(this->m_fScaleFactor);

	// fill patch and texture map size combo
	this->m_combPatchSize.ResetContent();
	this->m_combTextureMapSize.ResetContent();
	int iNewIndex=this->m_combTextureMapSize.AddString("Use patch size");
	if (iNewIndex>=0)
	{
		this->m_combTextureMapSize.SetItemData(iNewIndex,-1);
	}
	for (int iPatchSize=64;iPatchSize<=this->m_iMaxPatchSize;iPatchSize*=2)
	{
		CString str;
		str.Format("%d",iPatchSize);
		iNewIndex=this->m_combPatchSize.AddString(str);
		if (iNewIndex>=0)
		{
			this->m_combPatchSize.SetItemData(iNewIndex,iPatchSize);
		}
		iNewIndex=this->m_combTextureMapSize.AddString(str);
		if (iNewIndex>=0)
		{
			this->m_combTextureMapSize.SetItemData(iNewIndex,iPatchSize);
		}
	}

	// initialize combos
	CString str;
	str.Format("%d",this->m_iPatchSize);
	this->m_combPatchSize.SelectString(-1,str);
	this->m_combTextureExport.SetCurSel(this->m_iTextureExport);
	this->m_combTextureFormat.SetCurSel(this->m_iTextureFormat);
	this->m_combUpAxis.SetCurSel(this->m_iUpAxis);
	if (this->m_iTextureMapSize==-1)
	{
		this->m_combTextureMapSize.SetCurSel(0);
	}
	else
	{
		str.Format("%d",this->m_iTextureMapSize);
		this->m_combTextureMapSize.SelectString(-1,str);
	}
	this->m_combGridSize.AddString("1");
	this->m_combGridSize.AddString("2");
	this->m_combGridSize.AddString("4");
	this->m_combGridSize.AddString("8");
	this->m_combGridSize.AddString("16");
	this->m_combGridSize.AddString("32");
	this->FillReductionMethodCombo();
	this->FillTextureMapSizeCombo();

	// initialize light map combo name
	this->m_combLightMap.ResetContent();
	this->m_combLightMap.AddString("(No lightmap)");
	for (unsigned long n=0;n<this->m_vecLightMapNames.size();n++)
	{
		this->m_combLightMap.AddString(this->m_vecLightMapNames[n].c_str());
	}
	this->m_combLightMap.SelectString(-1,this->m_strLightMap);
	if (this->m_combLightMap.GetCurSel()<0) this->m_combLightMap.SelectString(-1,"(No lightmap)");
	if (this->m_combLightMap.GetCurSel()<0) this->m_combLightMap.SetCurSel(0);

	// initialize check boxes
	this->m_chkGlobalPositioning.SetCheck((this->m_bGlobalPositioning)?BST_CHECKED:BST_UNCHECKED);
	this->m_chkFlipY.SetCheck((this->m_bFlipY)?BST_CHECKED:BST_UNCHECKED);
	this->m_chkSimplifyBorder.SetCheck((this->m_bSimplifyBorder)?BST_CHECKED:BST_UNCHECKED);

	// set reduction values
	str.Format("%d",this->m_iGridSize);
	this->m_combGridSize.SelectString(-1,str);
	for (int i=0;i<this->m_combReductionMethod.GetCount();++i)
	{
		if (this->m_combReductionMethod.GetItemData(i)==this->m_iReductionMethod)
		{
			this->m_combReductionMethod.SetCurSel(i);
			break;
		}
	}
	this->m_txtErrorValue.SetValue(this->m_fErrorValue);

	// ensure anything is selected
	if (this->m_combPatchSize.GetCurSel()<0) this->m_combPatchSize.SetCurSel(0);
	if (this->m_combTextureExport.GetCurSel()<0) this->m_combTextureExport.SetCurSel(0);
	if (this->m_combTextureFormat.GetCurSel()<0) this->m_combTextureFormat.SetCurSel(0);
	if (this->m_combUpAxis.GetCurSel()<0) this->m_combUpAxis.SetCurSel(0);
	if (this->m_combTextureMapSize.GetCurSel()<0) this->m_combTextureMapSize.SetCurSel(0);
	if (this->m_combGridSize.GetCurSel()<0) this->m_combGridSize.SetCurSel(0);
	if (this->m_combReductionMethod.GetCurSel()<0) this->m_combReductionMethod.SetCurSel(0);

	// enable texture and reduction export controls
	this->EnableTextureControls();
	this->EnableReductionControls();

	return TRUE;  // return TRUE unless you set the focus to a control
	// EXCEPTION: OCX Property Pages should return FALSE
}

void CXFileFormatDlg::OnOK()
{
	// get patch size
	int iSelected=this->m_combPatchSize.GetCurSel();
	if (iSelected>=0)
	{
		this->m_iPatchSize=static_cast<int>(this->m_combPatchSize.GetItemData(iSelected));
	}

	// read combo values
	this->m_iTextureExport=this->m_combTextureExport.GetCurSel();
	this->m_iTextureFormat=this->m_combTextureFormat.GetCurSel();
	this->m_iUpAxis=this->m_combUpAxis.GetCurSel();

	// get texture map size
	iSelected=this->m_combTextureMapSize.GetCurSel();
	if (iSelected>=0)
	{
		this->m_iTextureMapSize=static_cast<int>(this->m_combTextureMapSize.GetItemData(iSelected));
	}

	// get light map name
	this->m_combLightMap.GetWindowText(this->m_strLightMap);
	if (this->m_strLightMap.CompareNoCase("(No lightmap)")==0) this->m_strLightMap="";

	// read check boxes
	this->m_bGlobalPositioning=(this->m_chkGlobalPositioning.GetCheck()==BST_CHECKED);
	this->m_bFlipY=(this->m_chkFlipY.GetCheck()==BST_CHECKED);
	this->m_bSimplifyBorder=(this->m_chkSimplifyBorder.GetCheck()==BST_CHECKED);

	// get reduction values
	iSelected=this->m_combReductionMethod.GetCurSel();
	if (iSelected>=0)
	{
		this->m_iReductionMethod=static_cast<int>(this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel()));
	}
	iSelected=this->m_combGridSize.GetCurSel();
	if (iSelected>=0)
	{
		CString strSelected;
		this->m_combGridSize.GetWindowText(strSelected);
		this->m_iGridSize=atoi((LPCSTR)strSelected);
	}
	this->m_fErrorValue=this->m_txtErrorValue.GetValue();

	// get scale factor
	this->m_fScaleFactor=this->m_txtFactor.GetValue();

	// close dialog
	CDialog::OnOK();
}

void CXFileFormatDlg::EnableTextureControls()
{
	// controls are only enabled, if texture export is active
	this->m_combTextureFormat.EnableWindow(this->m_combTextureExport.GetCurSel()>=1);
	this->m_combLightMap.EnableWindow(this->m_combTextureExport.GetCurSel()==2);
	this->m_chkFlipY.EnableWindow(this->m_combTextureExport.GetCurSel()==2);
	this->m_combTextureMapSize.EnableWindow(this->m_combTextureExport.GetCurSel()==2);
}

void CXFileFormatDlg::EnableReductionControls()
{
	// enable vertex reduction controls
	this->m_combGridSize.EnableWindow(this->m_combReductionMethod.GetCurSel()>=0 && this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel())==1);
	this->m_txtErrorValue.EnableWindow(this->m_combReductionMethod.GetCurSel()>=0 && this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel())==2);
	this->m_chkSimplifyBorder.EnableWindow(this->m_combReductionMethod.GetCurSel()>=0 && this->m_combReductionMethod.GetItemData(this->m_combReductionMethod.GetCurSel())==2);
}

void CXFileFormatDlg::OnCbnSelchangeComboTextureexport()
{
	// enable texture export controls
	this->EnableTextureControls();

	// regular reduction is not possible for detail blended textures
	CString strReductionMethod;
	this->m_combReductionMethod.GetWindowText(strReductionMethod);
	this->FillReductionMethodCombo();
	this->m_combReductionMethod.SelectString(-1,strReductionMethod);
	if (this->m_combReductionMethod.GetCurSel()<0)
	{
		this->m_combReductionMethod.SetCurSel(0);
		this->EnableReductionControls();
	}
}

void CXFileFormatDlg::OnCbnSelchangeComboReductionmethod()
{
	// enable reduction controls
	this->EnableReductionControls();
}

void CXFileFormatDlg::OnCbnSelchangeComboPatchsize()
{
	// refill texture map size combo
	this->FillTextureMapSizeCombo();
}

void CXFileFormatDlg::FillReductionMethodCombo()
{
	// fill reduction methods
	this->m_combReductionMethod.ResetContent();
	int iNewItem=this->m_combReductionMethod.AddString("no reduction");
	if (iNewItem>=0) this->m_combReductionMethod.SetItemData(iNewItem,0);
	if (this->m_combTextureExport.GetCurSel()!=1)
	{
		iNewItem=this->m_combReductionMethod.AddString("regular");
		if (iNewItem>=0) this->m_combReductionMethod.SetItemData(iNewItem,1);
	}
	iNewItem=this->m_combReductionMethod.AddString("height deviation");
	if (iNewItem>=0) this->m_combReductionMethod.SetItemData(iNewItem,2);
}

void CXFileFormatDlg::FillTextureMapSizeCombo()
{
	// get current patch size
	int iPatchSize=this->m_iMaxPatchSize;
	if (this->m_combPatchSize.GetCurSel()>=0)
	{
		iPatchSize=static_cast<int>(this->m_combPatchSize.GetItemData(this->m_combPatchSize.GetCurSel()));
	}

	// refill texture map size combo
	int iLastSelection=this->m_combTextureMapSize.GetCurSel();
	this->m_combTextureMapSize.ResetContent();
	int iNewIndex=this->m_combTextureMapSize.AddString("Use patch size");
	if (iNewIndex>=0)
	{
		this->m_combTextureMapSize.SetItemData(iNewIndex,-1);
	}
	for (int iTextureMapSize=64;iTextureMapSize<=iPatchSize;iTextureMapSize*=2)
	{
		CString str;
		str.Format("%d",iTextureMapSize);
		iNewIndex=this->m_combTextureMapSize.AddString(str);
		if (iNewIndex>=0)
		{
			this->m_combTextureMapSize.SetItemData(iNewIndex,iTextureMapSize);
		}
	}

	// restore selection
	this->m_combTextureMapSize.SetCurSel(iLastSelection);
	if (this->m_combTextureMapSize.GetCurSel()<0) this->m_combTextureMapSize.SetCurSel(0);
}


void CXFileFormatDlg::OnBnClickedChkGlobalpositioning()
{
	// TODO: Fgen Sie hier Ihren Kontrollbehandlungscode fr die Benachrichtigung ein.
}
