/////////////////////////////////////////////////////////////////////////////////////
// XFileFormatDlg.h
// x file format specification dialog.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  06. Dec. 2005 : Added lightmap export possibilities
//  01. Jan. 2006 : Added y flipping of texture maps.
//  12. Mai  2008 : Added texture map size option and vertex reduction.
//  23. Jun. 2008 : Added possibility to exclude border quads from terrain 
//					mesh simplification.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#pragma once
#include "afxwin.h"
#include "../PnPTCUtils/Controls/NumericEdit.h"


// CXFileFormatDlg dialog

class CXFileFormatDlg : public CDialog
{
	DECLARE_DYNAMIC(CXFileFormatDlg)

public:
	CXFileFormatDlg(CWnd* pParent = NULL);   // standard constructor
	virtual ~CXFileFormatDlg();

	/// list of all available light map names
	std::vector<std::string>	m_vecLightMapNames;

protected:

	/// Enable controls which depend on the texture export setting.
	void	EnableTextureControls();

	/// Enable controls which depend on the vertex reduction setting.
	void	EnableReductionControls();

	/// Fills the combo box with the available reduction methods.
	void	FillReductionMethodCombo();

	/// Fills the texturemap size combo.
	void	FillTextureMapSizeCombo();


// Dialog Data
	enum { IDD = IDD_XFILE_FORMAT_DLG };

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support

	DECLARE_MESSAGE_MAP()
public:
	int m_iMaxPatchSize;
	int m_iXFileFormat;
	int m_iPatchSize;
	int m_iTextureExport;
	int m_iTextureFormat;
	int m_iUpAxis;
	bool m_bGlobalPositioning;
	float m_fScaleFactor;
	bool m_bFlipY;
	bool m_bSimplifyBorder;
	int m_iTextureMapSize;
	int m_iGridSize;
	float m_fErrorValue;
	int m_iReductionMethod;
	CString m_strLightMap;
	CComboBox m_combPatchSize;
	CComboBox m_combTextureExport;
	CComboBox m_combTextureFormat;
	CComboBox m_combLightMap;
	CNumericEdit m_txtFactor;
	CComboBox m_combUpAxis;
	CButton m_chkGlobalPositioning;
	CButton m_chkFlipY;
	CButton m_chkSimplifyBorder;
	CComboBox m_combTextureMapSize;
	CComboBox m_combReductionMethod;
	CComboBox m_combGridSize;
	CNumericEdit m_txtErrorValue;
	virtual BOOL OnInitDialog();
	afx_msg void OnCbnSelchangeComboTextureexport();
	afx_msg void OnCbnSelchangeComboReductionmethod();
	afx_msg void OnCbnSelchangeComboPatchsize();
protected:
	virtual void OnOK();
public:
	afx_msg void OnBnClickedChkGlobalpositioning();
};
