/////////////////////////////////////////////////////////////////////////////////////
// CategorizedDataEnumeration.cpp
// Enumerates categorized data in a given directory.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation: 05. Mar. 2007 (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
//  07. Jul. 2007 :	Doesn't require MFC any more.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////


#ifndef WINVER
#if _MSC_VER >= 1300
#define WINVER 0x0500
#else
#define WINVER 0x0400
#endif
#endif


#include "CategorizedDataEnumeration.h"
#include <io.h>
#include "errno.h"

unsigned int CCategorizedDataEnumeration::EnumerateData(const std::string& strPath,const std::string& strCategory)
{
	// number of found files
	unsigned int nCounter=0;

	// find all files
	struct _finddata_t c_file;
	intptr_t nFind=_findfirst((strPath+"*.*").c_str(),&c_file);
	if (nFind!=ENOENT && nFind!=EINVAL)
	{
		// go through all files
		bool bContinue=true;
		while (bContinue)
		{
			// get info, no hidden files/directories
			if ((c_file.attrib & _A_HIDDEN)==0)
			{
				// is it a directory?
				if ((c_file.attrib & _A_SUBDIR)!=0)
				{
					// make sure it isn't the current or the parent dir
					if (std::string(c_file.name)!="." && std::string(c_file.name)!="..")
					{
						// scan that directory
						std::string strNewPath=strPath+c_file.name+"\\";
						std::string strNewCategory=strCategory;
						if (!strNewCategory.empty())
						{
							strNewCategory+="\\";
						}
						strNewCategory+=c_file.name;
						nCounter+=this->EnumerateData(strNewPath,strNewCategory);
					}
				}
				else
				{
					// add this file
					if (this->AddDataFile(strPath+c_file.name,(!strCategory.empty())?strCategory:"default category"))
					{
						nCounter++;
					}
				}
			}

			// get next file
			bContinue=(_findnext(nFind,&c_file)==0);
		}

		// close file search
		_findclose(nFind);
	}

	// return number of found files
	return nCounter;
}

