/////////////////////////////////////////////////////////////////////////////////////
// CategorizedDataEnumeration.h
// Enumerates categorized data in a given directory.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation: 05. Mar. 2007 (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
//
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_CATEGORIZEDDATAENUMERATION_H_INCLUDED)
#define PNPTC_CATEGORIZEDDATAENUMERATION_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


#include <string>


class CCategorizedDataEnumeration
{
protected:

	/// Enumerates categorized data in a given directory.
	/// @param strPath Path in which categorized data files should be searched.
	/// @param strCategory Current category for the files in the path.
	/// @return Returns the number of files which have been added.
	unsigned int	EnumerateData(const std::string& strPath,const std::string& strCategory);

	/// Callback method used to add a categorized data file.
	/// @param strFilePath The name including the path of the file to be added.
	/// @param strCategory The category into which the file should be added.
	/// @return Returns <i>true</i> or <i>false</i>, whether or not the
	/// file could be added.
	virtual bool	AddDataFile(const std::string& strFilePath,const std::string& strCategory)=0;

};	// class CategorizedDataEnumeration



#endif // !defined(PNPTC_CATEGORIZEDDATAENUMERATION_H_INCLUDED)
