/////////////////////////////////////////////////////////////////////////////////////
// DXUtils.h
// various utility methods to support common plugin functionality working
// with DirectX data types and methods.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  03. Dec. 2008 :	Added GlobalVectorTransform
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_DXUTILS_H_INCLUDED)
#define PNPTC_DXUTILS_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// ----------------------------------------------------------------
// global includes
// ----------------------------------------------------------------

#include <d3dx9.h>
#include "../../includes/IPnPTCPlugIn.h"
#include <string>
#include "utils.h"

// ----------------------------------------------------------------
// color conversion
// ----------------------------------------------------------------

inline D3DXCOLOR TCColor2DXColor(const PNPTC_RGBA& col)
{
	return D3DXCOLOR(static_cast<float>(col.r)/255.f,static_cast<float>(col.g)/255.f,static_cast<float>(col.b)/255.f,static_cast<float>(col.a)/255.f);
}

inline D3DXCOLOR TCColor2DXColor(const PNPTC_RGB& col)
{
	return D3DXCOLOR(static_cast<float>(col.r)/255.f,static_cast<float>(col.g)/255.f,static_cast<float>(col.b)/255.f,1);
}

inline PNPTC_RGBA DXColor2TCColor(const D3DXCOLOR& col)
{
	return PNPTC_RGBA(static_cast<unsigned char>(col.r*255.f),static_cast<unsigned char>(col.g*255.f),static_cast<unsigned char>(col.b*255.f),static_cast<unsigned char>(col.a*255.f));
}

inline D3DXIMAGE_FILEFORMAT GetImageFormat(const std::string& strFileName)
{
	// check all supported formats
	if (EqualNoCase(Right(strFileName,3),"bmp"))
	{
		return D3DXIFF_BMP;
	}
	else if (EqualNoCase(Right(strFileName,3),"jpg") || EqualNoCase(Right(strFileName,4),"jpeg"))
	{
		return D3DXIFF_JPG;
	}
	else if (EqualNoCase(Right(strFileName,3),"png"))
	{
		return D3DXIFF_PNG;
	}
	else if (EqualNoCase(Right(strFileName,3),"dds"))
	{
		return D3DXIFF_DDS;
	}
	else if (EqualNoCase(Right(strFileName,3),"tga"))
	{
		return D3DXIFF_TGA;
	}
	else if (EqualNoCase(Right(strFileName,3),"ppm"))
	{
		return D3DXIFF_PPM;
	}
	else if (EqualNoCase(Right(strFileName,3),"dib"))
	{
		return D3DXIFF_DIB;
	}
	else if (EqualNoCase(Right(strFileName,3),"hdr"))
	{
		return D3DXIFF_HDR;
	}
	else if (EqualNoCase(Right(strFileName,3),"pfm"))
	{
		return D3DXIFF_PFM;
	}
	else
	{
		return D3DXIFF_BMP;
	}
}

/// Affects a 3D vector by a given global transformation.
/// @param vecTransform The vector, which should be transformed.
/// @param matTransform A transformation matrix to be applied.
/// @param fScale A scaling factor, which will be applied to all vertex
/// positions during export.
/// @param iUpRotation A rotation, which will be applied to all vertices
/// in order to correclty align the object's up axis. Possible values are
/// UPROTATION_NONE, UPROTATION_ROTX90, UPROTATION_ROTX270, and UPROTATION_ROTX180.
/// @param iFlipAxis The axis along which all vertex positions are negated
/// during the export. Possible values are FLIP_XAXIS, FLIP_YAXIS, and FLIP_ZAXIS.
void		GlobalVectorTransform(PnPTC3DVec& vecTransform,const D3DXMATRIX& matTransform,float fScale,int iUpRotation,int iFlipAxis);


// ----------------------------------------------------------------
// textures
// ----------------------------------------------------------------


/// compresses a texture into DX5 format.
/// @param strTexture The name of the texture file to be compressed.
/// @param strFormat The format into which the texture should be compressed
/// @return Returns <i>true</i> or <i>false</i>, whether or not the
/// file could be compressed successfully.
bool DXCompressTexture(const std::string& strTexture,const std::string& strFormat="DXT5");


#endif // !defined(PNPTC_DXUTILS_H_INCLUDED)