/////////////////////////////////////////////////////////////////////////////////////
// ProgressBar.h
// Resource management class for the application's progress bar.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:	30. Mai 2007
// ----------------------------------------------------------------------------------
// change history:
//  23. Nov. 2008	: Improved application responsiveness during import/export
//                    by peeking the message map.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "..\..\includes\CIPnPTCApplication.h"
#include ".\progressbar.h"

/// PnP TerrainCreator name space
namespace pnptc
{

void ProcessWindowsMessages()
{
	MSG msg;
	if (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
	{
		TranslateMessage(&msg);
		DispatchMessage(&msg);
	}
}

CProgressBar::CProgressBar(CIPnPTCApplication01& App,const char* cTaskName,unsigned int nIncrements/*=100*/)
	: m_iTaskID(-1)
	, m_App(App)
{
	// try to create the progress bar
	this->m_iTaskID=App.DisplayProgressBar(cTaskName,nIncrements);
}

CProgressBar::~CProgressBar(void)
{
	// hide progress bar
	if (this->m_iTaskID>=0)
	{
		this->m_App.HideProgressBar(this->m_iTaskID);
		this->m_iTaskID=-1;
	}
}

void CProgressBar::SetPos(unsigned int nPercentage,const char* cText/*=NULL*/) const
{
	// hide progress bar
	if (this->m_iTaskID>=0)
	{
		this->m_App.SetProgressBarPos(this->m_iTaskID,nPercentage,cText);
	}

	// process windows messages
	ProcessWindowsMessages();
}

void CProgressBar::IncrementPos(const char* cText/*=NULL*/) const
{
	// hide progress bar
	if (this->m_iTaskID>=0)
	{
		this->m_App.IncrementProgressBarPos(this->m_iTaskID,cText);
	}

	// process windows messages
	ProcessWindowsMessages();
}


};	// namespace pnptc