/////////////////////////////////////////////////////////////////////////////////////
// ProgressBar.h
// Resource management class for the application's progress bar.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:	30. Mai 2007
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////


#if !defined(PNPTC_PROGRESSBAR_H_INCLUDED)
#define PNPTC_PROGRESSBAR_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



// forward definitions
class CIPnPTCApplication01;



/// PnP TerrainCreator name space
namespace pnptc
{


/// A resource management class for the application's progress bar.
class CProgressBar
{
public:

	// -------------------------------------------------------------------
	// construction / destruction
	// -------------------------------------------------------------------

	/// Construction of a new progress bar.
	/// @param App Reference to the PnPTC application object.
	/// @param cTaskName Name of the task which will be displayed for this progress bar.
	/// @param nIncrements Number of increments of this progress bar.
	CProgressBar(CIPnPTCApplication01& App,const char* cTaskName,unsigned int nIncrements=100);

	/// Destruction of the progress bar
	~CProgressBar(void);


	// -------------------------------------------------------------------
	// progress bar manipulation
	// -------------------------------------------------------------------

	/// @brief Sets the current value of the application's progress bar.
	///
	/// Updates the application's progress bar for a given task ID
	/// to the specified percentage value.
	/// @param nPercentage Percentage [0..100] of the progress bar to be displayed.
	/// @param cText (optional) Additional text to be display in the progress window.
	void			SetPos(unsigned int nPercentage,const char* cText=NULL) const;

	/// @brief Increments the progress bar.
	///
	/// The progress bar is incremented by one position. The progress bar can
	/// be incremented until the number of increments set by the constructor
	/// method are reached. Further incrementing has no affect.
	/// @param cText (optional) Additional text to be display in the progress window.
	void			IncrementPos(const char* cText=NULL) const;


private:

	// -------------------------------------------------------------------
	// member variables
	// -------------------------------------------------------------------

	/// The task ID of the progress bar.
	int						m_iTaskID;

	/// Reference to the PnPTC application object.
	CIPnPTCApplication01&	m_App;


};	// class CProgressBar

};	// namespace pnptc

#endif	// !defined(PNPTC_PROGRESSBAR_H_INCLUDED)
