/////////////////////////////////////////////////////////////////////////////////////
// QuatInterpolator.h
// Intrepolates betwen the four border point values of a quad.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:
//  02. Feb. 2007 : (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
//
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_QUATINTERPOLATOR_H_INCLUDED)
#define PNPTC_QUATINTERPOLATOR_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000



template<class T>
class CQuadInterpolator
{
public:

	/// interpolates between the four border point values of a quad.
	/// @param edges The four border points of the quad in the order
	/// bottom left, bottom right, top left, top right.
	/// @param fX The interpolation value in x direction (between the
	/// left and right border) in the range [0..1].
	/// @param fY The interpolation value in y direction (between the
	/// bottom and top border) in the range [0..1].
	/// @param result (output) The result of the interpolation.
	static void	Interpolate(T edges[4],float fX,float fY,T& result);
};





template<class T>
void CQuadInterpolator<T>::Interpolate(T edges[4],float fX,float fY,T& result)
{
	// interpolate over x
	T t1=edges[1]*fX+edges[0]*(1-fX);
	T t2=edges[3]*fX+edges[2]*(1-fX);

	// interpolate over y
	result=t2*fY+t1*(1-fY);
};



#endif // !defined(PNPTC_QUATINTERPOLATOR_H_INCLUDED)