/////////////////////////////////////////////////////////////////////////////////////
// ReferenceCount.h
// Definition of a base class for reference counted objects.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////
#if !defined(PNPTC_REFERENCECOUNT_H_INCLUDED)
#define PNPTC_REFERENCECOUNT_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/// Base class for reference counted objects.
class CReferenceCount
{
public:

	// --------------------------------------------------------------
	// construction / destruction
	// --------------------------------------------------------------

	/// Construction of a new reference counted object.
	CReferenceCount(void)
		: m_iRefCount(0)
	{};

	/// Destruction of the reference counted object.
	~CReferenceCount(void) {};


	// --------------------------------------------------------------
	// reference counting
	// --------------------------------------------------------------

	/// Increments the reference counter.
	/// @return Returns the current reference counter value after adding a reference.
	virtual inline int		AddReference() {this->m_iRefCount++;return this->m_iRefCount;};

	/// Decrements the reference counter.
	/// @return Returns the current reference counter value after release of a reference.
	virtual inline int		ReleaseReference() {if (this->m_iRefCount>0) this->m_iRefCount--;return this->m_iRefCount;};

	/// Returns the current reference counter.
	/// @return Returns the current reference counter value.
	inline int				GetReferenceCount() const {return this->m_iRefCount;};


private:

	// --------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------

	/// The current reference counter
	int				m_iRefCount;
};

#endif // !defined(PNPTC_REFERENCECOUNT_H_INCLUDED)