/////////////////////////////////////////////////////////////////////////////////////
// TempFileManager.cpp
// Manages all temporary files including the necessary clean-up.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation
//  29. Aug. 2008 (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "TempFileManager.h"
#include "Utils.h"

CTempFileManager::CTempFileManager()
{
	// return system temp path
	char szTempName[MAX_PATH];
	::GetTempPath(MAX_PATH,szTempName);
	this->m_strTempPath=szTempName;
	if (Right(this->m_strTempPath,1)!="\\")
	{
		this->m_strTempPath+="\\";
	}

	// make sure, the temp path exists
	CreateDir(this->m_strTempPath);
}

CTempFileManager::~CTempFileManager()
{
	// delete all files
	for (std::vector<std::string>::iterator itFile=this->m_vecTempFiles.begin();itFile!=this->m_vecTempFiles.end();++itFile)
	{
		::remove(itFile->c_str());
	}
	this->m_vecTempFiles.clear();
}

void CTempFileManager::SetTempPath(const std::string& strTempPath)
{
	// store the path name
	this->m_strTempPath=strTempPath;
	if (Right(this->m_strTempPath,1)!="\\")
	{
		this->m_strTempPath+="\\";
	}

	// make sure, the temp path exists
	CreateDir(this->m_strTempPath);
}

std::string CTempFileManager::CreateTempFile(const std::string& strPrefix,unsigned int nNumber/*=0*/)
{
	// create the file
	char szTempName[MAX_PATH];
	::GetTempFileName(this->m_strTempPath.c_str(), strPrefix.c_str(), nNumber, szTempName);
	std::string strTempName=szTempName;

	// check valid string
	if (!strTempName.empty())
	{
		// create that file
		HANDLE hTempFile = ::CreateFile(strTempName.c_str(),		  // file name 
										GENERIC_READ | GENERIC_WRITE, // open for read/write 
										0,                            // do not share 
										NULL,                         // default security 
										CREATE_ALWAYS,                // overwrite existing file
										FILE_ATTRIBUTE_NORMAL,        // normal file 
										NULL);                        // no template 
		if (hTempFile)
		{
			// add to list and close file handle
			this->m_vecTempFiles.push_back(strTempName);
			::CloseHandle(hTempFile); 
		}
		else
		{
			// error
			strTempName="";
		}
	}

	// return that file name
	return strTempName;
}
