/////////////////////////////////////////////////////////////////////////////////////
// TempFileManager.h
// Manages all temporary files including the necessary clean-up.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation
//  29. Aug. 2008 (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_TEMPFILEMANAGER_H_INCLUDED)
#define PNPTC_TEMPFILEMANAGER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

// includes
#include <vector>
#include <string>


/// The temporary file manager class.
/// Creates new temporary files and automatically
/// deletes all created files, when the manager
/// instance is destructed.
class CTempFileManager
{

public:

	// ---------------------------------------------------------------------------
	// construction / destruction
	// ---------------------------------------------------------------------------

	/// Construction of a new temporary file manager instance.
	CTempFileManager();

	/// Destruction of the temporary file manager instance.
	/// The deletion of the temporary files is performed during
	/// the instance destruction
	virtual ~CTempFileManager();


	// ---------------------------------------------------------------------------
	// temporary file management
	// ---------------------------------------------------------------------------

	/// Sets the path below which temporary files should be stored.
	/// @param strTempPath (in) The temporary path, below which temporary files should be stored.
	void		SetTempPath(const std::string& strTempPath);

	/// Creates a new temporary file.
	/// @param strPrefix (in) The prefix of the temporary file to be created.
	/// Can have up to three characters.
	/// @param nNumber (in) A number to be used for the temporary file.
	/// If <i>0</i>, a number is created automatically based on the current
	/// PC time. If a temporary file with the given number already exists,
	/// the next free number will be used instead.
	/// @return Returns the name of the created temporary file including
	/// the full path to this file. In case of an error, an empty
	/// string is returned.
	std::string	CreateTempFile(const std::string& strPrefix,unsigned int nNumber=0);


private:

	// ---------------------------------------------------------------------------
	// member variables
	// ---------------------------------------------------------------------------

	/// The temp file path to be used.
	std::string					m_strTempPath;

	/// The list of all temporary files, which habe been created.
	std::vector<std::string>	m_vecTempFiles;

};	// class CTempFileManager



#endif // !defined(PNPTC_TEMPFILEMANAGER_H_INCLUDED)
