/////////////////////////////////////////////////////////////////////////////////////
// AreaDisplayCtrl.cpp
// MFC control to visualize overlapping regions during terrain import
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:
//   02. Feb. 2007 (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
//
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#ifndef WINVER
#if _MSC_VER >= 1300
#define WINVER 0x0500
#else
#define WINVER 0x0400
#endif
#endif

#include ".\AreaDisplayCtrl.h"
#include "../pnpse/pnpserect.h"

const int BORDER_OFFSET=2;
const int BORDER_OFFSET2=2*BORDER_OFFSET;

// CAreaDisplayCtrl

IMPLEMENT_DYNAMIC(CAreaDisplayCtrl, CStatic)
CAreaDisplayCtrl::CAreaDisplayCtrl()
	: m_fSourceSizeX(1)
	, m_fSourceSizeY(1)
	, m_fSourceOffsetX(1)
	, m_fSourceOffsetY(1)
	, m_fSourceImportSizeX(1)
	, m_fSourceImportSizeY(1)
	, m_fDestSizeX(1)
	, m_fDestSizeY(1)
	, m_fDestOffsetX(1)
	, m_fDestOffsetY(1)
	, m_fDestImportSizeX(1)
	, m_fDestImportSizeY(1)
{
}

CAreaDisplayCtrl::~CAreaDisplayCtrl()
{
}


BEGIN_MESSAGE_MAP(CAreaDisplayCtrl, CStatic)
	ON_WM_PAINT()
END_MESSAGE_MAP()



// CAreaDisplayCtrl-Meldungshandler

void CAreaDisplayCtrl::SetDisplaySourceParameters(float fSourceSizeX,float fSourceSizeY,float fSourceOffsetX,float fSourceOffsetY,float fSourceImportSizeX,float fSourceImportSizeY)
{
	// set values
	this->m_fSourceSizeX=fSourceSizeX;
	this->m_fSourceSizeY=fSourceSizeY;
	this->m_fSourceOffsetX=fSourceOffsetX;
	this->m_fSourceOffsetY=fSourceOffsetY;
	this->m_fSourceImportSizeX=fSourceImportSizeX;
	this->m_fSourceImportSizeY=fSourceImportSizeY;
}

void CAreaDisplayCtrl::SetDisplayDestParameters(float fDestSizeX,float fDestSizeY,float fDestOffsetX,float fDestOffsetY,float fDestImportSizeX,float fDestImportSizeY)
{
	// set values
	this->m_fDestSizeX=fDestSizeX;
	this->m_fDestSizeY=fDestSizeY;
	this->m_fDestOffsetX=fDestOffsetX;
	this->m_fDestOffsetY=fDestOffsetY;
	this->m_fDestImportSizeX=fDestImportSizeX;
	this->m_fDestImportSizeY=fDestImportSizeY;
}

void CAreaDisplayCtrl::OnPaint()
{
	CPaintDC dc(this); // device context for painting

	// get size
	CRect rc;
	this->GetClientRect(&rc);

	// get scale between source and dest
	float fSourceDestScaleX=(this->m_fSourceImportSizeX!=0)?minmax(this->m_fDestImportSizeX/this->m_fSourceImportSizeX,0.1f,10.f):1;
	float fSourceDestScaleY=(this->m_fSourceImportSizeY!=0)?minmax(this->m_fDestImportSizeY/this->m_fSourceImportSizeY,0.1f,10.f):1;

	// create source and dest rectangles
	pnpse::CRect rcDest(-this->m_fDestOffsetX,-this->m_fDestOffsetY,-this->m_fDestOffsetX+this->m_fDestSizeX,-this->m_fDestOffsetY+this->m_fDestSizeY);
	pnpse::CRect rcSource((-this->m_fSourceOffsetX)*fSourceDestScaleX,(-this->m_fSourceOffsetY)*fSourceDestScaleY,(-this->m_fSourceOffsetX+this->m_fSourceSizeX)*fSourceDestScaleX,(-this->m_fSourceOffsetY+this->m_fSourceSizeY)*fSourceDestScaleY);
	pnpse::CRect rcOverlap(0,0,this->m_fDestImportSizeX,this->m_fDestImportSizeY);
	pnpse::CVector2 vecMin(minimum(rcDest.GetMin().x,rcSource.GetMin().x),minimum(rcDest.GetMin().y,rcSource.GetMin().y));
	pnpse::CVector2 vecMax(maximum(rcDest.GetMax().x,rcSource.GetMax().x),minimum(rcDest.GetMax().y,rcSource.GetMax().y));

	// get overall size 
	float fSizeX=vecMax.x-vecMin.x;
	float fSizeY=vecMax.y-vecMin.y;
	float fScaleX=static_cast<float>(rc.Width()-BORDER_OFFSET2)/((fSizeX!=0)?fSizeX:1);
	float fScaleY=static_cast<float>(rc.Height()-BORDER_OFFSET2)/((fSizeY!=0)?fSizeY:1);
	float fScale=minimum(fScaleX,fScaleY);

	// bring areas into display range
	rcDest.MoveRel(-vecMin.x,-vecMin.y);
	rcSource.MoveRel(-vecMin.x,-vecMin.y);
	rcOverlap.MoveRel(-vecMin.x,-vecMin.y);
	rcDest.Scale(fScale);
	rcSource.Scale(fScale);
	rcOverlap.Scale(fScale);

	// clean background
	dc.FillSolidRect(rc,::GetSysColor(COLOR_3DFACE));
	int iBKRestore=dc.SetBkMode(TRANSPARENT);

	// set clipping region
	CRgn rgn;
	rgn.CreateRectRgn(0,0,rc.Width()-1,rc.Height()-1);
	dc.SelectClipRgn(&rgn);
	rgn.DeleteObject();

	// draw destination terrain
	CBrush brshDest;
	CPen penDest(PS_SOLID,1,RGB(0,128,0));
	brshDest.CreateHatchBrush(HS_BDIAGONAL,RGB(0,128,0));
	CBrush* pbrshRestore=dc.SelectObject(&brshDest);
	CPen* ppenRestore=dc.SelectObject(&penDest);
	CRect rcDestArea;
	rcDestArea.left=BORDER_OFFSET+static_cast<int>(rcDest.GetMin().x);
	rcDestArea.right=BORDER_OFFSET+static_cast<int>(rcDest.GetMax().x);
	rcDestArea.bottom=rc.Height()-BORDER_OFFSET-static_cast<int>(rcDest.GetMin().y);
	rcDestArea.top=rc.Height()-BORDER_OFFSET-static_cast<int>(rcDest.GetMax().y);
	dc.Rectangle(rcDestArea);

	// draw source terrain
	CBrush brshSource;
	CPen penSource(PS_SOLID,1,RGB(200,0,0));
	brshSource.CreateHatchBrush(HS_FDIAGONAL,RGB(200,0,0));
	dc.SelectObject(&brshSource);
	dc.SelectObject(&penSource);
	CRect rcSourceArea;
	rcSourceArea.left=BORDER_OFFSET+static_cast<int>(rcSource.GetMin().x);
	rcSourceArea.right=BORDER_OFFSET+static_cast<int>(rcSource.GetMax().x);
	rcSourceArea.bottom=rc.Height()-BORDER_OFFSET-static_cast<int>(rcSource.GetMin().y);
	rcSourceArea.top=rc.Height()-BORDER_OFFSET-static_cast<int>(rcSource.GetMax().y);
	dc.Rectangle(rcSourceArea);

	// draw overlapping area
	CBrush brshOverlap;
	CPen penOverlap(PS_SOLID,1,RGB(0,0,200));
	brshOverlap.CreateHatchBrush(HS_VERTICAL,RGB(0,0,200));
	dc.SelectObject(&brshOverlap);
	dc.SelectObject(&penOverlap);
	CRect rcOverlapArea;
	rcOverlapArea.left=BORDER_OFFSET+static_cast<int>(rcOverlap.GetMin().x);
	rcOverlapArea.right=BORDER_OFFSET+static_cast<int>(rcOverlap.GetMax().x);
	rcOverlapArea.bottom=rc.Height()-BORDER_OFFSET-static_cast<int>(rcOverlap.GetMin().y);
	rcOverlapArea.top=rc.Height()-BORDER_OFFSET-static_cast<int>(rcOverlap.GetMax().y);
	dc.Rectangle(rcOverlapArea);

	// restore gdi
	dc.SelectObject(pbrshRestore);
	dc.SelectObject(ppenRestore);
	dc.SetBkMode(iBKRestore);
	dc.SelectClipRgn(NULL);
	brshDest.DeleteObject();
	brshSource.DeleteObject();
	penDest.DeleteObject();
	penSource.DeleteObject();
	penOverlap.DeleteObject();
	brshOverlap.DeleteObject();
}