/////////////////////////////////////////////////////////////////////////////////////
// AreaDisplayCtrl.h
// MFC control to visualize overlapping regions during terrain import
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// creation:
//   02. Feb. 2007 (Ralf Westphal)
// ----------------------------------------------------------------------------------
// change history:
//
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#pragma once

#include <afxwin.h>

// CAreaDisplayCtrl

class CAreaDisplayCtrl : public CStatic
{
	DECLARE_DYNAMIC(CAreaDisplayCtrl)

public:
	CAreaDisplayCtrl();
	virtual ~CAreaDisplayCtrl();

	/// Set display parameters of the source area.
	/// @param fSourceSizeX Overall size of the source area in x direction.
	/// @param fSourceSizeY Overall size of the source area in y direction.
	/// @param fSourceOffsetX Offset in the source area to the position where the 
	/// import should start in x direction.
	/// @param fSourceOffsetY Offset in the source area to the position where the 
	/// import should start in y direction.
	/// @param fSourceImportSizeX The size of the imported source area in x direction.
	/// @param fSourceImportSizeX The size of the imported source area in y direction.
	void	SetDisplaySourceParameters(float fSourceSizeX,float fSourceSizeY,float fSourceOffsetX,float fSourceOffsetY,float fSourceImportSizeX,float fSourceImportSizeY);

	/// Set display parameters of the destination area.
	/// @param fDestSizeX Overall size of the destination area in x direction.
	/// @param fDestSizeY Overall size of the destination area in y direction.
	/// @param fDestOffsetX Offset in the destination area to the position where the
	/// import should start in x direction.
	/// @param fDestOffsetY Offset in the destination area to the position where the 
	/// import should start in y direction.
	/// @param fDestImportSizeX The size of the imported destination area in x direction.
	/// @param fDestImportSizeX The size of the imported destination area in y direction.
	void	SetDisplayDestParameters(float fDestSizeX,float fDestSizeY,float fDestOffsetX,float fDestOffsetY,float fDestImportSizeX,float fDestImportSizeY);


private:

	// ----------------------------------------------------------
	// member variables
	// ----------------------------------------------------------

	/// Overall size of the source area in x direction.
	float	m_fSourceSizeX;

	/// Overall size of the source area in y direction.
	float	m_fSourceSizeY;

	/// Offset in the source area to the position where the import should start
	/// in x direction.
	float	m_fSourceOffsetX;

	/// Offset in the source area to the position where the import should start
	/// in y direction.
	float	m_fSourceOffsetY;
	
	/// The size of the imported source area in x direction.
	float	m_fSourceImportSizeX;

	/// The size of the imported source area in y direction.
	float	m_fSourceImportSizeY;

	/// Overall size of the destination area in x direction.
	float	m_fDestSizeX;

	/// Overall size of the destination area in y direction.
	float	m_fDestSizeY;

	/// Offset in the destination area to the position where the import should start
	/// in x direction.
	float	m_fDestOffsetX;

	/// Offset in the destination area to the position where the import should start
	/// in y direction.
	float	m_fDestOffsetY;
	
	/// The size of the imported destination area in x direction.
	float	m_fDestImportSizeX;

	/// The size of the imported destination area in y direction.
	float	m_fDestImportSizeY;

protected:
	DECLARE_MESSAGE_MAP()
public:
	afx_msg void OnPaint();
};


