/////////////////////////////////////////////////////////////////////////////////////
// NumericEdit.h
// MFC edit/text control supporting numeric input format.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  04. Feb. 2007 : Added NotifyParent
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#pragma once

#define _CRT_SECURE_NO_DEPRECATE 1

// includes
#include <afxwin.h>

// new data available
#define EN_NEWDATA	(WM_USER+500)


// macros for message handling
#define	ON_EN_NEWDATA(id, memberFxn)	\
	ON_CONTROL(EN_NEWDATA,id, memberFxn)




// CNumericEdit
class CNumericEdit : public CEdit 
{
	friend class CEditTimerThread;

	DECLARE_DYNAMIC(CNumericEdit)

public:

	/// construction
	CNumericEdit();

	/// destruction
	virtual ~CNumericEdit();

	/// get text
	float	GetValue();

	/// set text
	void	SetValue(float fValue);

	/// allow decimals
	void	SetAllowDecimals(bool bAllowDecimals);

	/// allow negative entries
	void	SetAllowNegatives(bool bAllowNegatives);

	/// set range
	void	SetRange(float fMin,float fMax);

	/// Notifies the parent of a changed value
	void	NotifyParent();


protected:

	// -----------------------------------------------------------------------
	// member variables
	// -----------------------------------------------------------------------

	/// decimal delimiter
	CString				m_strDelim;

	/// data changed?
	bool				m_bChanged;

	/// time of last key stroke
	DWORD				m_dwLastKey;

	/// allow decimals
	bool				m_bAllowDecimals;

	/// allow negative values
	bool				m_bAllowNegatives;

	/// check input value range
	bool				m_bCheckInputValueRange;

	/// minimum value
	float				m_fMin;

	/// maximum value
	float				m_fMax;

	/// ID of change check timer
	UINT				m_nTimerID;


	// -----------------------------------------------------------------------
	// internal methods
	// -----------------------------------------------------------------------

	/// checks the input value range
	void				CheckInputValueRange();

	/// stops the change check timer
	void				StopTimer();


public:
	DECLARE_MESSAGE_MAP()
	afx_msg void OnEnKillfocus();
	afx_msg void OnEnSetfocus();
	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnDestroy();
	afx_msg void OnTimer(UINT nIDEvent);
};

