/////////////////////////////////////////////////////////////////////////////////////
// pnpseBoundingBox.h
// Definition of a 3 dimensional bounding box class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  11. Aug. 2008 : Added AddPoint methods.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPSE_BOUNDINGBOX_H_INCLUDED)
#define PNPSE_BOUNDINGBOX_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


// base includes
#include "pnpseBox.h"

// pnp sensory game engine
namespace pnpse	
{


/// Bounding box class.
class CBoundingBox : public CBox
{
public:

	// -----------------------------------------------------------------
	// construction of a bounding box
	// -----------------------------------------------------------------

	/// Constructs a bounding box without initialization.
	/// all elements are initialized with 0 by default.
	inline CBoundingBox() {};

	/// Construction of a bounding box from two points.
	/// @param vecMin One border point of the bounding box to be created.
	/// @param vecMin Second border point of the bounding box to be created.
	inline CBoundingBox(const CVector3& vecMin,const CVector3& vecMax)
		: CBox(vecMin,vecMax) {} ;

	/// Construction of a bounding box from two coordinate pairs.
	/// @param fX1 x coordinate of one border point of the bounding box to be created.
	/// @param fY1 y coordinate of one border point of the bounding box to be created.
	/// @param fZ1 z coordinate of one border point of the bounding box to be created.
	/// @param fX2 x coordinate of the second border point of bounding the box to be created.
	/// @param fY2 y coordinate of the second border point of bounding the box to be created.
	/// @param fZ2 z coordinate of the second border point of bounding the box to be created.
	inline CBoundingBox(const float fX1,const float fY1,const float fZ1,const float fX2,const float fY2,const float fZ2)
		: CBox(fX1,fY1,fZ1,fX2,fY2,fZ2) {};

	// -----------------------------------------------------------------
	// bounding box manipulation
	// -----------------------------------------------------------------

	/// Adds a point to the bounding box.
	/// If required, the bounding box is resized in order to contain this point.
	/// @param fX1 x coordinate of the point to be added to the bounding box.
	/// @param fY1 y coordinate of the point to be added to the bounding box.
	/// @param fZ1 z coordinate of the point to be added to the bounding box.
	inline void AddPoint(const float fX,const float fY,const float fZ)
	{
		this->m_vecMin.x=minimum(this->m_vecMin.x,fX);
		this->m_vecMin.y=minimum(this->m_vecMin.y,fY);
		this->m_vecMin.z=minimum(this->m_vecMin.z,fZ);
		this->m_vecMax.x=maximum(this->m_vecMax.x,fX);
		this->m_vecMax.y=maximum(this->m_vecMax.y,fY);
		this->m_vecMax.z=maximum(this->m_vecMax.z,fZ);
	}

	/// Adds a point to the bounding box.
	/// If required, the bounding box is resized in order to contain this point.
	/// @param vecAdd The point to be added to the bounding box.
	inline void AddPoint(const CVector3& vecAdd)
	{
		this->m_vecMin.x=minimum(this->m_vecMin.x,vecAdd.x);
		this->m_vecMin.y=minimum(this->m_vecMin.y,vecAdd.y);
		this->m_vecMin.z=minimum(this->m_vecMin.z,vecAdd.z);
		this->m_vecMax.x=maximum(this->m_vecMax.x,vecAdd.x);
		this->m_vecMax.y=maximum(this->m_vecMax.y,vecAdd.y);
		this->m_vecMax.z=maximum(this->m_vecMax.z,vecAdd.z);
	}

};	// class CBoundingBox

};	// namespace pnpse

#endif // !defined(PNPSE_BOUNDINGBOX_H_INCLUDED)