/////////////////////////////////////////////////////////////////////////////////////
// pnpseBox.cpp
// Implementation of a 3 dimensional box class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "pnpseBox.h"
#include "pnpseRay.h"

// pnp sensory game engine
namespace pnpse
{


bool CBox::Hit(const CRay& rayTest) const
{
	// the point hit
    CVector3 vecHit;
	float fRayDist;

	// get ray parameters
    const CVector3& vecRayStart = rayTest.GetStartPoint();
    const CVector3& vecRayDir = rayTest.GetDirection();

	// Check origin inside first
	if ( vecRayStart > this->m_vecMin && vecRayStart < this->m_vecMax )
	{
		return true;
	}

    // Check each face in turn, only check closest 3
    // Min x
    if (vecRayStart.x < this->m_vecMin.x && vecRayDir.x > 0)
    {
        fRayDist = (this->m_vecMin.x - vecRayStart.x) / vecRayDir.x;
        if (fRayDist > 0)
        {
            // Substitute t back into ray and check bounds and dist
            vecHit = vecRayStart + vecRayDir * fRayDist;
            if (vecHit.y >= this->m_vecMin.y && vecHit.y <= this->m_vecMax.y &&
                vecHit.z >= this->m_vecMin.z && vecHit.z <= this->m_vecMax.z )
            {
				return true;
            }
        }
    }
    // Max x
    if (vecRayStart.x > this->m_vecMax.x && vecRayDir.x < 0)
    {
        fRayDist = (this->m_vecMax.x - vecRayStart.x) / vecRayDir.x;
        if (fRayDist > 0)
        {
            // Substitute t back into ray and check bounds and dist
            vecHit = vecRayStart + vecRayDir * fRayDist;
            if (vecHit.y >= this->m_vecMin.y && vecHit.y <= this->m_vecMax.y &&
                vecHit.z >= this->m_vecMin.z && vecHit.z <= this->m_vecMax.z )
            {
				return true;
            }
        }
    }
    // Min y
    if (vecRayStart.y < this->m_vecMin.y && vecRayDir.y > 0)
    {
        fRayDist = (this->m_vecMin.y - vecRayStart.y) / vecRayDir.y;
        if (fRayDist > 0)
        {
            // Substitute t back into ray and check bounds and dist
            vecHit = vecRayStart + vecRayDir * fRayDist;
            if (vecHit.x >= this->m_vecMin.x && vecHit.x <= this->m_vecMax.x &&
                vecHit.z >= this->m_vecMin.z && vecHit.z <= this->m_vecMax.z )
            {
				return true;
            }
        }
    }
    // Max y
    if (vecRayStart.y > this->m_vecMax.y && vecRayDir.y < 0)
    {
        fRayDist = (this->m_vecMax.y - vecRayStart.y) / vecRayDir.y;
        if (fRayDist > 0)
        {
            // Substitute t back into ray and check bounds and dist
            vecHit = vecRayStart + vecRayDir * fRayDist;
            if (vecHit.x >= this->m_vecMin.x && vecHit.x <= this->m_vecMax.x &&
                vecHit.z >= this->m_vecMin.z && vecHit.z <= this->m_vecMax.z )
            {
				return true;
            }
        }
    }
    // Min z
    if (vecRayStart.z < this->m_vecMin.z && vecRayDir.z > 0)
    {
        fRayDist = (this->m_vecMin.z - vecRayStart.z) / vecRayDir.z;
        if (fRayDist > 0)
        {
            // Substitute t back into ray and check bounds and dist
            vecHit = vecRayStart + vecRayDir * fRayDist;
            if (vecHit.x >= this->m_vecMin.x && vecHit.x <= this->m_vecMax.x &&
                vecHit.y >= this->m_vecMin.y && vecHit.y <= this->m_vecMax.y )
            {
				return true;
            }
        }
    }
    // Max z
    if (vecRayStart.z > this->m_vecMax.z && vecRayDir.z < 0)
    {
        fRayDist = (this->m_vecMax.z - vecRayStart.z) / vecRayDir.z;
        if (fRayDist > 0)
        {
            // Substitute t back into ray and check bounds and dist
            vecHit = vecRayStart + vecRayDir * fRayDist;
            if (vecHit.x >= this->m_vecMin.x && vecHit.x <= this->m_vecMax.x &&
                vecHit.y >= this->m_vecMin.y && vecHit.y <= this->m_vecMax.y )
            {
				return true;
            }
        }
    }

	// not hit
	return false;
}



};	// namespace pnpse