/////////////////////////////////////////////////////////////////////////////////////
// pnpseRay.h
// Definition of a 3 dimensional ray class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPSE_RAY_H_INCLUDED)
#define PNPSE_RAY_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


/// some base includes
#include "pnpseVector3.h"


// pnp sensory game engine
namespace pnpse	
{



/// 3D ray class
class CRay
{
public:

	// ----------------------------------------------------------------------
	// Contruction / Destruction
	// ----------------------------------------------------------------------

	/// Default ray construction.
	inline CRay() {};


	// ----------------------------------------------------------------------
	// Ray initialization
	// ----------------------------------------------------------------------

	/// Creates a ray through two points.
	/// @param vecStart Starting point of the ray.
	/// @param vecEnd A second point on that ray.
	inline void MakeRayFrom2Points(const CVector3& vecStart,const CVector3& vecEnd)
	{
		this->m_vecPoint=vecStart;
		this->m_vecDirection=CVector3(vecEnd-vecStart);
		this->m_vecDirection.Normalize();
	};

	/// Creates a ray from a point and a direction vector.
	/// @param vecStart Starting point of the ray.
	/// @param vecDirection Direction vector of that ray.
	void MakeRayFromPointDirection(const CVector3& vecStart,const CVector3& vecDirection)
	{
		this->m_vecPoint=vecStart;
		this->m_vecDirection=vecDirection;
		this->m_vecDirection.Normalize();
	};


	// ----------------------------------------------------------------------
	// Return ray parameters
	// ----------------------------------------------------------------------

	/// Get starting point of ray.
	/// @return Returns the ray's starting point.
	inline const CVector3&	GetStartPoint() const {return this->m_vecPoint;};

	/// Get direction vector.
	/// @return Returns the ray's direction vector.
	inline const CVector3&	GetDirection() const {return this->m_vecDirection;};


private:

	// --------------------------------------------------------------------
	// member variables
	// --------------------------------------------------------------------

	/// starting point of the ray
	CVector3	m_vecPoint;	

	/// direction into which the ray is pointing
	CVector3	m_vecDirection;

};	// CRay


};	// namespace pnpse


#endif // !defined(PNPSE_RAY_H_INCLUDED)