/////////////////////////////////////////////////////////////////////////////////////
// pnpseTimer.h
// Implementation of a timer/counter class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "pnpseTimer.h"


// pnp sensory game engine
namespace pnpse
{


CTimer::CTimer()
{
	// get frequency of performance counter
	LARGE_INTEGER qwFrequency;
	if (QueryPerformanceFrequency(&qwFrequency))
	{
		// PerformanceCounter counter exists
		this->m_iTimerType=2;
		// TicksPerSecound
		this->m_llQPFTicksPerSec=qwFrequency.QuadPart;
	}
	else
	{
		// Normal Windows Timer
		this->m_iTimerType=1;
		// set ticks per second to ms
		this->m_llQPFTicksPerSec=1000;
	}
}

CTimer::~CTimer()
{
}

CTimer& CTimer::GetTimer()
{
	// return a static/global timer object
	static CTimer sTimer;
	return sTimer;
}

LONGLONG CTimer::GetTicksPerSecond() const
{
	// return ticks per second
	return this->m_llQPFTicksPerSec;
}

void CTimer::GetTimeStamp(LONGLONG* pllTimeStamp) const
{
	// compute time
	if (this->m_iTimerType==2)
	{
		// Performance-Counter
		static LARGE_INTEGER qwTime;
		QueryPerformanceCounter( &qwTime ); 
		*pllTimeStamp=qwTime.QuadPart;
	}
	else if (this->m_iTimerType==3)
	{
		// Tick-Counter
		*pllTimeStamp=GetTickCount();
	}		
}

float CTimer::GetElapsedTime(const LONGLONG& llTimeStamp) const
{
	// get actual time
	static LONGLONG llTickCount;
	GetTimeStamp(&llTickCount);

	// compute elapsed time
	return static_cast<float>(llTickCount-llTimeStamp)/static_cast<float>(this->m_llQPFTicksPerSec);
}

float CTimer::GetTimeDistance(const LONGLONG& llTimeStamp1,const LONGLONG& llTimeStamp2) const
{
	// compute time distance
	return static_cast<float>(llTimeStamp2-llTimeStamp1)/static_cast<float>(this->m_llQPFTicksPerSec);
}


};	// namespace pnpse