/////////////////////////////////////////////////////////////////////////////////////
// pnpseTimer.h
// Definition of a timer/counter class.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the "PnP Sensory Game Engine".
//   It is included in a simplified form with the PnP TerrainCreator product 
//   and may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPSE_TIMER_H_INCLUDED)
#define PNPSE_TIMER_H_INCLUDED

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <windows.h>


// pnp sensory game engine
namespace pnpse	
{


/// timer/counter class.
class CTimer 
{
public:

	// ---------------------------------------------------
	// construction / destruction
	// ---------------------------------------------------

	/// Default constructor.
	CTimer();

	/// Destruction.
	virtual ~CTimer();


	// ---------------------------------------------------
	// global timer
	// ---------------------------------------------------

	/// Returns the global timer object
	static CTimer&	GetTimer();


	// ---------------------------------------------------
	// timer parameters
	// ---------------------------------------------------

	/// Returns the number of possible ticks per second.
	/// @return Ticks per second.
	LONGLONG		GetTicksPerSecond() const;


	// ---------------------------------------------------
	// time stamps
	// ---------------------------------------------------

	/// Returns the actual time stamp.
	/// @param pllTimeStamp Returned timestamp
	void			GetTimeStamp(LONGLONG* pllTimeStamp) const;

	/// Returns the time elapsed since a given
	/// time stamp in seconds.
	/// @param llTimeStamp Last time stamp.
	/// @return Time elapsed since time stamp in seconds.
	float			GetElapsedTime(const LONGLONG& llTimeStamp) const;

	/// Returns the time distance between two given
	/// time stamps in seconds.
	/// @param llTimeStamp1 First (older) time stamp.
	/// @param llTimeStamp2 Second (newer) time stamp.
	/// @return Time distance between time stamps in seconds.
	float			GetTimeDistance(const LONGLONG& llTimeStamp1,const LONGLONG& llTimeStamp2) const;


private:

	// ---------------------------------------------------
	// member variables
	// ---------------------------------------------------

	/// Type of timer used.<br>
	/// 1: Windows Timer
	/// 2: PerformanceCounter
	int				m_iTimerType;	

	/// Ticks per second
	LONGLONG		m_llQPFTicksPerSec;

};	// CTimer


};	// namespace pnpse


#endif // !defined(PNPSE_TIMER_H_INCLUDED)
