/////////////////////////////////////////////////////////////////////////////////////
// XmlUtils.h
// various utility methods for working with xml elements.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  16. Jan  2009 :	(Ralf Westphal) Created.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#include "xmlutils.h"

#include "windows.h"	// The Windows PlatformSDK has to be installed in order to compile PlugIns
#include <direct.h> 
#include <algorithm>

#include "utils.h"


std::string GetXmlAttributeString(TiXmlElement* pNode,const std::string& strAttribute,const std::string& strDefault/*=""*/)
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		return std::string(*pNode->Attribute(strAttribute));
	}
	else
	{
		// return default value
		return strDefault;
	}
}

int GetXmlAttributeInt(TiXmlElement* pNode,const std::string& strAttribute,int iDefault/*=0*/)
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		return atoi(pNode->Attribute(strAttribute)->c_str());
	}
	else
	{
		// return default value
		return iDefault;
	}
}

float GetXmlAttributeFloat(TiXmlElement* pNode,const std::string& strAttribute,float fDefault/*=0*/)
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		return static_cast<float>(atof(pNode->Attribute(strAttribute)->c_str()));
	}
	else
	{
		// return default value
		return fDefault;
	}
}

bool GetXmlAttributeBool(TiXmlElement* pNode,const std::string& strAttribute,bool bDefault/*=false*/)
{
	// test attribute existance
	if (pNode && pNode->Attribute(strAttribute))
	{
		// return attribute value
		std::string strValue=pNode->Attribute(strAttribute)->c_str();
		return ::EqualNoCase(strValue,"true") || ::EqualNoCase(strValue,"yes") || ::EqualNoCase(strValue,"1");
	}
	else
	{
		// return default value
		return bDefault;
	}
}

