/////////////////////////////////////////////////////////////////////////////////////
// XmlUtils.h
// various utility methods for working with xml elements.
// ----------------------------------------------------------------------------------
// author:		Ralf Westphal
// ----------------------------------------------------------------------------------
// change history:
//  16. Jan  2009 :	(Ralf Westphal) Created.
// ----------------------------------------------------------------------------------
// copyright notice:
//   Copyright (C) 2004-2009 Westphal Data Solution (Ralf Westphal), Germany
//   All Rights Reserved.
//   homepage: www.pnp-terraincreator.com
//   email:    rwe@pnp-terraincreator.com
//
//   This file is part of the PnP TerrainCreator product and
//   may only be used to create PlugIns for PnP TerrainCreator.
/////////////////////////////////////////////////////////////////////////////////////

#if !defined(PNPTC_XMLUTILS_H_INCLUDED)
#define PNPTC_XMLUTILS_H_INCLUDED

#include <string>
#include "../3rd-party/tinyxml/tinyxml.h"

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/// Returns a string XML attribute.
/// @param pNode The XML element from which to read an attribute.
/// @param strAttribute The attribute name.
/// @param strDefault The default value to be returned if
/// the attribute is missing in the XML element.
/// @return Returns the attribute value.
std::string GetXmlAttributeString(TiXmlElement* pNode,const std::string& strAttribute,const std::string& strDefault="");

/// Returns an integer XML attribute.
/// @param pNode The XML element from which to read an attribute.
/// @param strAttribute The attribute name.
/// @param iDefault The default value to be returned if
/// the attribute is missing in the XML element.
/// @return Returns the attribute value.
int GetXmlAttributeInt(TiXmlElement* pNode,const std::string& strAttribute,int iDefault=0);

/// Returns a boolean XML attribute. The attribute values true/false,
/// yes/no, and 1/0 can be used and cast to boolean.
/// @param pNode The XML element from which to read an attribute.
/// @param strAttribute The attribute name.
/// @param bDefault The default value to be returned if
/// the attribute is missing in the XML element.
/// @return Returns the attribute value.
bool GetXmlAttributeBool(TiXmlElement* pNode,const std::string& strAttribute,bool bDefault=false);

/// Returns a real value XML attribute.
/// @param pNode The XML element from which to read an attribute.
/// @param strAttribute The attribute name.
/// @param fDefault The default value to be returned if
/// the attribute is missing in the XML element.
/// @return Returns the attribute value.
float GetXmlAttributeFloat(TiXmlElement* pNode,const std::string& strAttribute,float fDefault=0);


#endif // !defined(PNPTC_XMLUTILS_H_INCLUDED)